@@ -142,6 +142,7 @@ class CategoryTreeHooks {
 	 * @param string $cat
 	 * @param array $argv
 	 * @param Parser|null $parser
+	 * @param PPFrame|null $frame
 	 * @param bool $allowMissing
 	 * @return bool|string
 	 */
@@ -149,6 +150,7 @@ class CategoryTreeHooks {
 		$cat,
 		array $argv,
 		Parser $parser = null,
+		PPFrame $frame = null,
 		$allowMissing = false
 	) {
 		if ( $parser ) {
@@ -237,7 +239,7 @@ class CategoryTreeHooks {
 		}
 
 		foreach ( $categories as $category => $type ) {
-			$links[$type][] = self::parserHook( $category, $wgCategoryTreePageCategoryOptions, null, true );
+			$links[$type][] = self::parserHook( $category, $wgCategoryTreePageCategoryOptions, null, null, true );
 			CategoryTree::setHeaders( $out );
 		}
 
