@@ -2,6 +2,8 @@
 
 use MediaWiki\Linker\LinkTarget;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Revision\RevisionLookup;
+use MediaWiki\Revision\SlotRecord;
 use Wikimedia\Rdbms\Database;
 
 /**
@@ -14,15 +16,22 @@ class GadgetDefinitionNamespaceRepo extends GadgetRepo {
 	 * How long in seconds the list of gadget ids and
 	 * individual gadgets should be cached for (1 day)
 	 */
-	const CACHE_TTL = 86400;
+	private const CACHE_TTL = 86400;
 
 	/**
 	 * @var WANObjectCache
 	 */
 	private $wanCache;
 
+	/**
+	 * @var RevisionLookup
+	 */
+	private $revLookup;
+
 	public function __construct() {
-		$this->wanCache = MediaWikiServices::getInstance()->getMainWANObjectCache();
+		$services = MediaWikiServices::getInstance();
+		$this->wanCache = $services->getMainWANObjectCache();
+		$this->revLookup = $services->getRevisionLookup();
 	}
 
 	/**
@@ -33,10 +42,11 @@ class GadgetDefinitionNamespaceRepo extends GadgetRepo {
 	public function getGadgetIds() {
 		$key = $this->getGadgetIdsKey();
 
+		$fname = __METHOD__;
 		return $this->wanCache->getWithSetCallback(
 			$key,
 			self::CACHE_TTL,
-			function ( $oldValue, &$ttl, array &$setOpts ) {
+			function ( $oldValue, &$ttl, array &$setOpts ) use ( $fname ) {
 				$dbr = wfGetDB( DB_REPLICA );
 				$setOpts += Database::getCacheSetOptions( $dbr );
 
@@ -44,7 +54,7 @@ class GadgetDefinitionNamespaceRepo extends GadgetRepo {
 					'page',
 					'page_title',
 					[ 'page_namespace' => NS_GADGET_DEFINITION ],
-					__METHOD__
+					$fname
 				);
 			},
 			[
@@ -100,9 +110,6 @@ class GadgetDefinitionNamespaceRepo extends GadgetRepo {
 		$gadget = $this->wanCache->getWithSetCallback(
 			$key,
 			self::CACHE_TTL,
-			/**
-			 * @suppress PhanTypeMismatchArgument
-			 */
 			function ( $old, &$ttl, array &$setOpts ) use ( $id ) {
 				$setOpts += Database::getCacheSetOptions( wfGetDB( DB_REPLICA ) );
 				$title = Title::makeTitleSafe( NS_GADGET_DEFINITION, $id );
@@ -111,13 +118,13 @@ class GadgetDefinitionNamespaceRepo extends GadgetRepo {
 					return null;
 				}
 
-				$rev = Revision::newFromTitle( $title );
-				if ( !$rev ) {
+				$revRecord = $this->revLookup->getRevisionByTitle( $title );
+				if ( !$revRecord ) {
 					$ttl = WANObjectCache::TTL_UNCACHEABLE;
 					return null;
 				}
 
-				$content = $rev->getContent();
+				$content = $revRecord->getContent( SlotRecord::MAIN );
 				if ( !$content instanceof GadgetDefinitionContent ) {
 					// Uhm...
 					$ttl = WANObjectCache::TTL_UNCACHEABLE;
