@@ -15,6 +15,6 @@
  * along with MultimediaViewer.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-( function ( mw ) {
+( function () {
 	mw.mmv.model = {};
-}( mediaWiki ) );
+}() );
