@@ -1,7 +1,7 @@
-( function ( mw, $ ) {
+( function () {
 	QUnit.module( 'mmv', QUnit.newMwEnvironment() );
 
-	QUnit.test( 'eachPrealoadableLightboxIndex()', function ( assert ) {
+	QUnit.test( 'eachPreloadableLightboxIndex()', function ( assert ) {
 		var viewer = mw.mmv.testHelpers.getMultimediaViewer(),
 			expectedIndices,
 			i;
@@ -17,14 +17,14 @@
 		viewer.currentIndex = 2;
 		i = 0;
 		expectedIndices = [ 2, 3, 1, 4, 0, 5 ];
-		viewer.eachPrealoadableLightboxIndex( function ( index ) {
+		viewer.eachPreloadableLightboxIndex( function ( index ) {
 			assert.strictEqual( index, expectedIndices[ i++ ], 'preload on left edge' );
 		} );
 
 		viewer.currentIndex = 9;
 		i = 0;
 		expectedIndices = [ 9, 10, 8, 7, 6 ];
-		viewer.eachPrealoadableLightboxIndex( function ( index ) {
+		viewer.eachPreloadableLightboxIndex( function ( index ) {
 			assert.strictEqual( index, expectedIndices[ i++ ], 'preload on right edge' );
 		} );
 	} );
@@ -39,7 +39,7 @@
 		// animation would keep running, conflict with other tests
 		this.sandbox.stub( $.fn, 'animate' ).returnsThis();
 
-		window.location.hash = '';
+		location.hash = '';
 
 		viewer.setupEventHandlers();
 		oldUnattach = ui.unattach;
@@ -52,17 +52,16 @@
 		viewer.ui = ui;
 		viewer.close();
 
-		assert.ok( !viewer.isOpen, 'Viewer is closed' );
+		assert.strictEqual( viewer.isOpen, false, 'Viewer is closed' );
 
-		viewer.isOpen = true;
+		viewer.loadImageByTitle( image.filePageTitle );
 
 		// Verify that passing an invalid mmv hash when the mmv is open triggers unattach()
-		window.location.hash = 'Foo';
-		viewer.hash();
+		location.hash = 'Foo';
 
 		// Verify that mmv doesn't reset a foreign hash
-		assert.strictEqual( window.location.hash, '#Foo', 'Foreign hash remains intact' );
-		assert.ok( !viewer.isOpen, 'Viewer is closed' );
+		assert.strictEqual( location.hash, '#Foo', 'Foreign hash remains intact' );
+		assert.strictEqual( viewer.isOpen, false, 'Viewer is closed' );
 
 		ui.unattach = function () {
 			assert.ok( false, 'Lightbox was not unattached' );
@@ -70,13 +69,12 @@
 		};
 
 		// Verify that passing an invalid mmv hash when the mmv is closed doesn't trigger unattach()
-		window.location.hash = 'Bar';
-		viewer.hash();
+		location.hash = 'Bar';
 
 		// Verify that mmv doesn't reset a foreign hash
-		assert.strictEqual( window.location.hash, '#Bar', 'Foreign hash remains intact' );
+		assert.strictEqual( location.hash, '#Bar', 'Foreign hash remains intact' );
 
-		viewer.ui = { images: [ image ], disconnect: $.noop };
+		viewer.ui = { images: [ image ], disconnect: function () {} };
 
 		$( '#qunit-fixture' ).append( '<a class="image"><img src="' + imageSrc + '"></a>' );
 
@@ -86,29 +84,24 @@
 
 		// Open a valid mmv hash link and check that the right image is requested.
 		// imageSrc contains a space without any encoding on purpose
-		window.location.hash = '/media/File:' + imageSrc;
-		viewer.hash();
+		location.hash = '/media/File:' + imageSrc;
 
 		// Reset the hash, because for some browsers switching from the non-URI-encoded to
 		// the non-URI-encoded version of the same text with a space will not trigger a hash change
-		window.location.hash = '';
-		viewer.hash();
+		location.hash = '';
 
 		// Try again with an URI-encoded imageSrc containing a space
-		window.location.hash = '/media/File:' + encodeURIComponent( imageSrc );
-		viewer.hash();
+		location.hash = '/media/File:' + encodeURIComponent( imageSrc );
 
 		// Reset the hash
-		window.location.hash = '';
-		viewer.hash();
+		location.hash = '';
 
 		// Try again with a legacy hash
-		window.location.hash = 'mediaviewer/File:' + imageSrc;
-		viewer.hash();
+		location.hash = 'mediaviewer/File:' + imageSrc;
 
 		viewer.cleanupEventHandlers();
 
-		window.location.hash = '';
+		location.hash = '';
 	} );
 
 	QUnit.test( 'Progress', function ( assert ) {
@@ -122,31 +115,31 @@
 			clock = this.sandbox.useFakeTimers();
 
 		viewer.thumbs = [];
-		viewer.displayPlaceholderThumbnail = $.noop;
-		viewer.setImage = $.noop;
-		viewer.scroll = $.noop;
-		viewer.preloadFullscreenThumbnail = $.noop;
+		viewer.displayPlaceholderThumbnail = function () {};
+		viewer.setImage = function () {};
+		viewer.scroll = function () {};
+		viewer.preloadFullscreenThumbnail = function () {};
 		viewer.fetchSizeIndependentLightboxInfo = function () { return $.Deferred().resolve( {} ); };
 		viewer.ui = {
-			setFileReuseData: $.noop,
-			setupForLoad: $.noop,
-			canvas: { set: $.noop,
-				unblurWithAnimation: $.noop,
-				unblur: $.noop,
+			setFileReuseData: function () {},
+			setupForLoad: function () {},
+			canvas: { set: function () {},
+				unblurWithAnimation: function () {},
+				unblur: function () {},
 				getCurrentImageWidths: function () { return { real: 0 }; },
 				getDimensions: function () { return {}; }
 			},
 			panel: {
-				setImageInfo: $.noop,
+				setImageInfo: function () {},
 				scroller: {
-					animateMetadataOnce: $.noop
+					animateMetadataOnce: function () {}
 				},
 				progressBar: {
 					animateTo: this.sandbox.stub(),
 					jumpTo: this.sandbox.stub()
 				}
 			},
-			open: $.noop };
+			open: function () {} };
 
 		viewer.imageProvider.get = function () { return imageDeferred.promise(); };
 		viewer.imageInfoProvider.get = function () { return $.Deferred().resolve( {} ); };
@@ -156,19 +149,19 @@
 		// progress handlers
 		viewer.loadImage( fakeImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.jumpTo.lastCall.calledWith( 0 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.jumpTo.lastCall.calledWith( 0 ), true,
 			'Percentage correctly reset by loadImage' );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.firstCall.calledWith( 5 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.firstCall.calledWith( 5 ), true,
 			'Percentage correctly animated to 5 by loadImage' );
 
 		imageDeferred.notify( 'response', 45 );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.secondCall.calledWith( 45 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.secondCall.calledWith( 45 ), true,
 			'Percentage correctly funneled to panel UI' );
 
 		imageDeferred.resolve( {}, {} );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.thirdCall.calledWith( 100 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.thirdCall.calledWith( 100 ), true,
 			'Percentage correctly funneled to panel UI' );
 
 		clock.restore();
@@ -197,26 +190,26 @@
 		this.sandbox.stub( $.fn, 'animate' ).returnsThis();
 
 		viewer.thumbs = [];
-		viewer.displayPlaceholderThumbnail = $.noop;
-		viewer.setImage = $.noop;
-		viewer.scroll = $.noop;
-		viewer.preloadFullscreenThumbnail = $.noop;
-		viewer.preloadImagesMetadata = $.noop;
-		viewer.preloadThumbnails = $.noop;
+		viewer.displayPlaceholderThumbnail = function () {};
+		viewer.setImage = function () {};
+		viewer.scroll = function () {};
+		viewer.preloadFullscreenThumbnail = function () {};
+		viewer.preloadImagesMetadata = function () {};
+		viewer.preloadThumbnails = function () {};
 		viewer.fetchSizeIndependentLightboxInfo = function () { return $.Deferred().resolve( {} ); };
 		viewer.ui = {
-			setFileReuseData: $.noop,
-			setupForLoad: $.noop,
-			canvas: { set: $.noop,
-				unblurWithAnimation: $.noop,
-				unblur: $.noop,
+			setFileReuseData: function () {},
+			setupForLoad: function () {},
+			canvas: { set: function () {},
+				unblurWithAnimation: function () {},
+				unblur: function () {},
 				getCurrentImageWidths: function () { return { real: 0 }; },
 				getDimensions: function () { return {}; }
 			},
 			panel: {
-				setImageInfo: $.noop,
+				setImageInfo: function () {},
 				scroller: {
-					animateMetadataOnce: $.noop
+					animateMetadataOnce: function () {}
 				},
 				progressBar: {
 					hide: this.sandbox.stub(),
@@ -224,8 +217,8 @@
 					jumpTo: this.sandbox.stub()
 				}
 			},
-			open: $.noop,
-			empty: $.noop };
+			open: function () {},
+			empty: function () {} };
 
 		viewer.imageInfoProvider.get = function () { return $.Deferred().resolve( {} ); };
 		viewer.thumbnailInfoProvider.get = function () { return $.Deferred().resolve( {} ); };
@@ -234,61 +227,61 @@
 		viewer.imageProvider.get = this.sandbox.stub().returns( firstImageDeferred );
 		viewer.loadImage( firstImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.jumpTo.getCall( 0 ).calledWith( 0 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.jumpTo.getCall( 0 ).calledWith( 0 ), true,
 			'Percentage correctly reset for new first image' );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.getCall( 0 ).calledWith( 5 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.getCall( 0 ).calledWith( 5 ), true,
 			'Percentage correctly animated to 5 for first new image' );
 
 		// progress on active image
 		firstImageDeferred.notify( 'response', 20 );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.getCall( 1 ).calledWith( 20 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.getCall( 1 ).calledWith( 20 ), true,
 			'Percentage correctly animated when active image is loading' );
 
 		// change to another image
 		viewer.imageProvider.get = this.sandbox.stub().returns( secondImageDeferred );
 		viewer.loadImage( secondImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.jumpTo.getCall( 1 ).calledWith( 0 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.jumpTo.getCall( 1 ).calledWith( 0 ), true,
 			'Percentage correctly reset for second new image' );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.getCall( 2 ).calledWith( 5 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.getCall( 2 ).calledWith( 5 ), true,
 			'Percentage correctly animated to 5 for second new image' );
 
 		// progress on active image
 		secondImageDeferred.notify( 'response', 30 );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.getCall( 3 ).calledWith( 30 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.getCall( 3 ).calledWith( 30 ), true,
 			'Percentage correctly animated when active image is loading' );
 
 		// progress on inactive image
 		firstImageDeferred.notify( 'response', 40 );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.callCount === 4,
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.callCount, 4,
 			'Percentage not animated when inactive image is loading' );
 
 		// progress on active image
 		secondImageDeferred.notify( 'response', 50 );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.animateTo.getCall( 4 ).calledWith( 50 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.getCall( 4 ).calledWith( 50 ), true,
 			'Percentage correctly ignored inactive image & only animated when active image is loading' );
 
 		// change back to first image
 		viewer.imageProvider.get = this.sandbox.stub().returns( firstImageDeferred );
 		viewer.loadImage( firstImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.jumpTo.getCall( 2 ).calledWith( 40 ),
+		assert.strictEqual( viewer.ui.panel.progressBar.jumpTo.getCall( 2 ).calledWith( 40 ), true,
 			'Percentage jumps to right value when changing images' );
 
 		secondImageDeferred.resolve( {}, {} );
 		clock.tick( 10 );
-		assert.ok( !viewer.ui.panel.progressBar.hide.called,
+		assert.strictEqual( viewer.ui.panel.progressBar.hide.called, false,
 			'Progress bar not hidden when something finishes in the background' );
 
 		// change back to second image, which has finished loading
 		viewer.imageProvider.get = this.sandbox.stub().returns( secondImageDeferred );
 		viewer.loadImage( secondImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( viewer.ui.panel.progressBar.hide.called,
+		assert.strictEqual( viewer.ui.panel.progressBar.hide.called, true,
 			'Progress bar hidden when switching to finished image' );
 
 		clock.restore();
@@ -302,102 +295,102 @@
 		// animation would keep running, conflict with other tests
 		this.sandbox.stub( $.fn, 'animate' ).returnsThis();
 
-		assert.ok( !viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( !viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, false, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, false, 'Placeholder state is correct' );
 
 		viewer.realThumbnailShown = true;
 		viewer.blurredThumbnailShown = true;
 
 		viewer.resetBlurredThumbnailStates();
 
-		assert.ok( !viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( !viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, false, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, false, 'Placeholder state is correct' );
 	} );
 
 	QUnit.test( 'Placeholder first, then real thumbnail', function ( assert ) {
 		var viewer = mw.mmv.testHelpers.getMultimediaViewer();
 
-		viewer.setImage = $.noop;
+		viewer.setImage = function () {};
 		viewer.ui = { canvas: {
-			unblurWithAnimation: $.noop,
-			unblur: $.noop,
+			unblurWithAnimation: function () {},
+			unblur: function () {},
 			maybeDisplayPlaceholder: function () { return true; }
 		} };
 		viewer.imageInfoProvider.get = this.sandbox.stub();
 
 		viewer.displayPlaceholderThumbnail( { originalWidth: 100, originalHeight: 100 }, undefined, undefined );
 
-		assert.ok( viewer.blurredThumbnailShown, 'Placeholder state is correct' );
-		assert.ok( !viewer.realThumbnailShown, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, true, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, false, 'Real thumbnail state is correct' );
 
 		viewer.displayRealThumbnail( { url: undefined } );
 
-		assert.ok( viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, true, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, true, 'Placeholder state is correct' );
 	} );
 
 	QUnit.test( 'Placeholder first, then real thumbnail - missing size', function ( assert ) {
 		var viewer = mw.mmv.testHelpers.getMultimediaViewer();
 
 		viewer.currentIndex = 1;
-		viewer.setImage = $.noop;
+		viewer.setImage = function () {};
 		viewer.ui = { canvas: {
-			unblurWithAnimation: $.noop,
-			unblur: $.noop,
+			unblurWithAnimation: function () {},
+			unblur: function () {},
 			maybeDisplayPlaceholder: function () { return true; }
 		} };
 		viewer.imageInfoProvider.get = this.sandbox.stub().returns( $.Deferred().resolve( { width: 100, height: 100 } ) );
 
 		viewer.displayPlaceholderThumbnail( { index: 1 }, undefined, undefined );
 
-		assert.ok( viewer.blurredThumbnailShown, 'Placeholder state is correct' );
-		assert.ok( !viewer.realThumbnailShown, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, true, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, false, 'Real thumbnail state is correct' );
 
 		viewer.displayRealThumbnail( { url: undefined } );
 
-		assert.ok( viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, true, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, true, 'Placeholder state is correct' );
 	} );
 
 	QUnit.test( 'Real thumbnail first, then placeholder', function ( assert ) {
 		var viewer = mw.mmv.testHelpers.getMultimediaViewer();
 
-		viewer.setImage = $.noop;
+		viewer.setImage = function () {};
 		viewer.ui = {
-			showImage: $.noop,
+			showImage: function () {},
 			canvas: {
-				unblurWithAnimation: $.noop,
-				unblur: $.noop
+				unblurWithAnimation: function () {},
+				unblur: function () {}
 			} };
 
 		viewer.displayRealThumbnail( { url: undefined } );
 
-		assert.ok( viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( !viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, true, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, false, 'Placeholder state is correct' );
 
 		viewer.displayPlaceholderThumbnail( {}, undefined, undefined );
 
-		assert.ok( viewer.realThumbnailShown, 'Real thumbnail state is correct' );
-		assert.ok( !viewer.blurredThumbnailShown, 'Placeholder state is correct' );
+		assert.strictEqual( viewer.realThumbnailShown, true, 'Real thumbnail state is correct' );
+		assert.strictEqual( viewer.blurredThumbnailShown, false, 'Placeholder state is correct' );
 	} );
 
 	QUnit.test( 'displayRealThumbnail', function ( assert ) {
 		var viewer = mw.mmv.testHelpers.getMultimediaViewer();
 
-		viewer.setImage = $.noop;
+		viewer.setImage = function () {};
 		viewer.ui = { canvas: {
 			unblurWithAnimation: this.sandbox.stub(),
-			unblur: $.noop
+			unblur: function () {}
 		} };
 		viewer.blurredThumbnailShown = true;
 
 		// Should not result in an unblurWithAnimation animation (image cache from cache)
 		viewer.displayRealThumbnail( { url: undefined }, undefined, undefined, 5 );
-		assert.ok( !viewer.ui.canvas.unblurWithAnimation.called, 'There should not be an unblurWithAnimation animation' );
+		assert.strictEqual( viewer.ui.canvas.unblurWithAnimation.called, false, 'There should not be an unblurWithAnimation animation' );
 
 		// Should result in an unblurWithAnimation (image didn't come from cache)
 		viewer.displayRealThumbnail( { url: undefined }, undefined, undefined, 1000 );
-		assert.ok( viewer.ui.canvas.unblurWithAnimation.called, 'There should be an unblurWithAnimation animation' );
+		assert.strictEqual( viewer.ui.canvas.unblurWithAnimation.called, true, 'There should be an unblurWithAnimation animation' );
 	} );
 
 	QUnit.test( 'New image loaded while another one is loading', function ( assert ) {
@@ -419,31 +412,31 @@
 			// custom clock ensures progress handlers execute in correct sequence
 			clock = this.sandbox.useFakeTimers();
 
-		viewer.preloadFullscreenThumbnail = $.noop;
+		viewer.preloadFullscreenThumbnail = function () {};
 		viewer.fetchSizeIndependentLightboxInfo = this.sandbox.stub();
 		viewer.ui = {
-			setFileReuseData: $.noop,
-			setupForLoad: $.noop,
+			setFileReuseData: function () {},
+			setupForLoad: function () {},
 			canvas: {
-				set: $.noop,
+				set: function () {},
 				getCurrentImageWidths: function () { return { real: 0 }; },
 				getDimensions: function () { return {}; }
 			},
 			panel: {
 				setImageInfo: this.sandbox.stub(),
 				scroller: {
-					animateMetadataOnce: $.noop
+					animateMetadataOnce: function () {}
 				},
 				progressBar: {
 					animateTo: this.sandbox.stub(),
 					jumpTo: this.sandbox.stub()
 				},
-				empty: $.noop
+				empty: function () {}
 			},
-			open: $.noop,
-			empty: $.noop };
+			open: function () {},
+			empty: function () {} };
 		viewer.displayRealThumbnail = this.sandbox.stub();
-		viewer.eachPrealoadableLightboxIndex = $.noop;
+		viewer.eachPreloadableLightboxIndex = function () {};
 		viewer.animateMetadataDivOnce = this.sandbox.stub().returns( $.Deferred().reject() );
 		viewer.imageProvider.get = this.sandbox.stub();
 		viewer.imageInfoProvider.get = function () { return $.Deferred().reject(); };
@@ -453,8 +446,8 @@
 		viewer.fetchSizeIndependentLightboxInfo.returns( firstLigthboxInfoDeferred.promise() );
 		viewer.loadImage( firstImage, new Image() );
 		clock.tick( 10 );
-		assert.ok( !viewer.animateMetadataDivOnce.called, 'Metadata of the first image should not be animated' );
-		assert.ok( !viewer.ui.panel.setImageInfo.called, 'Metadata of the first image should not be shown' );
+		assert.strictEqual( viewer.animateMetadataDivOnce.called, false, 'Metadata of the first image should not be animated' );
+		assert.strictEqual( viewer.ui.panel.setImageInfo.called, false, 'Metadata of the first image should not be shown' );
 
 		viewer.imageProvider.get.returns( secondImageDeferred.promise() );
 		viewer.fetchSizeIndependentLightboxInfo.returns( secondLigthboxInfoDeferred.promise() );
@@ -464,18 +457,18 @@
 		viewer.ui.panel.progressBar.animateTo.reset();
 		firstImageDeferred.notify( undefined, 45 );
 		clock.tick( 10 );
-		assert.ok( !viewer.ui.panel.progressBar.animateTo.reset.called, 'Progress of the first image should not be shown' );
+		assert.strictEqual( viewer.ui.panel.progressBar.animateTo.reset.called, undefined, 'Progress of the first image should not be shown' );
 
 		firstImageDeferred.resolve( {}, {} );
 		firstLigthboxInfoDeferred.resolve( {} );
 		clock.tick( 10 );
-		assert.ok( !viewer.displayRealThumbnail.called, 'The first image being done loading should have no effect' );
+		assert.strictEqual( viewer.displayRealThumbnail.called, false, 'The first image being done loading should have no effect' );
 
 		viewer.displayRealThumbnail = this.sandbox.spy( function () { viewer.close(); } );
 		secondImageDeferred.resolve( {}, {} );
 		secondLigthboxInfoDeferred.resolve( {} );
 		clock.tick( 10 );
-		assert.ok( viewer.displayRealThumbnail.called, 'The second image being done loading should result in the image being shown' );
+		assert.strictEqual( viewer.displayRealThumbnail.called, true, 'The second image being done loading should result in the image being shown' );
 
 		clock.restore();
 	} );
@@ -496,7 +489,7 @@
 		// animation would keep running, conflict with other tests
 		this.sandbox.stub( $.fn, 'animate' ).returnsThis();
 
-		$.scrollTo = function () { return { scrollTop: $.noop, on: $.noop, off: $.noop }; };
+		$.scrollTo = function () { return { scrollTop: function () {}, on: function () {}, off: function () {} }; };
 
 		viewer.setupEventHandlers();
 
@@ -505,7 +498,7 @@
 		viewer.thumbnailInfoProvider.get = function () { return $.Deferred().reject(); };
 		viewer.fileRepoInfoProvider.get = function () { return $.Deferred().reject(); };
 
-		viewer.preloadFullscreenThumbnail = $.noop;
+		viewer.preloadFullscreenThumbnail = function () {};
 		viewer.initWithThumbs( [] );
 
 		viewer.loadImage(
@@ -517,7 +510,7 @@
 			new Image()
 		);
 
-		viewer.ui.$closeButton.click();
+		viewer.ui.$closeButton.trigger( 'click' );
 
 		function eventHandler( e ) {
 			if ( e.isDefaultPrevented() ) {
@@ -562,16 +555,25 @@
 	} );
 
 	QUnit.test( 'Refuse to load too-big thumbnails', function ( assert ) {
-		var viewer = mw.mmv.testHelpers.getMultimediaViewer(),
-			intendedWidth = 50,
-			title = mw.Title.newFromText( 'File:Foobar.svg' );
+		var title, expectedWidth,
+			reuestedWidth = 1000,
+			originalWidth = 50,
+			viewer = mw.mmv.testHelpers.getMultimediaViewer();
 
 		viewer.thumbnailInfoProvider.get = function ( fileTitle, width ) {
-			assert.strictEqual( width, intendedWidth );
+			assert.strictEqual( width, expectedWidth );
 			return $.Deferred().reject();
 		};
 
-		viewer.fetchThumbnail( title, 1000, null, intendedWidth, 60 );
+		// non-vector should be capped to original size
+		title = mw.Title.newFromText( 'File:Foobar.png' );
+		expectedWidth = originalWidth;
+		viewer.fetchThumbnail( title, reuestedWidth, null, originalWidth, 60 );
+
+		// vector images can be aritrarily large
+		title = mw.Title.newFromText( 'File:Foobar.svg' );
+		expectedWidth = reuestedWidth;
+		viewer.fetchThumbnail( title, reuestedWidth, null, originalWidth, 60 );
 	} );
 
 	QUnit.test( 'fetchThumbnail()', function ( assert ) {
@@ -580,7 +582,7 @@
 			imageStub,
 			promise,
 			useThumbnailGuessing,
-			viewer = new mw.mmv.MultimediaViewer( { imageQueryParameter: $.noop, language: $.noop, recordVirtualViewBeaconURI: $.noop, extensions: function () { return { jpg: 'default' }; }, useThumbnailGuessing: function () { return useThumbnailGuessing; } } ),
+			viewer = new mw.mmv.MultimediaViewer( { imageQueryParameter: function () {}, language: function () {}, recordVirtualViewBeaconURI: function () {}, extensions: function () { return { jpg: 'default' }; }, useThumbnailGuessing: function () { return useThumbnailGuessing; } } ),
 			sandbox = this.sandbox,
 			file = new mw.Title( 'File:Copyleft.svg' ),
 			sampleURL = 'http://upload.wikimedia.org/wikipedia/commons/thumb/8/8b/Copyleft.svg/300px-Copyleft.svg.png',
@@ -606,10 +608,10 @@
 		imageStub.returns( $.Deferred().resolve( image ) );
 		promise = viewer.fetchThumbnail( file, width );
 		clock.tick( 10 );
-		assert.ok( !guessedThumbnailInfoStub.called, 'When we lack sample URL and original dimensions, GuessedThumbnailInfoProvider is not called' );
-		assert.ok( thumbnailInfoStub.calledOnce, 'When we lack sample URL and original dimensions, ThumbnailInfoProvider is called once' );
-		assert.ok( imageStub.calledOnce, 'When we lack sample URL and original dimensions, ImageProvider is called once' );
-		assert.ok( imageStub.calledWith( 'apiURL' ), 'When we lack sample URL and original dimensions, ImageProvider is called with the API url' );
+		assert.strictEqual( guessedThumbnailInfoStub.called, false, 'When we lack sample URL and original dimensions, GuessedThumbnailInfoProvider is not called' );
+		assert.strictEqual( thumbnailInfoStub.calledOnce, true, 'When we lack sample URL and original dimensions, ThumbnailInfoProvider is called once' );
+		assert.strictEqual( imageStub.calledOnce, true, 'When we lack sample URL and original dimensions, ImageProvider is called once' );
+		assert.strictEqual( imageStub.calledWith( 'apiURL' ), true, 'When we lack sample URL and original dimensions, ImageProvider is called with the API url' );
 		assert.strictEqual( promise.state(), 'resolved', 'When we lack sample URL and original dimensions, fetchThumbnail resolves' );
 
 		// When the guesser bails out, the classic provider should be used
@@ -619,10 +621,10 @@
 		imageStub.returns( $.Deferred().resolve( image ) );
 		promise = viewer.fetchThumbnail( file, width, sampleURL, originalWidth, originalHeight );
 		clock.tick( 10 );
-		assert.ok( guessedThumbnailInfoStub.calledOnce, 'When the guesser bails out, GuessedThumbnailInfoProvider is called once' );
-		assert.ok( thumbnailInfoStub.calledOnce, 'When the guesser bails out, ThumbnailInfoProvider is called once' );
-		assert.ok( imageStub.calledOnce, 'When the guesser bails out, ImageProvider is called once' );
-		assert.ok( imageStub.calledWith( 'apiURL' ), 'When the guesser bails out, ImageProvider is called with the API url' );
+		assert.strictEqual( guessedThumbnailInfoStub.calledOnce, true, 'When the guesser bails out, GuessedThumbnailInfoProvider is called once' );
+		assert.strictEqual( thumbnailInfoStub.calledOnce, true, 'When the guesser bails out, ThumbnailInfoProvider is called once' );
+		assert.strictEqual( imageStub.calledOnce, true, 'When the guesser bails out, ImageProvider is called once' );
+		assert.strictEqual( imageStub.calledWith( 'apiURL' ), true, 'When the guesser bails out, ImageProvider is called with the API url' );
 		assert.strictEqual( promise.state(), 'resolved', 'When the guesser bails out, fetchThumbnail resolves' );
 
 		// When the guesser returns an URL, that should be used
@@ -632,10 +634,10 @@
 		imageStub.returns( $.Deferred().resolve( image ) );
 		promise = viewer.fetchThumbnail( file, width, sampleURL, originalWidth, originalHeight );
 		clock.tick( 10 );
-		assert.ok( guessedThumbnailInfoStub.calledOnce, 'When the guesser returns an URL, GuessedThumbnailInfoProvider is called once' );
-		assert.ok( !thumbnailInfoStub.called, 'When the guesser returns an URL, ThumbnailInfoProvider is not called' );
-		assert.ok( imageStub.calledOnce, 'When the guesser returns an URL, ImageProvider is called once' );
-		assert.ok( imageStub.calledWith( 'guessedURL' ), 'When the guesser returns an URL, ImageProvider is called with the guessed url' );
+		assert.strictEqual( guessedThumbnailInfoStub.calledOnce, true, 'When the guesser returns an URL, GuessedThumbnailInfoProvider is called once' );
+		assert.strictEqual( thumbnailInfoStub.called, false, 'When the guesser returns an URL, ThumbnailInfoProvider is not called' );
+		assert.strictEqual( imageStub.calledOnce, true, 'When the guesser returns an URL, ImageProvider is called once' );
+		assert.strictEqual( imageStub.calledWith( 'guessedURL' ), true, 'When the guesser returns an URL, ImageProvider is called with the guessed url' );
 		assert.strictEqual( promise.state(), 'resolved', 'When the guesser returns an URL, fetchThumbnail resolves' );
 
 		// When the guesser returns an URL, but that returns 404, image loading should be retried with the classic provider
@@ -646,11 +648,11 @@
 		imageStub.withArgs( 'apiURL' ).returns( $.Deferred().resolve( image ) );
 		promise = viewer.fetchThumbnail( file, width, sampleURL, originalWidth, originalHeight );
 		clock.tick( 10 );
-		assert.ok( guessedThumbnailInfoStub.calledOnce, 'When the guesser returns an URL, but that returns 404, GuessedThumbnailInfoProvider is called once' );
-		assert.ok( thumbnailInfoStub.calledOnce, 'When the guesser returns an URL, but that returns 404, ThumbnailInfoProvider is called once' );
-		assert.ok( imageStub.calledTwice, 'When the guesser returns an URL, but that returns 404, ImageProvider is called twice' );
-		assert.ok( imageStub.getCall( 0 ).calledWith( 'guessedURL' ), 'When the guesser returns an URL, but that returns 404, ImageProvider is called first with the guessed url' );
-		assert.ok( imageStub.getCall( 1 ).calledWith( 'apiURL' ), 'When the guesser returns an URL, but that returns 404, ImageProvider is called second with the guessed url' );
+		assert.strictEqual( guessedThumbnailInfoStub.calledOnce, true, 'When the guesser returns an URL, but that returns 404, GuessedThumbnailInfoProvider is called once' );
+		assert.strictEqual( thumbnailInfoStub.calledOnce, true, 'When the guesser returns an URL, but that returns 404, ThumbnailInfoProvider is called once' );
+		assert.strictEqual( imageStub.calledTwice, true, 'When the guesser returns an URL, but that returns 404, ImageProvider is called twice' );
+		assert.strictEqual( imageStub.getCall( 0 ).calledWith( 'guessedURL' ), true, 'When the guesser returns an URL, but that returns 404, ImageProvider is called first with the guessed url' );
+		assert.strictEqual( imageStub.getCall( 1 ).calledWith( 'apiURL' ), true, 'When the guesser returns an URL, but that returns 404, ImageProvider is called second with the guessed url' );
 		assert.strictEqual( promise.state(), 'resolved', 'When the guesser returns an URL, but that returns 404, fetchThumbnail resolves' );
 
 		// When even the retry fails, fetchThumbnail() should reject
@@ -661,11 +663,11 @@
 		imageStub.withArgs( 'apiURL' ).returns( $.Deferred().reject() );
 		promise = viewer.fetchThumbnail( file, width, sampleURL, originalWidth, originalHeight );
 		clock.tick( 10 );
-		assert.ok( guessedThumbnailInfoStub.calledOnce, 'When even the retry fails, GuessedThumbnailInfoProvider is called once' );
-		assert.ok( thumbnailInfoStub.calledOnce, 'When even the retry fails, ThumbnailInfoProvider is called once' );
-		assert.ok( imageStub.calledTwice, 'When even the retry fails, ImageProvider is called twice' );
-		assert.ok( imageStub.getCall( 0 ).calledWith( 'guessedURL' ), 'When even the retry fails, ImageProvider is called first with the guessed url' );
-		assert.ok( imageStub.getCall( 1 ).calledWith( 'apiURL' ), 'When even the retry fails, ImageProvider is called second with the guessed url' );
+		assert.strictEqual( guessedThumbnailInfoStub.calledOnce, true, 'When even the retry fails, GuessedThumbnailInfoProvider is called once' );
+		assert.strictEqual( thumbnailInfoStub.calledOnce, true, 'When even the retry fails, ThumbnailInfoProvider is called once' );
+		assert.strictEqual( imageStub.calledTwice, true, 'When even the retry fails, ImageProvider is called twice' );
+		assert.strictEqual( imageStub.getCall( 0 ).calledWith( 'guessedURL' ), true, 'When even the retry fails, ImageProvider is called first with the guessed url' );
+		assert.strictEqual( imageStub.getCall( 1 ).calledWith( 'apiURL' ), true, 'When even the retry fails, ImageProvider is called second with the guessed url' );
 		assert.strictEqual( promise.state(), 'rejected', 'When even the retry fails, fetchThumbnail rejects' );
 
 		useThumbnailGuessing = false;
@@ -677,10 +679,10 @@
 		imageStub.returns( $.Deferred().resolve( image ) );
 		promise = viewer.fetchThumbnail( file, width );
 		clock.tick( 10 );
-		assert.ok( !guessedThumbnailInfoStub.called, 'When guessing is disabled, GuessedThumbnailInfoProvider is not called' );
-		assert.ok( thumbnailInfoStub.calledOnce, 'When guessing is disabled, ThumbnailInfoProvider is called once' );
-		assert.ok( imageStub.calledOnce, 'When guessing is disabled, ImageProvider is called once' );
-		assert.ok( imageStub.calledWith( 'apiURL' ), 'When guessing is disabled, ImageProvider is called with the API url' );
+		assert.strictEqual( guessedThumbnailInfoStub.called, false, 'When guessing is disabled, GuessedThumbnailInfoProvider is not called' );
+		assert.strictEqual( thumbnailInfoStub.calledOnce, true, 'When guessing is disabled, ThumbnailInfoProvider is called once' );
+		assert.strictEqual( imageStub.calledOnce, true, 'When guessing is disabled, ImageProvider is called once' );
+		assert.strictEqual( imageStub.calledWith( 'apiURL' ), true, 'When guessing is disabled, ImageProvider is called with the API url' );
 		assert.strictEqual( promise.state(), 'resolved', 'When guessing is disabled, fetchThumbnail resolves' );
 
 		clock.restore();
@@ -694,7 +696,7 @@
 
 		viewer.currentImageFileTitle = title;
 		bootstrap.setupEventHandlers();
-		viewer.setHash();
+		viewer.setMediaHash();
 
 		assert.ok( document.title.match( title.getNameText() ), 'File name is visible in title' );
 
@@ -703,4 +705,4 @@
 
 		assert.strictEqual( document.title, oldDocumentTitle, 'Original title restored after viewer is closed' );
 	} );
-}( mediaWiki, jQuery ) );
+}() );
