@@ -1,5 +1,7 @@
 <?php
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * Special page that allows authorised users to rename
  * user accounts
@@ -23,7 +25,8 @@ class SpecialRenameuser extends SpecialPage {
 	 * @throws UserBlockedError
 	 */
 	public function execute( $par ) {
-		global $wgContLang, $wgCapitalLinks;
+		global $wgCapitalLinks;
+		$contLang = MediaWikiServices::getInstance()->getContentLanguage();
 
 		$this->setHeaders();
 		$this->addHelpLink( 'Help:Renameuser' );
@@ -36,12 +39,10 @@ class SpecialRenameuser extends SpecialPage {
 			throw new PermissionsError( 'renameuser' );
 		}
 
-		if ( wfReadOnly() ) {
-			throw new ReadOnlyError;
-		}
+		$this->checkReadOnly();
 
 		if ( $user->isBlocked() ) {
-			throw new UserBlockedError( $this->getUser()->mBlock );
+			throw new UserBlockedError( $this->getUser()->getBlock() );
 		}
 
 		$this->useTransactionalTimeLimit();
@@ -51,11 +52,11 @@ class SpecialRenameuser extends SpecialPage {
 		$usernames = explode( '/', $par, 2 ); // this works as "/" is not valid in usernames
 		$oldnamePar = trim( str_replace( '_', ' ', $request->getText( 'oldusername', $usernames[0] ) ) );
 		$oldusername = Title::makeTitle( NS_USER, $oldnamePar );
-		$newnamePar = isset( $usernames[1] ) ? $usernames[1] : null;
+		$newnamePar = $usernames[1] ?? '';
 		$newnamePar = trim( str_replace( '_', ' ', $request->getText( 'newusername', $newnamePar ) ) );
 		// Force uppercase of newusername, otherwise wikis
 		// with wgCapitalLinks=false can create lc usernames
-		$newusername = Title::makeTitleSafe( NS_USER, $wgContLang->ucfirst( $newnamePar ) );
+		$newusername = Title::makeTitleSafe( NS_USER, $contLang->ucfirst( $newnamePar ) );
 		$oun = is_object( $oldusername ) ? $oldusername->getText() : '';
 		$nun = is_object( $newusername ) ? $newusername->getText() : '';
 		$token = $user->getEditToken();
@@ -138,6 +139,7 @@ class SpecialRenameuser extends SpecialPage {
 				);
 			}
 		}
+		// @phan-suppress-next-line PhanImpossibleCondition May set by hook
 		if ( $warnings ) {
 			$warningsHtml = [];
 			foreach ( $warnings as $warning ) {
@@ -255,7 +257,7 @@ class SpecialRenameuser extends SpecialPage {
 
 		// Check for the existence of lowercase oldusername in database.
 		// Until r19631 it was possible to rename a user to a name with first character as lowercase
-		if ( $oldusername->getText() !== $wgContLang->ucfirst( $oldusername->getText() ) ) {
+		if ( $oldusername->getText() !== $contLang->ucfirst( $oldusername->getText() ) ) {
 			// oldusername was entered as lowercase -> check for existence in table 'user'
 			$dbr = wfGetDB( DB_REPLICA );
 			$uid = $dbr->selectField( 'user', 'user_id',
@@ -309,7 +311,7 @@ class SpecialRenameuser extends SpecialPage {
 			return;
 		}
 
-		// If this user is renaming his/herself, make sure that Title::moveTo()
+		// If this user is renaming his/herself, make sure that MovePage::move()
 		// doesn't make a bunch of null move edits under the old name!
 		if ( $user->getId() === $uid ) {
 			$user->setName( $newusername->getText() );
@@ -340,12 +342,17 @@ class SpecialRenameuser extends SpecialPage {
 
 			$output = '';
 			$linkRenderer = $this->getLinkRenderer();
+			$movePageFactory = MediaWikiServices::getInstance()->getMovePageFactory();
 			foreach ( $pages as $row ) {
 				$oldPage = Title::makeTitleSafe( $row->page_namespace, $row->page_title );
 				$newPage = Title::makeTitleSafe( $row->page_namespace,
 					preg_replace( '!^[^/]+!', $newusername->getDBkey(), $row->page_title ) );
+
+				$movePage = $movePageFactory->newMovePage( $oldPage, $newPage );
+				$validMoveStatus = $movePage->isValidMove();
+
 				# Do not autodelete or anything, title must not exist
-				if ( $newPage->exists() && !$oldPage->isValidMoveTarget( $newPage ) ) {
+				if ( $newPage->exists() && !$validMoveStatus->isOK() ) {
 					$link = $linkRenderer->makeKnownLink( $newPage );
 					$output .= Html::rawElement(
 						'li',
@@ -353,16 +360,13 @@ class SpecialRenameuser extends SpecialPage {
 						$this->msg( 'renameuser-page-exists' )->rawParams( $link )->escaped()
 					);
 				} else {
-					$success = $oldPage->moveTo(
-						$newPage,
-						false,
-						$this->msg(
-							'renameuser-move-log',
-							$oldusername->getText(),
-							$newusername->getText() )->inContentLanguage()->text(),
-						!$suppressRedirect
-					);
-					if ( $success === true ) {
+					$logReason = $this->msg(
+						'renameuser-move-log', $oldusername->getText(), $newusername->getText()
+					)->inContentLanguage()->text();
+
+					$moveStatus = $movePage->move( $user, $logReason, !$suppressRedirect );
+
+					if ( $moveStatus->isOK() ) {
 						# oldPage is not known in case of redirect suppression
 						$oldLink = $linkRenderer->makeLink( $oldPage, null, [], [ 'redirect' => 'no' ] );
 
@@ -415,9 +419,6 @@ class SpecialRenameuser extends SpecialPage {
 	 * @return string[] Matching subpages
 	 */
 	public function prefixSearchSubpages( $search, $limit, $offset ) {
-		if ( !class_exists( 'UserNamePrefixSearch' ) ) { // check for version 1.27
-			return [];
-		}
 		$user = User::newFromName( $search );
 		if ( !$user ) {
 			// No prefix suggestion for invalid user
