@@ -13,6 +13,7 @@ mw.TemplateData.Model = function mwTemplateDataModel() {
 	// Properties
 	this.params = {};
 	this.description = {};
+	this.maps = {};
 	this.paramOrder = [];
 	this.format = null;
 	this.paramOrderChanged = false;
@@ -268,6 +269,10 @@ mw.TemplateData.Model.static.newFromObject = function ( tdObject, paramsInSource
 			model.addParam( param, tdObject.params[ param ] );
 		}
 	}
+
+	// maps
+	model.setMapInfo( JSON.stringify( tdObject.maps, null, 4 ) );
+
 	model.setTemplateDescription( tdObject.description );
 
 	// Override the param order if it exists in the templatedata string
@@ -523,6 +528,35 @@ mw.TemplateData.Model.prototype.getTemplateDescription = function ( language ) {
 	return this.description[ language ];
 };
 
+/**
+ * Set the template description
+ *
+ * @param {string|Object} map New template map info
+ * @fires change-map
+ * @fires change
+ */
+mw.TemplateData.Model.prototype.setMapInfo = function ( map ) {
+	if ( !this.constructor.static.compare( this.maps, map ) ) {
+		if ( typeof map === 'object' ) {
+			$.extend( this.maps, map );
+			this.emit( 'change-map', map );
+		} else {
+			this.maps = map;
+			this.emit( 'change-map', map );
+		}
+		this.emit( 'change' );
+	}
+};
+
+/**
+ * Get the template info.
+ *
+ * @return {string|Object} The template map info.
+ */
+mw.TemplateData.Model.prototype.getMapInfo = function () {
+	return this.maps;
+};
+
 /**
  * Get a specific parameter's localized property
  *
@@ -595,6 +629,8 @@ mw.TemplateData.Model.prototype.addKeyTemplateParamOrder = function ( key ) {
 /**
  * TODO: document
  *
+ * @param {string} key
+ * @param {number} newIndex
  * @fires change-paramOrder
  * @fires change
  */
@@ -1055,6 +1091,7 @@ mw.TemplateData.Model.prototype.getNewValidParameterKey = function ( key ) {
 /**
  * Go over a language property and remove empty language key values
  *
+ * @param {Object} propData Property data
  * @return {Object} Property data with only used language keys
  */
 mw.TemplateData.Model.prototype.propRemoveUnusedLanguages = function ( propData ) {
