@@ -145,7 +145,7 @@
 		var $textarea = $( '#wpTextbox1' ),
 			$editingSessionIdInput = $( '#editingStatsId' ),
 			origText = $textarea.val(),
-			submitting, onUnloadFallback, dialogsConfig, readyTime;
+			submitting, onUnloadFallback, readyTime;
 
 		if ( $editingSessionIdInput.length ) {
 			editingSessionId = $editingSessionIdInput.val();
@@ -217,6 +217,12 @@
 		$( '#toolbar' ).remove();
 		// Add toolbar module
 		// TODO: Implement .wikiEditor( 'remove' )
+		mw.addWikiEditor( $textarea );
+	} );
+
+	mw.addWikiEditor = function ( $textarea ) {
+		var dialogsConfig;
+
 		$textarea.wikiEditor(
 			'addModule', require( './jquery.wikiEditor.toolbar.config.js' )
 		);
@@ -226,5 +232,6 @@
 		dialogsConfig.replaceIcons( $textarea );
 		// Add dialogs module
 		$textarea.wikiEditor( 'addModule', dialogsConfig.getDefaultConfig() );
-	} );
+
+	};
 }() );
