@@ -115,9 +115,9 @@ if ( !interface_exists( LoggerInterface::class ) ) {
 		'MediaWiki requires the <a href="https://github.com/php-fig/log">PSR-3 logging ' .
 		"library</a> to be present. This library is not embedded directly in MediaWiki's " .
 		"git repository and must be installed separately by the end user.\n\n" .
-		'Please see <a href="https://www.mediawiki.org/wiki/Download_from_Git' .
-		'#Fetch_external_libraries">mediawiki.org</a> for help on installing ' .
-		'the required components.'
+		'Please see the <a href="https://www.mediawiki.org/wiki/Download_from_Git' .
+		'#Fetch_external_libraries">instructions for installing libraries</a> on mediawiki.org ' .
+		'for help on installing the required components.'
 	);
 	echo $message;
 	trigger_error( $message, E_USER_ERROR );
