@@ -20,9 +20,9 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
 use Wikimedia\Rdbms\Database;
 use Wikimedia\Rdbms\IDatabase;
-use MediaWiki\MediaWikiServices;
 use Wikimedia\Rdbms\LoadBalancer;
 
 /**
@@ -52,14 +52,14 @@ class SiteStats {
 		$config = MediaWikiServices::getInstance()->getMainConfig();
 
 		$lb = self::getLB();
-		$dbr = $lb->getConnection( DB_REPLICA );
-		wfDebug( __METHOD__ . ": reading site_stats from replica DB\n" );
+		$dbr = $lb->getConnectionRef( DB_REPLICA );
+		wfDebug( __METHOD__ . ": reading site_stats from replica DB" );
 		$row = self::doLoadFromDB( $dbr );
 
 		if ( !self::isRowSane( $row ) && $lb->hasOrMadeRecentMasterChanges() ) {
 			// Might have just been initialized during this request? Underflow?
-			wfDebug( __METHOD__ . ": site_stats damaged or missing on replica DB\n" );
-			$row = self::doLoadFromDB( $lb->getConnection( DB_MASTER ) );
+			wfDebug( __METHOD__ . ": site_stats damaged or missing on replica DB" );
+			$row = self::doLoadFromDB( $lb->getConnectionRef( DB_MASTER ) );
 		}
 
 		if ( !self::isRowSane( $row ) ) {
@@ -72,15 +72,15 @@ class SiteStats {
 				// Some manual construction scenarios may leave the table empty or
 				// broken, however, for instance when importing from a dump into a
 				// clean schema with mwdumper.
-				wfDebug( __METHOD__ . ": initializing damaged or missing site_stats\n" );
+				wfDebug( __METHOD__ . ": initializing damaged or missing site_stats" );
 				SiteStatsInit::doAllAndCommit( $dbr );
 			}
 
-			$row = self::doLoadFromDB( $lb->getConnection( DB_MASTER ) );
+			$row = self::doLoadFromDB( $lb->getConnectionRef( DB_MASTER ) );
 		}
 
 		if ( !self::isRowSane( $row ) ) {
-			wfDebug( __METHOD__ . ": site_stats persistently nonsensical o_O\n" );
+			wfDebug( __METHOD__ . ": site_stats persistently nonsensical o_O" );
 			// Always return a row-like object
 			$row = self::salvageInsaneRow( $row );
 		}
@@ -149,12 +149,13 @@ class SiteStats {
 	 */
 	public static function numberingroup( $group ) {
 		$cache = MediaWikiServices::getInstance()->getMainWANObjectCache();
+		$fname = __METHOD__;
 
 		return $cache->getWithSetCallback(
 			$cache->makeKey( 'SiteStats', 'groupcounts', $group ),
 			$cache::TTL_HOUR,
-			function ( $oldValue, &$ttl, array &$setOpts ) use ( $group ) {
-				$dbr = self::getLB()->getConnection( DB_REPLICA );
+			function ( $oldValue, &$ttl, array &$setOpts ) use ( $group, $fname ) {
+				$dbr = self::getLB()->getConnectionRef( DB_REPLICA );
 				$setOpts += Database::getCacheSetOptions( $dbr );
 
 				return (int)$dbr->selectField(
@@ -164,7 +165,7 @@ class SiteStats {
 						'ug_group' => $group,
 						'ug_expiry IS NULL OR ug_expiry >= ' . $dbr->addQuotes( $dbr->timestamp() )
 					],
-					__METHOD__
+					$fname
 				);
 			},
 			[ 'pcTTL' => $cache::TTL_PROC_LONG ]
@@ -199,19 +200,20 @@ class SiteStats {
 	 */
 	public static function pagesInNs( $ns ) {
 		$cache = MediaWikiServices::getInstance()->getMainWANObjectCache();
+		$fname = __METHOD__;
 
 		return $cache->getWithSetCallback(
 			$cache->makeKey( 'SiteStats', 'page-in-namespace', $ns ),
 			$cache::TTL_HOUR,
-			function ( $oldValue, &$ttl, array &$setOpts ) use ( $ns ) {
-				$dbr = self::getLB()->getConnection( DB_REPLICA );
+			function ( $oldValue, &$ttl, array &$setOpts ) use ( $ns, $fname ) {
+				$dbr = self::getLB()->getConnectionRef( DB_REPLICA );
 				$setOpts += Database::getCacheSetOptions( $dbr );
 
 				return (int)$dbr->selectField(
 					'page',
 					'COUNT(*)',
 					[ 'page_namespace' => $ns ],
-					__METHOD__
+					$fname
 				);
 			},
 			[ 'pcTTL' => $cache::TTL_PROC_LONG ]
