@@ -67,7 +67,7 @@ class NameTableStore {
 	/**
 	 * @param ILoadBalancer $dbLoadBalancer A load balancer for acquiring database connections
 	 * @param WANObjectCache $cache A cache manager for caching data. This can be the local
-	 *        wiki's default instance even if $wikiId refers to a different wiki, since
+	 *        wiki's default instance even if $dbDomain refers to a different wiki, since
 	 *        makeGlobalKey() is used to constructed a key that allows cached names from
 	 *        the same database to be re-used between wikis. For example, enwiki and frwiki will
 	 *        use the same cache keys for names from the wikidatawiki database, regardless
@@ -170,10 +170,7 @@ class NameTableStore {
 			if ( $id === null ) {
 				// RACE: $name was already in the db, probably just inserted, so load from master.
 				// Use DBO_TRX to avoid missing inserts due to other threads or REPEATABLE-READs.
-				// ...but not during unit tests, because we need the fake DB tables of the default
-				// connection.
-				$connFlags = defined( 'MW_PHPUNIT_TEST' ) ? 0 : ILoadBalancer::CONN_TRX_AUTOCOMMIT;
-				$table = $this->reloadMap( $connFlags );
+				$table = $this->reloadMap( ILoadBalancer::CONN_TRX_AUTOCOMMIT );
 
 				$searchResult = array_search( $name, $table, true );
 				if ( $searchResult === false ) {
@@ -206,7 +203,7 @@ class NameTableStore {
 				$dbw = $this->getDBConnection( DB_MASTER );
 				$dbw->onTransactionPreCommitOrIdle( function () {
 					$this->cache->delete( $this->getCacheKey() );
-				} );
+				}, __METHOD__ );
 			}
 			$this->tableCache = $table;
 		}
@@ -237,7 +234,7 @@ class NameTableStore {
 		$this->tableCache = $this->loadTable( $dbw );
 		$dbw->onTransactionPreCommitOrIdle( function () {
 			$this->cache->reap( $this->getCacheKey(), INF );
-		} );
+		}, __METHOD__ );
 
 		return $this->tableCache;
 	}
@@ -460,7 +457,7 @@ class NameTableStore {
 		try {
 			$dbw->doAtomicSection(
 				__METHOD__,
-				function ( IDatabase $unused, $fname ) use ( $name, $id, &$ok, $dbw ) {
+				function ( IDatabase $unused, $fname ) use ( $name, $id, $dbw ) {
 					// Try to insert a row with the ID we originally got.
 					// If that fails (because of a key conflict), we will just try to get another ID again later.
 					$dbw->insert(
@@ -473,7 +470,7 @@ class NameTableStore {
 					// We could re-try once more, but that bears the risk of an infinite loop.
 					// So let's just give up on the ID.
 					$dbw->onAtomicSectionCancel(
-						function ( $trigger, IDatabase $unused ) use ( $name, $id, $dbw ) {
+						function ( $trigger, IDatabase $unused ) {
 							$this->logger->warning(
 								'Re-insertion of name into table ' . $this->table
 								. ' was rolled back. Giving up and reloading the cache.'
