@@ -79,7 +79,7 @@ abstract class Action implements MessageLocalizer {
 	 * @param array $overrides
 	 * @return bool|null|string|callable|Action
 	 */
-	final private static function getClass(
+	private static function getClass(
 		string $action,
 		array $overrides
 	) {
@@ -398,6 +398,16 @@ abstract class Action implements MessageLocalizer {
 		return null;
 	}
 
+	/**
+	 * Indicates whether this action requires read rights
+	 * @since 1.35.5
+	 * @stable to override
+	 * @return bool
+	 */
+	public function needsReadRights() {
+		return true;
+	}
+
 	/**
 	 * Checks if the given user (identified by an object) can perform this action.  Can be
 	 * overridden by sub-classes with more complicated permissions schemes.  Failures here
