@@ -44,7 +44,7 @@ class CreditsAction extends FormlessAction {
 	 * @return string HTML
 	 */
 	public function onView() {
-		if ( $this->page->getID() == 0 ) {
+		if ( $this->getWikiPage()->getId() == 0 ) {
 			$s = $this->msg( 'nocredits' )->parse();
 		} else {
 			$s = $this->getCredits( -1 );
@@ -64,7 +64,7 @@ class CreditsAction extends FormlessAction {
 		$s = '';
 
 		if ( $cnt != 0 ) {
-			$s = $this->getAuthor( $this->page );
+			$s = $this->getAuthor();
 			if ( $cnt > 1 || $cnt < 0 ) {
 				$s .= ' ' . $this->getContributors( $cnt - 1, $showIfMax );
 			}
@@ -75,10 +75,11 @@ class CreditsAction extends FormlessAction {
 
 	/**
 	 * Get the last author with the last modification time
-	 * @param Page $page
+	 *
 	 * @return string HTML
 	 */
-	protected function getAuthor( Page $page ) {
+	private function getAuthor() {
+		$page = $this->getWikiPage();
 		$user = User::newFromName( $page->getUserText(), false );
 
 		$timestamp = $page->getTimestamp();
@@ -113,7 +114,7 @@ class CreditsAction extends FormlessAction {
 	 * @return string Html
 	 */
 	protected function getContributors( $cnt, $showIfMax ) {
-		$contributors = $this->page->getContributors();
+		$contributors = $this->getWikiPage()->getContributors();
 
 		$others_link = false;
 
@@ -222,12 +223,10 @@ class CreditsAction extends FormlessAction {
 		$link = $this->link( $user );
 		if ( $user->isAnon() ) {
 			return $this->msg( 'anonuser' )->rawParams( $link )->parse();
+		} elseif ( $this->canShowRealUserName() && $user->getRealName() ) {
+			return $link;
 		} else {
-			if ( $this->canShowRealUserName() && $user->getRealName() ) {
-				return $link;
-			} else {
-				return $this->msg( 'siteuser' )->rawParams( $link )->params( $user->getName() )->escaped();
-			}
+			return $this->msg( 'siteuser' )->rawParams( $link )->params( $user->getName() )->escaped();
 		}
 	}
 
