@@ -27,6 +27,8 @@
  */
 
 use MediaWiki\Logger\LoggerFactory;
+use MediaWiki\MediaWikiServices;
+use MediaWiki\Revision\SlotRecord;
 
 /**
  * A simple method to retrieve the plain source of an article,
@@ -47,34 +49,35 @@ class RawAction extends FormlessAction {
 		return false;
 	}
 
-	function onView() {
+	/**
+	 * @suppress SecurityCheck-XSS Non html mime type
+	 * @return string|null
+	 */
+	public function onView() {
 		$this->getOutput()->disable();
 		$request = $this->getRequest();
 		$response = $request->response();
 		$config = $this->context->getConfig();
 
-		if ( !$request->checkUrlExtension() ) {
-			return;
-		}
-
-		if ( $this->getOutput()->checkLastModified( $this->page->getTouched() ) ) {
-			return; // Client cache fresh and headers sent, nothing more to do.
-		}
-
-		$gen = $request->getVal( 'gen' );
-		if ( $gen == 'css' || $gen == 'js' ) {
-			$this->gen = true;
+		if ( $this->getOutput()->checkLastModified(
+			$this->getWikiPage()->getTouched()
+		) ) {
+			return null; // Client cache fresh and headers sent, nothing more to do.
 		}
 
 		$contentType = $this->getContentType();
 
-		$maxage = $request->getInt( 'maxage', $config->get( 'SquidMaxage' ) );
+		$maxage = $request->getInt( 'maxage', $config->get( 'CdnMaxAge' ) );
 		$smaxage = $request->getIntOrNull( 'smaxage' );
 		if ( $smaxage === null ) {
-			if ( $contentType == 'text/css' || $contentType == 'text/javascript' ) {
-				// CSS/JS raw content has its own CDN max age configuration.
-				// Note: Title::getCdnUrls() includes action=raw for css/js pages,
-				// so if using the canonical url, this will get HTCP purges.
+			if (
+				$contentType == 'text/css' ||
+				$contentType == 'application/json' ||
+				$contentType == 'text/javascript'
+			) {
+				// CSS/JSON/JS raw content has its own CDN max age configuration.
+				// Note: Title::getCdnUrls() includes action=raw for css/json/js
+				// pages, so if using the canonical url, this will get HTCP purges.
 				$smaxage = intval( $config->get( 'ForcedRawSMaxage' ) );
 			} else {
 				// No CDN cache for anything else
@@ -84,13 +87,11 @@ class RawAction extends FormlessAction {
 
 		// Set standard Vary headers so cache varies on cookies and such (T125283)
 		$response->header( $this->getOutput()->getVaryHeader() );
-		if ( $config->get( 'UseKeyHeader' ) ) {
-			$response->header( $this->getOutput()->getKeyHeader() );
-		}
 
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 		// Output may contain user-specific data;
 		// vary generated content for open sessions on private wikis
-		$privateCache = !User::isEveryoneAllowed( 'read' ) &&
+		$privateCache = !$permissionManager->isEveryoneAllowed( 'read' ) &&
 			( $smaxage == 0 || MediaWiki\Session\SessionManager::getGlobalSession()->isPersistent() );
 		// Don't accidentally cache cookies if user is logged in (T55032)
 		$privateCache = $privateCache || $this->getUser()->isLoggedIn();
@@ -104,19 +105,22 @@ class RawAction extends FormlessAction {
 		// someone could register the account and take control of the
 		// JS/CSS/Json page.
 		$title = $this->getTitle();
-		if ( $title->isCssJsSubpage() && $contentType !== 'text/x-wiki' ) {
+		if ( $title->isUserConfigPage() && $contentType !== 'text/x-wiki' ) {
 			// not using getRootText() as we want this to work
 			// even if subpages are disabled.
 			$rootPage = strtok( $title->getText(), '/' );
 			$userFromTitle = User::newFromName( $rootPage, 'usable' );
 			if ( !$userFromTitle || $userFromTitle->getId() === 0 ) {
+				$elevated = $permissionManager->userHasRight( $this->getUser(), 'editinterface' );
+				$elevatedText = $elevated ? 'by elevated ' : '';
 				$log = LoggerFactory::getInstance( "security" );
 				$log->warning(
-					"Unsafe JS/CSS/Json load - {user} loaded {title} with {ctype}",
+					"Unsafe JS/CSS/Json {$elevatedText}load - {user} loaded {title} with {ctype}",
 					[
 						'user' => $this->getUser()->getName(),
-						'title' => $title->getPrefixedDBKey(),
+						'title' => $title->getPrefixedDBkey(),
 						'ctype' => $contentType,
+						'elevated' => $elevated
 					]
 				);
 				$msg = wfMessage( 'unregistered-user-config' );
@@ -132,16 +136,17 @@ class RawAction extends FormlessAction {
 		// allowance.
 		if (
 			$contentType === 'text/javascript' &&
-			!$title->isJsSubpage() &&
+			!$title->isUserJsConfigPage() &&
 			!$title->inNamespace( NS_MEDIAWIKI ) &&
 			!in_array( 'sysop', $title->getRestrictions( 'edit' ) ) &&
 			!in_array( 'editprotected', $title->getRestrictions( 'edit' ) )
 		) {
+
 			$log = LoggerFactory::getInstance( "security" );
 			$log->info( "Blocked loading unprotected JS {title} for {user}",
 				[
 					'user' => $this->getUser()->getName(),
-					'title' => $title->getPrefixedDBKey(),
+					'title' => $title->getPrefixedDBkey(),
 				]
 			);
 			throw new HttpError( 403, wfMessage( 'unprotected-js' ) );
@@ -159,13 +164,13 @@ class RawAction extends FormlessAction {
 			$response->statusHeader( 404 );
 		}
 
-		// Avoid PHP 7.1 warning of passing $this by reference
-		$rawAction = $this;
-		if ( !Hooks::run( 'RawPageViewBeforeOutput', [ &$rawAction, &$text ] ) ) {
-			wfDebug( __METHOD__ . ": RawPageViewBeforeOutput hook broke raw page output.\n" );
+		if ( !$this->getHookRunner()->onRawPageViewBeforeOutput( $this, $text ) ) {
+			wfDebug( __METHOD__ . ": RawPageViewBeforeOutput hook broke raw page output." );
 		}
 
 		echo $text;
+
+		return null;
 	}
 
 	/**
@@ -175,59 +180,47 @@ class RawAction extends FormlessAction {
 	 * @return string|bool
 	 */
 	public function getRawText() {
-		global $wgParser;
-
 		$text = false;
 		$title = $this->getTitle();
 		$request = $this->getRequest();
 
-		// If it's a MediaWiki message we can just hit the message cache
-		if ( $request->getBool( 'usemsgcache' ) && $title->getNamespace() == NS_MEDIAWIKI ) {
-			// The first "true" is to use the database, the second is to use
-			// the content langue and the last one is to specify the message
-			// key already contains the language in it ("/de", etc.).
-			$text = MessageCache::singleton()->get( $title->getDBkey(), true, true, true );
-			// If the message doesn't exist, return a blank
-			if ( $text === false ) {
-				$text = '';
-			}
-		} else {
-			// Get it from the DB
-			$rev = Revision::newFromTitle( $title, $this->getOldId() );
-			if ( $rev ) {
-				$lastmod = wfTimestamp( TS_RFC2822, $rev->getTimestamp() );
-				$request->response()->header( "Last-modified: $lastmod" );
+		// Get it from the DB
+		$rev = MediaWikiServices::getInstance()
+			->getRevisionLookup()
+			->getRevisionByTitle( $title, $this->getOldId() );
+		if ( $rev ) {
+			$lastmod = wfTimestamp( TS_RFC2822, $rev->getTimestamp() );
+			$request->response()->header( "Last-modified: $lastmod" );
 
-				// Public-only due to cache headers
-				$content = $rev->getContent();
+			// Public-only due to cache headers
+			$content = $rev->getContent( SlotRecord::MAIN );
 
-				if ( $content === null ) {
-					// revision not found (or suppressed)
+			if ( $content === null ) {
+				// revision not found (or suppressed)
+				$text = false;
+			} elseif ( !$content instanceof TextContent ) {
+				// non-text content
+				wfHttpError( 415, "Unsupported Media Type", "The requested page uses the content model `"
+					. $content->getModel() . "` which is not supported via this interface." );
+				die();
+			} else {
+				// want a section?
+				$section = $request->getIntOrNull( 'section' );
+				if ( $section !== null ) {
+					$content = $content->getSection( $section );
+				}
+
+				if ( $content === null || $content === false ) {
+					// section not found (or section not supported, e.g. for JS, JSON, and CSS)
 					$text = false;
-				} elseif ( !$content instanceof TextContent ) {
-					// non-text content
-					wfHttpError( 415, "Unsupported Media Type", "The requested page uses the content model `"
-						. $content->getModel() . "` which is not supported via this interface." );
-					die();
 				} else {
-					// want a section?
-					$section = $request->getIntOrNull( 'section' );
-					if ( $section !== null ) {
-						$content = $content->getSection( $section );
-					}
-
-					if ( $content === null || $content === false ) {
-						// section not found (or section not supported, e.g. for JS and CSS)
-						$text = false;
-					} else {
-						$text = $content->getNativeData();
-					}
+					$text = $content->getText();
 				}
 			}
 		}
 
-		if ( $text !== false && $text !== '' && $request->getVal( 'templates' ) === 'expand' ) {
-			$text = $wgParser->preprocess(
+		if ( $text !== false && $text !== '' && $request->getRawVal( 'templates' ) === 'expand' ) {
+			$text = MediaWikiServices::getInstance()->getParser()->preprocess(
 				$text,
 				$title,
 				ParserOptions::newFromContext( $this->getContext() )
@@ -244,23 +237,31 @@ class RawAction extends FormlessAction {
 	 */
 	public function getOldId() {
 		$oldid = $this->getRequest()->getInt( 'oldid' );
+		$rl = MediaWikiServices::getInstance()->getRevisionLookup();
 		switch ( $this->getRequest()->getText( 'direction' ) ) {
 			case 'next':
 				# output next revision, or nothing if there isn't one
-				$nextid = 0;
+				$nextRev = null;
 				if ( $oldid ) {
-					$nextid = $this->getTitle()->getNextRevisionID( $oldid );
+					$oldRev = $rl->getRevisionById( $oldid );
+					if ( $oldRev ) {
+						$nextRev = $rl->getNextRevision( $oldRev );
+					}
 				}
-				$oldid = $nextid ?: -1;
+				$oldid = $nextRev ? $nextRev->getId() : -1;
 				break;
 			case 'prev':
 				# output previous revision, or nothing if there isn't one
+				$prevRev = null;
 				if ( !$oldid ) {
 					# get the current revision so we can get the penultimate one
-					$oldid = $this->page->getLatest();
+					$oldid = $this->getWikiPage()->getLatest();
 				}
-				$previd = $this->getTitle()->getPreviousRevisionID( $oldid );
-				$oldid = $previd ?: -1;
+				$oldRev = $rl->getRevisionById( $oldid );
+				if ( $oldRev ) {
+					$prevRev = $rl->getPreviousRevision( $oldRev );
+				}
+				$oldid = $prevRev ? $prevRev->getId() : -1;
 				break;
 			case 'cur':
 				$oldid = 0;
@@ -276,10 +277,12 @@ class RawAction extends FormlessAction {
 	 * @return string
 	 */
 	public function getContentType() {
-		$ctype = $this->getRequest()->getVal( 'ctype' );
+		// Optimisation: Avoid slow getVal(), this isn't user-generated content.
+		$ctype = $this->getRequest()->getRawVal( 'ctype' );
 
 		if ( $ctype == '' ) {
-			$gen = $this->getRequest()->getVal( 'gen' );
+			// Legacy compatibilty
+			$gen = $this->getRequest()->getRawVal( 'gen' );
 			if ( $gen == 'js' ) {
 				$ctype = 'text/javascript';
 			} elseif ( $gen == 'css' ) {
@@ -287,7 +290,14 @@ class RawAction extends FormlessAction {
 			}
 		}
 
-		$allowedCTypes = [ 'text/x-wiki', 'text/javascript', 'text/css', 'application/x-zope-edit' ];
+		$allowedCTypes = [
+			'text/x-wiki',
+			'text/javascript',
+			'text/css',
+			// FIXME: Should we still allow Zope editing? External editing feature was dropped
+			'application/x-zope-edit',
+			'application/json'
+		];
 		if ( $ctype == '' || !in_array( $ctype, $allowedCTypes ) ) {
 			$ctype = 'text/x-wiki';
 		}
