@@ -20,6 +20,8 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * API module that facilitates deleting pages. The API equivalent of action=delete.
  * Requires API write mode to be enabled.
@@ -27,6 +29,16 @@
  * @ingroup API
  */
 class ApiDelete extends ApiBase {
+
+	use ApiWatchlistTrait;
+
+	public function __construct( ApiMain $mainModule, $moduleName, $modulePrefix = '' ) {
+		parent::__construct( $mainModule, $moduleName, $modulePrefix );
+
+		$this->watchlistExpiryEnabled = $this->getConfig()->get( 'WatchlistExpiry' );
+		$this->watchlistMaxDuration = $this->getConfig()->get( 'WatchlistExpiryMaxDuration' );
+	}
+
 	/**
 	 * Extracts the title and reason from the request parameters and invokes
 	 * the local delete() function with these as arguments. It does not make use of
@@ -42,6 +54,7 @@ class ApiDelete extends ApiBase {
 		$pageObj = $this->getTitleOrPageId( $params, 'fromdbmaster' );
 		$titleObj = $pageObj->getTitle();
 		if ( !$pageObj->exists() &&
+			// @phan-suppress-next-line PhanUndeclaredMethod
 			!( $titleObj->getNamespace() == NS_FILE && self::canDeleteFile( $pageObj->getFile() ) )
 		) {
 			$this->dieWithError( 'apierror-missingtitle' );
@@ -75,9 +88,10 @@ class ApiDelete extends ApiBase {
 			$status = self::delete( $pageObj, $user, $reason, $params['tags'] );
 		}
 
-		if ( !$status->isGood() ) {
+		if ( !$status->isOK() ) {
 			$this->dieStatus( $status );
 		}
+		$this->addMessagesFromStatus( $status, [ 'warning' ], [ 'delete-scheduled' ] );
 
 		// Deprecated parameters
 		if ( $params['watch'] ) {
@@ -87,30 +101,39 @@ class ApiDelete extends ApiBase {
 		} else {
 			$watch = $params['watchlist'];
 		}
-		$this->setWatch( $watch, $titleObj, 'watchdeletion' );
+
+		$watchlistExpiry = $this->getExpiryFromParams( $params );
+		$this->setWatch( $watch, $titleObj, $user, 'watchdeletion', $watchlistExpiry );
 
 		$r = [
 			'title' => $titleObj->getPrefixedText(),
 			'reason' => $reason,
-			'logid' => $status->value
 		];
+
+		if ( $status->hasMessage( 'delete-scheduled' ) ) {
+			$r['scheduled'] = true;
+		}
+		if ( $status->value !== null ) {
+			// Scheduled deletions don't currently have a log entry available at this point
+			$r['logid'] = $status->value;
+		}
 		$this->getResult()->addValue( null, $this->getModuleName(), $r );
 	}
 
 	/**
 	 * We have our own delete() function, since Article.php's implementation is split in two phases
 	 *
-	 * @param Page|WikiPage $page Page or WikiPage object to work on
+	 * @param WikiPage $page WikiPage object to work on
 	 * @param User $user User doing the action
 	 * @param string|null &$reason Reason for the deletion. Autogenerated if null
 	 * @param array $tags Tags to tag the deletion with
 	 * @return Status
 	 */
-	protected static function delete( Page $page, User $user, &$reason = null, $tags = [] ) {
+	private static function delete( WikiPage $page, User $user, &$reason = null, $tags = [] ) {
 		$title = $page->getTitle();
 
 		// Auto-generate a summary, if necessary
-		if ( is_null( $reason ) ) {
+		if ( $reason === null ) {
 			// Need to pass a throwaway variable because generateReason expects
 			// a reference
 			$hasHistory = false;
@@ -123,8 +146,16 @@ class ApiDelete extends ApiBase {
 
 		$error = '';
 
-		// Luckily, Article.php provides a reusable delete function that does the hard work for us
-		return $page->doDeleteArticleReal( $reason, false, 0, true, $error, $user, $tags );
+		// Luckily, WikiPage provides a reusable delete function that does the hard work for us
+		return $page->doDeleteArticleReal(
+			$reason,
+			$user,
+			false, // don't suppress
+			null, // unused
+			$error,
+			null, // unused
+			$tags
+		);
 	}
 
 	/**
@@ -136,19 +167,20 @@ class ApiDelete extends ApiBase {
 	}
 
 	/**
-	 * @param Page $page Object to work on
+	 * @param WikiPage $page Object to work on
 	 * @param User $user User doing the action
 	 * @param string $oldimage Archive name
-	 * @param string &$reason Reason for the deletion. Autogenerated if null.
+	 * @param string|null &$reason Reason for the deletion. Autogenerated if null.
 	 * @param bool $suppress Whether to mark all deleted versions as restricted
 	 * @param array $tags Tags to tag the deletion with
 	 * @return Status
 	 */
-	protected static function deleteFile( Page $page, User $user, $oldimage,
+	private static function deleteFile( WikiPage $page, User $user, $oldimage,
 		&$reason = null, $suppress = false, $tags = []
 	) {
 		$title = $page->getTitle();
 
+		// @phan-suppress-next-line PhanUndeclaredMethod There's no right typehint for it
 		$file = $page->getFile();
 		if ( !self::canDeleteFile( $file ) ) {
 			return self::delete( $page, $user, $reason, $tags );
@@ -158,13 +190,14 @@ class ApiDelete extends ApiBase {
 			if ( !FileDeleteForm::isValidOldSpec( $oldimage ) ) {
 				return Status::newFatal( 'invalidoldimage' );
 			}
-			$oldfile = RepoGroup::singleton()->getLocalRepo()->newFromArchiveName( $title, $oldimage );
+			$oldfile = MediaWikiServices::getInstance()->getRepoGroup()
+				->getLocalRepo()->newFromArchiveName( $title, $oldimage );
 			if ( !$oldfile->exists() || !$oldfile->isLocal() || $oldfile->getRedirected() ) {
 				return Status::newFatal( 'nodeleteablefile' );
 			}
 		}
 
-		if ( is_null( $reason ) ) { // Log and RC don't like null reasons
+		if ( $reason === null ) { // Log and RC don't like null reasons
 			$reason = '';
 		}
 
@@ -180,7 +213,7 @@ class ApiDelete extends ApiBase {
 	}
 
 	public function getAllowedParams() {
-		return [
+		$params = [
 			'title' => null,
 			'pageid' => [
 				ApiBase::PARAM_TYPE => 'integer'
@@ -194,15 +227,13 @@ class ApiDelete extends ApiBase {
 				ApiBase::PARAM_DFLT => false,
 				ApiBase::PARAM_DEPRECATED => true,
 			],
-			'watchlist' => [
-				ApiBase::PARAM_DFLT => 'preferences',
-				ApiBase::PARAM_TYPE => [
-					'watch',
-					'unwatch',
-					'preferences',
-					'nochange'
-				],
-			],
+		];
+
+		// Params appear in the docs in the order they are defined,
+		// which is why this is here and not at the bottom.
+		$params += $this->getWatchlistParams();
+
+		return $params + [
 			'unwatch' => [
 				ApiBase::PARAM_DFLT => false,
 				ApiBase::PARAM_DEPRECATED => true,
