@@ -20,6 +20,8 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * Query module to enumerate links from all pages together.
  *
@@ -90,7 +92,7 @@ class ApiQueryAllLinks extends ApiQueryGeneratorBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 * @return void
 	 */
 	private function run( $resultPageSet = null ) {
@@ -129,7 +131,7 @@ class ApiQueryAllLinks extends ApiQueryGeneratorBase {
 			$this->addWhereFld( $pfx . 'namespace', $namespace );
 		}
 
-		$continue = !is_null( $params['continue'] );
+		$continue = $params['continue'] !== null;
 		if ( $continue ) {
 			$continueArr = explode( '|', $params['continue'] );
 			$op = $params['dir'] == 'descending' ? '<' : '>';
@@ -183,6 +185,20 @@ class ApiQueryAllLinks extends ApiQueryGeneratorBase {
 
 		$res = $this->select( __METHOD__ );
 
+		// Get gender information
+		if ( $res->numRows() && $resultPageSet === null ) {
+			$services = MediaWikiServices::getInstance();
+			if ( $services->getNamespaceInfo()->hasGenderDistinction( $namespace ) ) {
+				$users = [];
+				foreach ( $res as $row ) {
+					$users[] = $row->pl_title;
+				}
+				if ( $users !== [] ) {
+					$services->getGenderCache()->doQuery( $users, __METHOD__ );
+				}
+			}
+		}
+
 		$pageids = [];
 		$titles = [];
 		$count = 0;
@@ -199,7 +215,7 @@ class ApiQueryAllLinks extends ApiQueryGeneratorBase {
 				break;
 			}
 
-			if ( is_null( $resultPageSet ) ) {
+			if ( $resultPageSet === null ) {
 				$vals = [
 					ApiResult::META_TYPE => 'assoc',
 				];
@@ -231,7 +247,7 @@ class ApiQueryAllLinks extends ApiQueryGeneratorBase {
 			}
 		}
 
-		if ( is_null( $resultPageSet ) ) {
+		if ( $resultPageSet === null ) {
 			$result->addIndexedTagName( [ 'query', $this->getModuleName() ], $this->indexTag );
 		} elseif ( $params['unique'] ) {
 			$resultPageSet->populateFromTitles( $titles );
