@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Dec 1, 2007
- *
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +20,8 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * A query action to return messages from site message cache
  *
@@ -31,26 +29,28 @@
  */
 class ApiQueryAllMessages extends ApiQueryBase {
 
-	public function __construct( $query, $moduleName ) {
+	public function __construct( ApiQuery $query, $moduleName ) {
 		parent::__construct( $query, $moduleName, 'am' );
 	}
 
 	public function execute() {
 		$params = $this->extractRequestParams();
-
-		if ( is_null( $params['lang'] ) ) {
+		$services = MediaWikiServices::getInstance();
+		if ( $params['lang'] === null ) {
 			$langObj = $this->getLanguage();
-		} elseif ( !Language::isValidCode( $params['lang'] ) ) {
-			$this->dieUsage( 'Invalid language code for parameter lang', 'invalidlang' );
+		} elseif ( !$services->getLanguageNameUtils()->isValidCode( $params['lang'] ) ) {
+			$this->dieWithError(
+				[ 'apierror-invalidlang', $this->encodeParamName( 'lang' ) ], 'invalidlang'
+			);
 		} else {
-			$langObj = Language::factory( $params['lang'] );
+			$langObj = $services->getLanguageFactory()->getLanguage( $params['lang'] );
 		}
 
 		if ( $params['enableparser'] ) {
-			if ( !is_null( $params['title'] ) ) {
+			if ( $params['title'] !== null ) {
 				$title = Title::newFromText( $params['title'] );
 				if ( !$title || $title->isExternal() ) {
-					$this->dieUsageMsg( array( 'invalidtitle', $params['title'] ) );
+					$this->dieWithError( [ 'apierror-invalidtitle', wfEscapeWikiText( $params['title'] ) ] );
 				}
 			} else {
 				$title = Title::newFromText( 'API' );
@@ -61,16 +61,17 @@ class ApiQueryAllMessages extends ApiQueryBase {
 
 		// Determine which messages should we print
 		if ( in_array( '*', $params['messages'] ) ) {
-			$message_names = Language::getMessageKeysFor( $langObj->getCode() );
+			$message_names = $services->getLocalisationCache()
+				->getSubitemList( $langObj->getCode(), 'messages' );
 			if ( $params['includelocal'] ) {
-				global $wgLanguageCode;
 				$message_names = array_unique( array_merge(
 					$message_names,
 					// Pass in the content language code so we get local messages that have a
 					// MediaWiki:msgkey page. We might theoretically miss messages that have no
 					// MediaWiki:msgkey page but do have a MediaWiki:msgkey/lang page, but that's
 					// just a stupid case.
-					MessageCache::singleton()->getAllMessageKeys( $wgLanguageCode )
+					$services->getMessageCache()
+						->getAllMessageKeys( $this->getConfig()->get( 'LanguageCode' ) )
 				) );
 			}
 			sort( $message_names );
@@ -83,7 +84,7 @@ class ApiQueryAllMessages extends ApiQueryBase {
 		// Because we sorted the message array earlier, they will appear in a clump:
 		if ( isset( $params['prefix'] ) ) {
 			$skip = false;
-			$messages_filtered = array();
+			$messages_filtered = [];
 			foreach ( $messages_target as $message ) {
 				// === 0: must be at beginning of string (position 0)
 				if ( strpos( $message, $params['prefix'] ) === 0 ) {
@@ -100,7 +101,7 @@ class ApiQueryAllMessages extends ApiQueryBase {
 
 		// Filter messages that contain specified string
 		if ( isset( $params['filter'] ) ) {
-			$messages_filtered = array();
+			$messages_filtered = [];
 			foreach ( $messages_target as $message ) {
 				// !== is used because filter can be at the beginning of the string
 				if ( strpos( $message, $params['filter'] ) !== false ) {
@@ -113,18 +114,21 @@ class ApiQueryAllMessages extends ApiQueryBase {
 		// Whether we have any sort of message customisation filtering
 		$customiseFilterEnabled = $params['customised'] !== 'all';
 		if ( $customiseFilterEnabled ) {
-			global $wgContLang;
-			$lang = $langObj->getCode();
-
-			$customisedMessages = AllmessagesTablePager::getCustomisedStatuses(
-				array_map( array( $langObj, 'ucfirst' ), $messages_target ), $lang, $lang != $wgContLang->getCode() );
+			$customisedMessages = AllMessagesTablePager::getCustomisedStatuses(
+				array_map(
+					[ $langObj, 'ucfirst' ],
+					$messages_target
+				),
+				$langObj->getCode(),
+				!$langObj->equals( MediaWikiServices::getInstance()->getContentLanguage() )
+			);
 
 			$customised = $params['customised'] === 'modified';
 		}
 
 		// Get all requested messages and print the result
-		$skip = !is_null( $params['from'] );
-		$useto = !is_null( $params['to'] );
+		$skip = $params['from'] !== null;
+		$useto = $params['to'] !== null;
 		$result = $this->getResult();
 		foreach ( $messages_target as $message ) {
 			// Skip all messages up to $params['from']
@@ -137,8 +141,12 @@ class ApiQueryAllMessages extends ApiQueryBase {
 			}
 
 			if ( !$skip ) {
-				$a = array( 'name' => $message );
-				$args = array();
+				$a = [
+					'name' => $message,
+					'normalizedname' => MessageCache::normalizeKey( $message ),
+				];
+
+				$args = [];
 				if ( isset( $params['args'] ) && count( $params['args'] ) != 0 ) {
 					$args = $params['args'];
 				}
@@ -147,7 +155,7 @@ class ApiQueryAllMessages extends ApiQueryBase {
 					$messageIsCustomised = isset( $customisedMessages['pages'][$langObj->ucfirst( $message )] );
 					if ( $customised === $messageIsCustomised ) {
 						if ( $customised ) {
-							$a['customised'] = '';
+							$a['customised'] = true;
 						}
 					} else {
 						continue;
@@ -157,7 +165,7 @@ class ApiQueryAllMessages extends ApiQueryBase {
 				$msg = wfMessage( $message, $args )->inLanguage( $langObj );
 
 				if ( !$msg->exists() ) {
-					$a['missing'] = '';
+					$a['missing'] = true;
 				} else {
 					// Check if the parser is enabled:
 					if ( $params['enableparser'] ) {
@@ -166,29 +174,29 @@ class ApiQueryAllMessages extends ApiQueryBase {
 						$msgString = $msg->plain();
 					}
 					if ( !$params['nocontent'] ) {
-						ApiResult::setContent( $a, $msgString );
+						ApiResult::setContentValue( $a, 'content', $msgString );
 					}
 					if ( isset( $prop['default'] ) ) {
 						$default = wfMessage( $message )->inLanguage( $langObj )->useDatabase( false );
 						if ( !$default->exists() ) {
-							$a['defaultmissing'] = '';
+							$a['defaultmissing'] = true;
 						} elseif ( $default->plain() != $msgString ) {
 							$a['default'] = $default->plain();
 						}
 					}
 				}
-				$fit = $result->addValue( array( 'query', $this->getModuleName() ), null, $a );
+				$fit = $result->addValue( [ 'query', $this->getModuleName() ], null, $a );
 				if ( !$fit ) {
 					$this->setContinueEnumParameter( 'from', $message );
 					break;
 				}
 			}
 		}
-		$result->setIndexedTagName_internal( array( 'query', $this->getModuleName() ), 'message' );
+		$result->addIndexedTagName( [ 'query', $this->getModuleName() ], 'message' );
 	}
 
 	public function getCacheMode( $params ) {
-		if ( is_null( $params['lang'] ) ) {
+		if ( $params['lang'] === null ) {
 			// Language not specified, will be fetched from preferences
 			return 'anon-public-user-private';
 		} elseif ( $params['enableparser'] ) {
@@ -201,101 +209,51 @@ class ApiQueryAllMessages extends ApiQueryBase {
 	}
 
 	public function getAllowedParams() {
-		return array(
-			'messages' => array(
+		return [
+			'messages' => [
 				ApiBase::PARAM_DFLT => '*',
 				ApiBase::PARAM_ISMULTI => true,
-			),
-			'prop' => array(
+			],
+			'prop' => [
 				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => array(
+				ApiBase::PARAM_TYPE => [
 					'default'
-				)
-			),
+				]
+			],
 			'enableparser' => false,
 			'nocontent' => false,
 			'includelocal' => false,
-			'args' => array(
+			'args' => [
 				ApiBase::PARAM_ISMULTI => true,
 				ApiBase::PARAM_ALLOW_DUPLICATES => true,
-			),
-			'filter' => array(),
-			'customised' => array(
+			],
+			'filter' => [],
+			'customised' => [
 				ApiBase::PARAM_DFLT => 'all',
-				ApiBase::PARAM_TYPE => array(
+				ApiBase::PARAM_TYPE => [
 					'all',
 					'modified',
 					'unmodified'
-				)
-			),
+				]
+			],
 			'lang' => null,
 			'from' => null,
 			'to' => null,
 			'title' => null,
 			'prefix' => null,
-		);
+		];
 	}
 
-	public function getParamDescription() {
-		return array(
-			'messages' => 'Which messages to output. "*" (default) means all messages',
-			'prop' => 'Which properties to get',
-			'enableparser' => array( 'Set to enable parser, will preprocess the wikitext of message',
-				'Will substitute magic words, handle templates etc.' ),
-			'nocontent' => 'If set, do not include the content of the messages in the output.',
-			'includelocal' => array( "Also include local messages, i.e. messages that don't exist in the software but do exist as a MediaWiki: page.",
-				"This lists all MediaWiki: pages, so it will also list those that aren't 'really' messages such as Common.js",
-			),
-			'title' => 'Page name to use as context when parsing message (for enableparser option)',
-			'args' => 'Arguments to be substituted into message',
-			'prefix' => 'Return messages with this prefix',
-			'filter' => 'Return only messages with names that contain this string',
-			'customised' => 'Return only messages in this customisation state',
-			'lang' => 'Return messages in this language',
-			'from' => 'Return messages starting at this message',
-			'to' => 'Return messages ending at this message',
-		);
-	}
-
-	public function getPossibleErrors() {
-		return array_merge( parent::getPossibleErrors(), array(
-			array( 'code' => 'invalidlang', 'info' => 'Invalid language code for parameter lang' ),
-		) );
-	}
-
-	public function getResultProperties() {
-		return array(
-			'' => array(
-				'name' => 'string',
-				'customised' => 'boolean',
-				'missing' => 'boolean',
-				'*' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				)
-			),
-			'default' => array(
-				'defaultmissing' => 'boolean',
-				'default' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				)
-			)
-		);
-	}
-
-	public function getDescription() {
-		return 'Return messages from this site.';
-	}
-
-	public function getExamples() {
-		return array(
-			'api.php?action=query&meta=allmessages&amprefix=ipb-',
-			'api.php?action=query&meta=allmessages&ammessages=august|mainpage&amlang=de',
-		);
+	protected function getExamplesMessages() {
+		return [
+			'action=query&meta=allmessages&amprefix=ipb-'
+				=> 'apihelp-query+allmessages-example-ipb',
+			'action=query&meta=allmessages&ammessages=august|mainpage&amlang=de'
+				=> 'apihelp-query+allmessages-example-de',
+		];
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Meta#allmessages_.2F_am';
+		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Allmessages';
 	}
 }
