@@ -20,6 +20,10 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+use MediaWiki\ParamValidator\TypeDef\UserDef;
+use MediaWiki\Revision\RevisionRecord;
+
 /**
  * Query module to enumerate all revisions.
  *
@@ -33,17 +37,30 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 * @return void
 	 */
 	protected function run( ApiPageSet $resultPageSet = null ) {
 		$db = $this->getDB();
 		$params = $this->extractRequestParams( false );
+		$services = MediaWikiServices::getInstance();
+		$revisionStore = $services->getRevisionStore();
 
 		$result = $this->getResult();
 
 		$this->requireMaxOneParameter( $params, 'user', 'excludeuser' );
 
+		$tsField = 'rev_timestamp';
+		$idField = 'rev_id';
+		$pageField = 'rev_page';
+		if ( $params['user'] !== null ) {
+			// The query is probably best done using the actor_timestamp index on
+			// revision_actor_temp. Use the denormalized fields from that table.
+			$tsField = 'revactor_timestamp';
+			$idField = 'revactor_rev';
+			$pageField = 'revactor_page';
+		}
+
 		// Namespace check is likely to be desired, but can't be done
 		// efficiently in SQL.
 		$miser_ns = null;
@@ -51,7 +68,7 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 		if ( $params['namespace'] !== null ) {
 			$params['namespace'] = array_unique( $params['namespace'] );
 			sort( $params['namespace'] );
-			if ( $params['namespace'] != MWNamespace::getValidNamespaces() ) {
+			if ( $params['namespace'] != $services->getNamespaceInfo()->getValidNamespaces() ) {
 				$needPageTable = true;
 				if ( $this->getConfig()->get( 'MiserMode' ) ) {
 					$miser_ns = $params['namespace'];
@@ -63,66 +80,83 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 
 		if ( $resultPageSet === null ) {
 			$this->parseParameters( $params );
-			$revQuery = Revision::getQueryInfo(
-				$this->fetchContent ? [ 'page', 'text' ] : [ 'page' ]
-			);
-			$this->addTables( $revQuery['tables'] );
-			$this->addFields( $revQuery['fields'] );
-			$this->addJoinConds( $revQuery['joins'] );
-
-			// Review this depeneding on the outcome of T113901
-			$this->addOption( 'STRAIGHT_JOIN' );
+			$revQuery = $revisionStore->getQueryInfo( [ 'page' ] );
 		} else {
 			$this->limit = $this->getParameter( 'limit' ) ?: 10;
-			$this->addTables( 'revision' );
-			$this->addFields( [ 'rev_timestamp', 'rev_id' ] );
+			$revQuery = [
+				'tables' => [ 'revision' ],
+				'fields' => [ 'rev_timestamp', 'rev_id' ],
+				'joins' => [],
+			];
+
 			if ( $params['generatetitles'] ) {
-				$this->addFields( [ 'rev_page' ] );
+				$revQuery['fields'][] = 'rev_page';
+			}
+
+			if ( $params['user'] !== null || $params['excludeuser'] !== null ) {
+				$actorQuery = ActorMigration::newMigration()->getJoin( 'rev_user' );
+				$revQuery['tables'] += $actorQuery['tables'];
+				$revQuery['joins'] += $actorQuery['joins'];
 			}
 
 			if ( $needPageTable ) {
-				$this->addTables( 'page' );
-				$this->addJoinConds(
-					[ 'page' => [ 'INNER JOIN', [ 'rev_page = page_id' ] ] ]
-				);
-				$this->addFieldsIf( [ 'page_namespace' ], (bool)$miser_ns );
-
-				// Review this depeneding on the outcome of T113901
-				$this->addOption( 'STRAIGHT_JOIN' );
+				$revQuery['tables'][] = 'page';
+				$revQuery['joins']['page'] = [ 'JOIN', [ "$pageField = page_id" ] ];
+				if ( (bool)$miser_ns ) {
+					$revQuery['fields'][] = 'page_namespace';
+				}
 			}
 		}
 
+		// If we're going to be using actor_timestamp, we need to swap the order of `revision`
+		// and `revision_actor_temp` in the query (for the straight join) and adjust some field aliases.
+		if ( $idField !== 'rev_id' && isset( $revQuery['tables']['temp_rev_user'] ) ) {
+			$aliasFields = [ 'rev_id' => $idField, 'rev_timestamp' => $tsField, 'rev_page' => $pageField ];
+			$revQuery['fields'] = array_merge(
+				$aliasFields,
+				array_diff( $revQuery['fields'], array_keys( $aliasFields ) )
+			);
+			unset( $revQuery['tables']['temp_rev_user'] );
+			$revQuery['tables'] = array_merge(
+				[ 'temp_rev_user' => 'revision_actor_temp' ],
+				$revQuery['tables']
+			);
+			$revQuery['joins']['revision'] = $revQuery['joins']['temp_rev_user'];
+			unset( $revQuery['joins']['temp_rev_user'] );
+		}
+
+		$this->addTables( $revQuery['tables'] );
+		$this->addFields( $revQuery['fields'] );
+		$this->addJoinConds( $revQuery['joins'] );
+
+		// Seems to be needed to avoid a planner bug (T113901)
+		$this->addOption( 'STRAIGHT_JOIN' );
+
 		$dir = $params['dir'];
-		$this->addTimestampWhereRange( 'rev_timestamp', $dir, $params['start'], $params['end'] );
+		$this->addTimestampWhereRange( $tsField, $dir, $params['start'], $params['end'] );
 
 		if ( $this->fld_tags ) {
-			$this->addTables( 'tag_summary' );
-			$this->addJoinConds(
-				[ 'tag_summary' => [ 'LEFT JOIN', [ 'rev_id=ts_rev_id' ] ] ]
-			);
-			$this->addFields( 'ts_tags' );
+			$this->addFields( [ 'ts_tags' => ChangeTags::makeTagSummarySubquery( 'revision' ) ] );
 		}
 
 		if ( $params['user'] !== null ) {
 			$actorQuery = ActorMigration::newMigration()
-				->getWhere( $db, 'rev_user', User::newFromName( $params['user'], false ) );
-			$this->addTables( $actorQuery['tables'] );
-			$this->addJoinConds( $actorQuery['joins'] );
+				->getWhere( $db, 'rev_user', $params['user'] );
 			$this->addWhere( $actorQuery['conds'] );
 		} elseif ( $params['excludeuser'] !== null ) {
 			$actorQuery = ActorMigration::newMigration()
-				->getWhere( $db, 'rev_user', User::newFromName( $params['excludeuser'], false ) );
-			$this->addTables( $actorQuery['tables'] );
-			$this->addJoinConds( $actorQuery['joins'] );
+				->getWhere( $db, 'rev_user', $params['excludeuser'] );
 			$this->addWhere( 'NOT(' . $actorQuery['conds'] . ')' );
 		}
 
 		if ( $params['user'] !== null || $params['excludeuser'] !== null ) {
 			// Paranoia: avoid brute force searches (T19342)
-			if ( !$this->getUser()->isAllowed( 'deletedhistory' ) ) {
-				$bitmask = Revision::DELETED_USER;
-			} elseif ( !$this->getUser()->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
-				$bitmask = Revision::DELETED_USER | Revision::DELETED_RESTRICTED;
+			if ( !$this->getPermissionManager()->userHasRight( $this->getUser(), 'deletedhistory' ) ) {
+				$bitmask = RevisionRecord::DELETED_USER;
+			} elseif ( !$this->getPermissionManager()
+				->userHasAnyRight( $this->getUser(), 'suppressrevision', 'viewsuppressed' )
+			) {
+				$bitmask = RevisionRecord::DELETED_USER | RevisionRecord::DELETED_RESTRICTED;
 			} else {
 				$bitmask = 0;
 			}
@@ -138,23 +172,28 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 			$ts = $db->addQuotes( $db->timestamp( $cont[0] ) );
 			$rev_id = (int)$cont[1];
 			$this->dieContinueUsageIf( strval( $rev_id ) !== $cont[1] );
-			$this->addWhere( "rev_timestamp $op $ts OR " .
-				"(rev_timestamp = $ts AND " .
-				"rev_id $op= $rev_id)" );
+			$this->addWhere( "$tsField $op $ts OR " .
+				"($tsField = $ts AND " .
+				"$idField $op= $rev_id)" );
 		}
 
 		$this->addOption( 'LIMIT', $this->limit + 1 );
 
 		$sort = ( $dir == 'newer' ? '' : ' DESC' );
 		$orderby = [];
-		// Targeting index rev_timestamp, user_timestamp, or usertext_timestamp
-		// But 'user' is always constant for the latter two, so it doesn't matter here.
+		// Targeting index rev_timestamp, user_timestamp, usertext_timestamp, or actor_timestamp.
+		// But 'user' is always constant for the latter three, so it doesn't matter here.
 		$orderby[] = "rev_timestamp $sort";
 		$orderby[] = "rev_id $sort";
 		$this->addOption( 'ORDER BY', $orderby );
 
 		$hookData = [];
 		$res = $this->select( __METHOD__, [], $hookData );
+
+		if ( $resultPageSet === null ) {
+			$this->executeGenderCacheFromResultWrapper( $res, __METHOD__ );
+		}
+
 		$pageMap = []; // Maps rev_page to array index
 		$count = 0;
 		$nextIndex = 0;
@@ -185,13 +224,13 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 					$generated[] = $row->rev_id;
 				}
 			} else {
-				$revision = Revision::newFromRow( $row );
+				$revision = $revisionStore->newRevisionFromRow( $row, 0, Title::newFromRow( $row ) );
 				$rev = $this->extractRevisionInfo( $revision, $row );
 
 				if ( !isset( $pageMap[$row->rev_page] ) ) {
 					$index = $nextIndex++;
 					$pageMap[$row->rev_page] = $index;
-					$title = $revision->getTitle();
+					$title = Title::newFromLinkTarget( $revision->getPageAsLinkTarget() );
 					$a = [
 						'pageid' => $title->getArticleID(),
 						'revisions' => [ $rev ],
@@ -227,6 +266,8 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 		$ret = parent::getAllowedParams() + [
 			'user' => [
 				ApiBase::PARAM_TYPE => 'user',
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_RETURN_OBJECT => true,
 			],
 			'namespace' => [
 				ApiBase::PARAM_ISMULTI => true,
@@ -249,6 +290,8 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 			],
 			'excludeuser' => [
 				ApiBase::PARAM_TYPE => 'user',
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_RETURN_OBJECT => true,
 			],
 			'continue' => [
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
@@ -272,7 +315,7 @@ class ApiQueryAllRevisions extends ApiQueryRevisionsBase {
 			'action=query&list=allrevisions&arvuser=Example&arvlimit=50'
 				=> 'apihelp-query+allrevisions-example-user',
 			'action=query&list=allrevisions&arvdir=newer&arvlimit=50'
-				=> 'apihelp-query+allrevisions-example-ns-main',
+				=> 'apihelp-query+allrevisions-example-ns-any',
 		];
 	}
 
