@@ -20,6 +20,9 @@
  * @file
  */
 
+use Wikimedia\ParamValidator\ParamValidator;
+use Wikimedia\ParamValidator\TypeDef\IntegerDef;
+
 /**
  * This is a three-in-one module to query:
  *   * backlinks  - links pointing to the given page,
@@ -35,9 +38,15 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 	 */
 	private $rootTitle;
 
-	private $params, $cont, $redirect;
+	private $params;
+	/** @var array */
+	private $cont;
+	private $redirect;
 	private $bl_ns, $bl_from, $bl_from_ns, $bl_table, $bl_code, $bl_title, $bl_fields, $hasNS;
 
+	/** @var string */
+	private $helpUrl;
+
 	/**
 	 * Maps ns and title to pageid
 	 *
@@ -113,13 +122,13 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 * @return void
 	 */
 	private function runFirstQuery( $resultPageSet = null ) {
 		$this->addTables( [ $this->bl_table, 'page' ] );
 		$this->addWhere( "{$this->bl_from}=page_id" );
-		if ( is_null( $resultPageSet ) ) {
+		if ( $resultPageSet === null ) {
 			$this->addFields( [ 'page_id', 'page_title', 'page_namespace' ] );
 		} else {
 			$this->addFields( $resultPageSet->getPageTableFields() );
@@ -164,6 +173,11 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 		$this->addOption( 'STRAIGHT_JOIN' );
 
 		$res = $this->select( __METHOD__ );
+
+		if ( $resultPageSet === null ) {
+			$this->executeGenderCacheFromResultWrapper( $res, __METHOD__ );
+		}
+
 		$count = 0;
 		foreach ( $res as $row ) {
 			if ( ++$count > $this->params['limit'] ) {
@@ -186,8 +200,8 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 				$this->redirTitles[] = $t;
 			}
 
-			if ( is_null( $resultPageSet ) ) {
-				$a = [ 'pageid' => intval( $row->page_id ) ];
+			if ( $resultPageSet === null ) {
+				$a = [ 'pageid' => (int)$row->page_id ];
 				ApiQueryBase::addTitleInfo( $a, $t );
 				if ( $row->page_is_redirect ) {
 					$a['redirect'] = true;
@@ -201,7 +215,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 * @return void
 	 */
 	private function runSecondQuery( $resultPageSet = null ) {
@@ -209,7 +223,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 		$this->addTables( [ $this->bl_table, 'page' ] );
 		$this->addWhere( "{$this->bl_from}=page_id" );
 
-		if ( is_null( $resultPageSet ) ) {
+		if ( $resultPageSet === null ) {
 			$this->addFields( [ 'page_id', 'page_title', 'page_namespace', 'page_is_redirect' ] );
 		} else {
 			$this->addFields( $resultPageSet->getPageTableFields() );
@@ -284,6 +298,11 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 		$this->addOption( 'STRAIGHT_JOIN' );
 
 		$res = $this->select( __METHOD__ );
+
+		if ( $resultPageSet === null ) {
+			$this->executeGenderCacheFromResultWrapper( $res, __METHOD__ );
+		}
+
 		$count = 0;
 		foreach ( $res as $row ) {
 			$ns = $this->hasNS ? $row->{$this->bl_ns} : NS_FILE;
@@ -307,8 +326,8 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 				$this->cont[] = $row->page_id;
 			}
 
-			if ( is_null( $resultPageSet ) ) {
-				$a['pageid'] = intval( $row->page_id );
+			if ( $resultPageSet === null ) {
+				$a = [ 'pageid' => (int)$row->page_id ];
 				ApiQueryBase::addTitleInfo( $a, Title::makeTitle( $row->page_namespace, $row->page_title ) );
 				if ( $row->page_is_redirect ) {
 					$a['redirect'] = true;
@@ -323,7 +342,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 * @return void
 	 */
 	private function run( $resultPageSet = null ) {
@@ -338,8 +357,15 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 			$this->params['limit'] = $this->getMain()->canApiHighLimits() ? $botMax : $userMax;
 			$result->addParsedLimit( $this->getModuleName(), $this->params['limit'] );
 		} else {
-			$this->params['limit'] = intval( $this->params['limit'] );
-			$this->validateLimit( 'limit', $this->params['limit'], 1, $userMax, $botMax );
+			$this->params['limit'] = $this->getMain()->getParamValidator()->validateValue(
+				$this, 'limit', (int)$this->params['limit'], [
+					ParamValidator::PARAM_TYPE => 'limit',
+					IntegerDef::PARAM_MIN => 1,
+					IntegerDef::PARAM_MAX => $userMax,
+					IntegerDef::PARAM_MAX2 => $botMax,
+					IntegerDef::PARAM_IGNORE_RANGE => true,
+				]
+			);
 		}
 
 		$this->rootTitle = $this->getTitleFromTitleOrPageId( $this->params );
@@ -402,6 +428,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 					break;
 
 				default:
+					// @phan-suppress-next-line PhanImpossibleCondition
 					$this->dieContinueUsageIf( true );
 			}
 
@@ -417,10 +444,10 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 		// Fill in any missing fields in case it's needed below
 		$this->cont += [ 0, 0, 0, '', 0, 0, 0 ];
 
-		if ( is_null( $resultPageSet ) ) {
+		if ( $resultPageSet === null ) {
 			// Try to add the result data in one go and pray that it fits
 			$code = $this->bl_code;
-			$data = array_map( function ( $arr ) use ( $result, $code ) {
+			$data = array_map( function ( $arr ) use ( $code ) {
 				if ( isset( $arr['redirlinks'] ) ) {
 					$arr['redirlinks'] = array_values( $arr['redirlinks'] );
 					ApiResult::setIndexedTagName( $arr['redirlinks'], $code );
@@ -432,6 +459,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 				// It didn't fit. Add elements one by one until the
 				// result is full.
 				ksort( $this->resultArr );
+				// @phan-suppress-next-line PhanSuspiciousValueComparison
 				if ( count( $this->cont ) >= 7 ) {
 					$startAt = $this->cont[6];
 				} else {
@@ -457,6 +485,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 					$hasRedirs = false;
 					$redirLinks = isset( $arr['redirlinks'] ) ? (array)$arr['redirlinks'] : [];
 					ksort( $redirLinks );
+					// @phan-suppress-next-line PhanSuspiciousValueComparisonInLoop
 					if ( count( $this->cont ) >= 8 && $pageID == $startAt ) {
 						$redirStartAt = $this->cont[7];
 					} else {
@@ -496,7 +525,7 @@ class ApiQueryBacklinks extends ApiQueryGeneratorBase {
 				$this->bl_code
 			);
 		}
-		if ( !is_null( $this->continueStr ) ) {
+		if ( $this->continueStr !== null ) {
 			$this->setContinueEnumParameter( 'continue', $this->continueStr );
 		}
 	}
