@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Sep 27, 2008
- *
  * Copyright © 2008 Roan Kattouw "<Firstname>.<Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +20,8 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * A query module to list duplicates of the given file(s)
  *
@@ -48,7 +46,7 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 	}
 
 	/**
-	 * @param ApiPageSet $resultPageSet
+	 * @param ApiPageSet|null $resultPageSet
 	 */
 	private function run( $resultPageSet = null ) {
 		$params = $this->extractRequestParams();
@@ -79,10 +77,11 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 		}
 
 		$filesToFind = array_keys( $images );
+		$repoGroup = MediaWikiServices::getInstance()->getRepoGroup();
 		if ( $params['localonly'] ) {
-			$files = RepoGroup::singleton()->getLocalRepo()->findFiles( $filesToFind );
+			$files = $repoGroup->getLocalRepo()->findFiles( $filesToFind );
 		} else {
-			$files = RepoGroup::singleton()->findFiles( $filesToFind );
+			$files = $repoGroup->findFiles( $filesToFind );
 		}
 
 		$fit = true;
@@ -91,17 +90,17 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 
 		$sha1s = [];
 		foreach ( $files as $file ) {
-			/** @var $file File */
+			/** @var File $file */
 			$sha1s[$file->getName()] = $file->getSha1();
 		}
 
 		// find all files with the hashes, result format is:
-		// array( hash => array( dup1, dup2 ), hash1 => ... )
+		// [ hash => [ dup1, dup2 ], hash1 => ... ]
 		$filesToFindBySha1s = array_unique( array_values( $sha1s ) );
 		if ( $params['localonly'] ) {
-			$filesBySha1s = RepoGroup::singleton()->getLocalRepo()->findBySha1s( $filesToFindBySha1s );
+			$filesBySha1s = $repoGroup->getLocalRepo()->findBySha1s( $filesToFindBySha1s );
 		} else {
-			$filesBySha1s = RepoGroup::singleton()->findBySha1s( $filesToFindBySha1s );
+			$filesBySha1s = $repoGroup->findBySha1s( $filesToFindBySha1s );
 		}
 
 		// iterate over $images to handle continue param correct
@@ -114,7 +113,7 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 			if ( $params['dir'] == 'descending' ) {
 				$dupFiles = array_reverse( $dupFiles );
 			}
-			/** @var $dupFile File */
+			/** @var File $dupFile */
 			foreach ( $dupFiles as $dupFile ) {
 				$dupName = $dupFile->getName();
 				if ( $image == $dupName && $dupFile->isLocal() ) {
@@ -131,7 +130,7 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 					$this->setContinueEnumParameter( 'continue', $image . '|' . $dupName );
 					break;
 				}
-				if ( !is_null( $resultPageSet ) ) {
+				if ( $resultPageSet !== null ) {
 					$titles[] = $dupFile->getTitle();
 				} else {
 					$r = [
@@ -151,7 +150,7 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 				break;
 			}
 		}
-		if ( !is_null( $resultPageSet ) ) {
+		if ( $resultPageSet !== null ) {
 			$resultPageSet->populateFromTitles( $titles );
 		}
 	}
@@ -189,6 +188,6 @@ class ApiQueryDuplicateFiles extends ApiQueryGeneratorBase {
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Duplicatefiles';
+		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Duplicatefiles';
 	}
 }
