@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Oct 16, 2006
- *
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +20,10 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+use MediaWiki\ParamValidator\TypeDef\UserDef;
+use MediaWiki\Storage\NameTableAccessException;
+
 /**
  * Query action to List the log events, with optional filtering by various parameters.
  *
@@ -31,6 +31,9 @@
  */
 class ApiQueryLogEvents extends ApiQueryBase {
 
+	/** @var CommentStore */
+	private $commentStore;
+
 	public function __construct( ApiQuery $query, $moduleName ) {
 		parent::__construct( $query, $moduleName, 'le' );
 	}
@@ -43,6 +46,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 	public function execute() {
 		$params = $this->extractRequestParams();
 		$db = $this->getDB();
+		$this->commentStore = CommentStore::getStore();
 		$this->requireMaxOneParameter( $params, 'title', 'prefix', 'namespace' );
 
 		$prop = array_flip( $params['prop'] );
@@ -63,51 +67,65 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			$this->addWhere( $hideLogs );
 		}
 
-		// Order is significant here
-		$this->addTables( array( 'logging', 'user', 'page' ) );
-		$this->addJoinConds( array(
-			'user' => array( 'LEFT JOIN',
-				'user_id=log_user' ),
-			'page' => array( 'LEFT JOIN',
-				array( 'log_namespace=page_namespace',
-					'log_title=page_title' ) ) ) );
+		$actorMigration = ActorMigration::newMigration();
+		$actorQuery = $actorMigration->getJoin( 'log_user' );
+		$this->addTables( 'logging' );
+		$this->addTables( $actorQuery['tables'] );
+		$this->addTables( [ 'user', 'page' ] );
+		$this->addJoinConds( $actorQuery['joins'] );
+		$this->addJoinConds( [
+			'user' => [ 'LEFT JOIN',
+				'user_id=' . $actorQuery['fields']['log_user'] ],
+			'page' => [ 'LEFT JOIN',
+				[ 'log_namespace=page_namespace',
+					'log_title=page_title' ] ] ] );
 
-		$this->addFields( array(
+		$this->addFields( [
 			'log_id',
 			'log_type',
 			'log_action',
 			'log_timestamp',
 			'log_deleted',
-		) );
+		] );
 
 		$this->addFieldsIf( 'page_id', $this->fld_ids );
 		// log_page is the page_id saved at log time, whereas page_id is from a
 		// join at query time.  This leads to different results in various
 		// scenarios, e.g. deletion, recreation.
 		$this->addFieldsIf( 'log_page', $this->fld_ids );
-		$this->addFieldsIf( array( 'log_user', 'log_user_text', 'user_name' ), $this->fld_user );
-		$this->addFieldsIf( 'log_user', $this->fld_userid );
+		$this->addFieldsIf( $actorQuery['fields'] + [ 'user_name' ], $this->fld_user );
+		$this->addFieldsIf( $actorQuery['fields'], $this->fld_userid );
 		$this->addFieldsIf(
-			array( 'log_namespace', 'log_title' ),
+			[ 'log_namespace', 'log_title' ],
 			$this->fld_title || $this->fld_parsedcomment
 		);
-		$this->addFieldsIf( 'log_comment', $this->fld_comment || $this->fld_parsedcomment );
 		$this->addFieldsIf( 'log_params', $this->fld_details );
 
+		if ( $this->fld_comment || $this->fld_parsedcomment ) {
+			$commentQuery = $this->commentStore->getJoin( 'log_comment' );
+			$this->addTables( $commentQuery['tables'] );
+			$this->addFields( $commentQuery['fields'] );
+			$this->addJoinConds( $commentQuery['joins'] );
+		}
+
 		if ( $this->fld_tags ) {
-			$this->addTables( 'tag_summary' );
-			$this->addJoinConds( array( 'tag_summary' => array( 'LEFT JOIN', 'log_id=ts_log_id' ) ) );
-			$this->addFields( 'ts_tags' );
+			$this->addFields( [ 'ts_tags' => ChangeTags::makeTagSummarySubquery( 'logging' ) ] );
 		}
 
-		if ( !is_null( $params['tag'] ) ) {
+		if ( $params['tag'] !== null ) {
 			$this->addTables( 'change_tag' );
-			$this->addJoinConds( array( 'change_tag' => array( 'INNER JOIN',
-				array( 'log_id=ct_log_id' ) ) ) );
-			$this->addWhereFld( 'ct_tag', $params['tag'] );
+			$this->addJoinConds( [ 'change_tag' => [ 'JOIN',
+				[ 'log_id=ct_log_id' ] ] ] );
+			$changeTagDefStore = MediaWikiServices::getInstance()->getChangeTagDefStore();
+			try {
+				$this->addWhereFld( 'ct_tag_id', $changeTagDefStore->getId( $params['tag'] ) );
+			} catch ( NameTableAccessException $exception ) {
+				// Return nothing.
+				$this->addWhere( '1=0' );
+			}
 		}
 
-		if ( !is_null( $params['action'] ) ) {
+		if ( $params['action'] !== null ) {
 			// Do validation of action param, list of allowed actions can contains wildcards
 			// Allow the param, when the actions is in the list or a wildcard version is listed.
 			$logAction = $params['action'];
@@ -121,16 +139,16 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			}
 
 			if ( !$valid ) {
-				$valueName = $this->encodeParamName( 'action' );
-				$this->dieUsage(
-					"Unrecognized value for parameter '$valueName': {$logAction}",
-					"unknown_$valueName"
+				$encParamName = $this->encodeParamName( 'action' );
+				$this->dieWithError(
+					[ 'apierror-unrecognizedvalue', $encParamName, wfEscapeWikiText( $logAction ) ],
+					"unknown_$encParamName"
 				);
 			}
 
 			$this->addWhereFld( 'log_type', $type );
 			$this->addWhereFld( 'log_action', $action );
-		} elseif ( !is_null( $params['type'] ) ) {
+		} elseif ( $params['type'] !== null ) {
 			$this->addWhereFld( 'log_type', $params['type'] );
 		}
 
@@ -143,7 +161,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		// Include in ORDER BY for uniqueness
 		$this->addWhereRange( 'log_id', $params['dir'], null, null );
 
-		if ( !is_null( $params['continue'] ) ) {
+		if ( $params['continue'] !== null ) {
 			$cont = explode( '|', $params['continue'] );
 			$this->dieContinueUsageIf( count( $cont ) != 2 );
 			$op = ( $params['dir'] === 'newer' ? '>' : '<' );
@@ -160,20 +178,23 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		$this->addOption( 'LIMIT', $limit + 1 );
 
 		$user = $params['user'];
-		if ( !is_null( $user ) ) {
-			$userid = User::idFromName( $user );
-			if ( $userid ) {
-				$this->addWhereFld( 'log_user', $userid );
-			} else {
-				$this->addWhereFld( 'log_user_text', IP::sanitizeIP( $user ) );
-			}
+		if ( $user !== null ) {
+			// Note the joins in $q are the same as those from ->getJoin() above
+			// so we only need to add 'conds' here.
+			$q = $actorMigration->getWhere( $db, 'log_user', $params['user'] );
+			$this->addWhere( $q['conds'] );
+
+			// T71222: MariaDB's optimizer, at least 10.1.37 and .38, likes to choose a wildly bad plan for
+			// some reason for this code path. Tell it not to use the wrong index it wants to pick.
+			// @phan-suppress-next-line PhanTypeMismatchArgument
+			$this->addOption( 'IGNORE INDEX', [ 'logging' => [ 'times' ] ] );
 		}
 
 		$title = $params['title'];
-		if ( !is_null( $title ) ) {
+		if ( $title !== null ) {
 			$titleObj = Title::newFromText( $title );
-			if ( is_null( $titleObj ) ) {
-				$this->dieUsage( "Bad title value '$title'", 'param_title' );
+			if ( $titleObj === null ) {
+				$this->dieWithError( [ 'apierror-invalidtitle', wfEscapeWikiText( $title ) ] );
 			}
 			$this->addWhereFld( 'log_namespace', $titleObj->getNamespace() );
 			$this->addWhereFld( 'log_title', $titleObj->getDBkey() );
@@ -185,41 +206,61 @@ class ApiQueryLogEvents extends ApiQueryBase {
 
 		$prefix = $params['prefix'];
 
-		if ( !is_null( $prefix ) ) {
+		if ( $prefix !== null ) {
 			if ( $this->getConfig()->get( 'MiserMode' ) ) {
-				$this->dieUsage( 'Prefix search disabled in Miser Mode', 'prefixsearchdisabled' );
+				$this->dieWithError( 'apierror-prefixsearchdisabled' );
 			}
 
 			$title = Title::newFromText( $prefix );
-			if ( is_null( $title ) ) {
-				$this->dieUsage( "Bad title value '$prefix'", 'param_prefix' );
+			if ( $title === null ) {
+				$this->dieWithError( [ 'apierror-invalidtitle', wfEscapeWikiText( $prefix ) ] );
 			}
 			$this->addWhereFld( 'log_namespace', $title->getNamespace() );
 			$this->addWhere( 'log_title ' . $db->buildLike( $title->getDBkey(), $db->anyString() ) );
 		}
 
-		// Paranoia: avoid brute force searches (bug 17342)
-		if ( $params['namespace'] !== null || !is_null( $title ) || !is_null( $user ) ) {
-			if ( !$this->getUser()->isAllowed( 'deletedhistory' ) ) {
+		// Paranoia: avoid brute force searches (T19342)
+		if ( $params['namespace'] !== null || $title !== null || $user !== null ) {
+			if ( !$this->getPermissionManager()->userHasRight( $this->getUser(), 'deletedhistory' ) ) {
 				$titleBits = LogPage::DELETED_ACTION;
 				$userBits = LogPage::DELETED_USER;
-			} elseif ( !$this->getUser()->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
+			} elseif ( !$this->getPermissionManager()
+				->userHasAnyRight( $this->getUser(), 'suppressrevision', 'viewsuppressed' )
+			) {
 				$titleBits = LogPage::DELETED_ACTION | LogPage::DELETED_RESTRICTED;
 				$userBits = LogPage::DELETED_USER | LogPage::DELETED_RESTRICTED;
 			} else {
 				$titleBits = 0;
 				$userBits = 0;
 			}
-			if ( ( $params['namespace'] !== null || !is_null( $title ) ) && $titleBits ) {
+			if ( ( $params['namespace'] !== null || $title !== null ) && $titleBits ) {
 				$this->addWhere( $db->bitAnd( 'log_deleted', $titleBits ) . " != $titleBits" );
 			}
-			if ( !is_null( $user ) && $userBits ) {
+			if ( $user !== null && $userBits ) {
 				$this->addWhere( $db->bitAnd( 'log_deleted', $userBits ) . " != $userBits" );
 			}
 		}
 
+		// T220999: MySQL/MariaDB (10.1.37) can sometimes irrationally decide that querying `actor` before
+		// `logging` and filesorting is somehow better than querying $limit+1 rows from `logging`.
+		// Tell it not to reorder the query. But not when `letag` was used, as it seems as likely
+		// to be harmed as helped in that case.
+		if ( $params['tag'] === null ) {
+			$this->addOption( 'STRAIGHT_JOIN' );
+		}
+
+		$this->addOption(
+			'MAX_EXECUTION_TIME',
+			$this->getConfig()->get( 'MaxExecutionTimeForExpensiveQueries' )
+		);
+
 		$count = 0;
 		$res = $this->select( __METHOD__ );
+
+		if ( $this->fld_title ) {
+			$this->executeGenderCacheFromResultWrapper( $res, __METHOD__, 'log' );
+		}
+
 		$result = $this->getResult();
 		foreach ( $res as $row ) {
 			if ( ++$count > $limit ) {
@@ -230,51 +271,25 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			}
 
 			$vals = $this->extractRowInfo( $row );
-			$fit = $result->addValue( array( 'query', $this->getModuleName() ), null, $vals );
+			$fit = $result->addValue( [ 'query', $this->getModuleName() ], null, $vals );
 			if ( !$fit ) {
 				$this->setContinueEnumParameter( 'continue', "$row->log_timestamp|$row->log_id" );
 				break;
 			}
 		}
-		$result->addIndexedTagName( array( 'query', $this->getModuleName() ), 'item' );
-	}
-
-	/**
-	 * @deprecated since 1.25 Use LogFormatter::formatParametersForApi instead
-	 * @param ApiResult $result
-	 * @param array $vals
-	 * @param string $params
-	 * @param string $type
-	 * @param string $action
-	 * @param string $ts
-	 * @param bool $legacy
-	 * @return array
-	 */
-	public static function addLogParams( $result, &$vals, $params, $type,
-		$action, $ts, $legacy = false
-	) {
-		wfDeprecated( __METHOD__, '1.25' );
-
-		$entry = new ManualLogEntry( $type, $action );
-		$entry->setParameters( $params );
-		$entry->setTimestamp( $ts );
-		$entry->setLegacy( $legacy );
-		$formatter = LogFormatter::newFromEntry( $entry );
-		$vals['params'] = $formatter->formatParametersForApi();
-
-		return $vals;
+		$result->addIndexedTagName( [ 'query', $this->getModuleName() ], 'item' );
 	}
 
 	private function extractRowInfo( $row ) {
 		$logEntry = DatabaseLogEntry::newFromRow( $row );
-		$vals = array(
+		$vals = [
 			ApiResult::META_TYPE => 'assoc',
-		);
+		];
 		$anyHidden = false;
 		$user = $this->getUser();
 
 		if ( $this->fld_ids ) {
-			$vals['logid'] = intval( $row->log_id );
+			$vals['logid'] = (int)$row->log_id;
 		}
 
 		if ( $this->fld_title || $this->fld_parsedcomment ) {
@@ -291,8 +306,8 @@ class ApiQueryLogEvents extends ApiQueryBase {
 					ApiQueryBase::addTitleInfo( $vals, $title );
 				}
 				if ( $this->fld_ids ) {
-					$vals['pageid'] = intval( $row->page_id );
-					$vals['logpage'] = intval( $row->log_page );
+					$vals['pageid'] = (int)$row->page_id;
+					$vals['logpage'] = (int)$row->log_page;
 				}
 				if ( $this->fld_details ) {
 					$vals['params'] = LogFormatter::newFromEntry( $logEntry )->formatParametersForApi();
@@ -312,10 +327,10 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			}
 			if ( LogEventsList::userCan( $row, LogPage::DELETED_USER, $user ) ) {
 				if ( $this->fld_user ) {
-					$vals['user'] = $row->user_name === null ? $row->log_user_text : $row->user_name;
+					$vals['user'] = $row->user_name ?? $row->log_user_text;
 				}
 				if ( $this->fld_userid ) {
-					$vals['userid'] = intval( $row->log_user );
+					$vals['userid'] = (int)$row->log_user;
 				}
 
 				if ( !$row->log_user ) {
@@ -327,18 +342,19 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			$vals['timestamp'] = wfTimestamp( TS_ISO_8601, $row->log_timestamp );
 		}
 
-		if ( ( $this->fld_comment || $this->fld_parsedcomment ) && isset( $row->log_comment ) ) {
+		if ( $this->fld_comment || $this->fld_parsedcomment ) {
 			if ( LogEventsList::isDeleted( $row, LogPage::DELETED_COMMENT ) ) {
 				$vals['commenthidden'] = true;
 				$anyHidden = true;
 			}
 			if ( LogEventsList::userCan( $row, LogPage::DELETED_COMMENT, $user ) ) {
+				$comment = $this->commentStore->getComment( 'log_comment', $row )->text;
 				if ( $this->fld_comment ) {
-					$vals['comment'] = $row->log_comment;
+					$vals['comment'] = $comment;
 				}
 
 				if ( $this->fld_parsedcomment ) {
-					$vals['parsedcomment'] = Linker::formatComment( $row->log_comment, $title );
+					$vals['parsedcomment'] = Linker::formatComment( $comment, $title );
 				}
 			}
 		}
@@ -349,7 +365,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 				ApiResult::setIndexedTagName( $tags, 'tag' );
 				$vals['tags'] = $tags;
 			} else {
-				$vals['tags'] = array();
+				$vals['tags'] = [];
 			}
 		}
 
@@ -365,14 +381,17 @@ class ApiQueryLogEvents extends ApiQueryBase {
 	 */
 	private function getAllowedLogActions() {
 		$config = $this->getConfig();
-		return array_keys( array_merge( $config->get( 'LogActions' ), $config->get( 'LogActionsHandlers' ) ) );
+		return array_keys( array_merge(
+			$config->get( 'LogActions' ),
+			$config->get( 'LogActionsHandlers' )
+		) );
 	}
 
 	public function getCacheMode( $params ) {
 		if ( $this->userCanSeeRevDel() ) {
 			return 'private';
 		}
-		if ( !is_null( $params['prop'] ) && in_array( 'parsedcomment', $params['prop'] ) ) {
+		if ( $params['prop'] !== null && in_array( 'parsedcomment', $params['prop'] ) ) {
 			// formatComment() calls wfMessage() among other things
 			return 'anon-public-user-private';
 		} elseif ( LogEventsList::getExcludeClause( $this->getDB(), 'user', $this->getUser() )
@@ -386,11 +405,17 @@ class ApiQueryLogEvents extends ApiQueryBase {
 
 	public function getAllowedParams( $flags = 0 ) {
 		$config = $this->getConfig();
-		$ret = array(
-			'prop' => array(
+		if ( $flags & ApiBase::GET_VALUES_FOR_HELP ) {
+			$logActions = $this->getAllowedLogActions();
+			sort( $logActions );
+		} else {
+			$logActions = null;
+		}
+		$ret = [
+			'prop' => [
 				ApiBase::PARAM_ISMULTI => true,
 				ApiBase::PARAM_DFLT => 'ids|title|type|user|timestamp|comment|details',
-				ApiBase::PARAM_TYPE => array(
+				ApiBase::PARAM_TYPE => [
 					'ids',
 					'title',
 					'type',
@@ -401,49 +426,53 @@ class ApiQueryLogEvents extends ApiQueryBase {
 					'parsedcomment',
 					'details',
 					'tags'
-				)
-			),
-			'type' => array(
-				ApiBase::PARAM_TYPE => $config->get( 'LogTypes' )
-			),
-			'action' => array(
+				],
+				ApiBase::PARAM_HELP_MSG_PER_VALUE => [],
+			],
+			'type' => [
+				ApiBase::PARAM_TYPE => LogPage::validTypes(),
+			],
+			'action' => [
 				// validation on request is done in execute()
-				ApiBase::PARAM_TYPE => ( $flags & ApiBase::GET_VALUES_FOR_HELP )
-					? $this->getAllowedLogActions()
-					: null
-			),
-			'start' => array(
+				ApiBase::PARAM_TYPE => $logActions
+			],
+			'start' => [
 				ApiBase::PARAM_TYPE => 'timestamp'
-			),
-			'end' => array(
+			],
+			'end' => [
 				ApiBase::PARAM_TYPE => 'timestamp'
-			),
-			'dir' => array(
+			],
+			'dir' => [
 				ApiBase::PARAM_DFLT => 'older',
-				ApiBase::PARAM_TYPE => array(
+				ApiBase::PARAM_TYPE => [
 					'newer',
 					'older'
-				),
+				],
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-direction',
-			),
-			'user' => null,
+			],
+			'user' => [
+				ApiBase::PARAM_TYPE => 'user',
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_RETURN_OBJECT => true,
+			],
 			'title' => null,
-			'namespace' => array(
-				ApiBase::PARAM_TYPE => 'namespace'
-			),
-			'prefix' => array(),
+			'namespace' => [
+				ApiBase::PARAM_TYPE => 'namespace',
+				ApiBase::PARAM_EXTRA_NAMESPACES => [ NS_MEDIA, NS_SPECIAL ],
+			],
+			'prefix' => [],
 			'tag' => null,
-			'limit' => array(
+			'limit' => [
 				ApiBase::PARAM_DFLT => 10,
 				ApiBase::PARAM_TYPE => 'limit',
 				ApiBase::PARAM_MIN => 1,
 				ApiBase::PARAM_MAX => ApiBase::LIMIT_BIG1,
 				ApiBase::PARAM_MAX2 => ApiBase::LIMIT_BIG2
-			),
-			'continue' => array(
+			],
+			'continue' => [
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
-			),
-		);
+			],
+		];
 
 		if ( $config->get( 'MiserMode' ) ) {
 			$ret['prefix'][ApiBase::PARAM_HELP_MSG] = 'api-help-param-disabled-in-miser-mode';
@@ -453,13 +482,13 @@ class ApiQueryLogEvents extends ApiQueryBase {
 	}
 
 	protected function getExamplesMessages() {
-		return array(
+		return [
 			'action=query&list=logevents'
 				=> 'apihelp-query+logevents-example-simple',
-		);
+		];
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Logevents';
+		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Logevents';
 	}
 }
