@@ -24,11 +24,21 @@
  * @ingroup API
  */
 class ApiUpload extends ApiBase {
+
+	use ApiWatchlistTrait;
+
 	/** @var UploadBase|UploadFromChunks */
 	protected $mUpload = null;
 
 	protected $mParams;
 
+	public function __construct( ApiMain $mainModule, $moduleName, $modulePrefix = '' ) {
+		parent::__construct( $mainModule, $moduleName, $modulePrefix );
+
+		$this->watchlistExpiryEnabled = $this->getConfig()->get( 'WatchlistExpiry' );
+		$this->watchlistMaxDuration = $this->getConfig()->get( 'WatchlistExpiryMaxDuration' );
+	}
+
 	public function execute() {
 		// Check whether upload is enabled
 		if ( !UploadBase::isEnabled() ) {
@@ -74,20 +84,7 @@ class ApiUpload extends ApiBase {
 		}
 
 		// Check if the uploaded file is sane
-		if ( $this->mParams['chunk'] ) {
-			$maxSize = UploadBase::getMaxUploadSize();
-			if ( $this->mParams['filesize'] > $maxSize ) {
-				$this->dieWithError( 'file-too-large' );
-			}
-			if ( !$this->mUpload->getTitle() ) {
-				$this->dieWithError( 'illegal-filename' );
-			}
-		} elseif ( $this->mParams['async'] && $this->mParams['filekey'] ) {
-			// defer verification to background process
-		} else {
-			wfDebug( __METHOD__ . " about to verify\n" );
-			$this->verifyUpload();
-		}
+		$this->verifyUpload();
 
 		// Check if the user has the rights to modify or overwrite the requested title
 		// (This check is irrelevant if stashing is already requested, since the errors
@@ -109,6 +106,7 @@ class ApiUpload extends ApiBase {
 
 		// Add 'imageinfo' in a separate addValue() call. File metadata can be unreasonably large,
 		// so otherwise when it exceeded $wgAPIMaxResultSize, no result would be returned (T143993).
+		// @phan-suppress-next-line PhanTypeArraySuspiciousNullable False positive
 		if ( $result['result'] === 'Success' ) {
 			$imageinfo = $this->mUpload->getImageInfo( $this->getResult() );
 			$this->getResult()->addValue( $this->getModuleName(), 'imageinfo', $imageinfo );
@@ -180,6 +178,33 @@ class ApiUpload extends ApiBase {
 		return $result;
 	}
 
+	/**
+	 * @since 1.35
+	 * @see $wgMinUploadChunkSize
+	 * @param Config $config Site configuration for MinUploadChunkSize
+	 * @return int
+	 */
+	public static function getMinUploadChunkSize( Config $config ) {
+		$configured = $config->get( 'MinUploadChunkSize' );
+
+		// Leave some room for other POST parameters
+		$postMax = (
+			wfShorthandToInteger(
+				ini_get( 'post_max_size' ),
+				PHP_INT_MAX
+			) ?: PHP_INT_MAX
+		) - 1024;
+
+		// Ensure the minimum chunk size is less than PHP upload limits
+		// or the maximum upload size.
+		return min(
+			$configured,
+			UploadBase::getMaxUploadSize( 'file' ),
+			UploadBase::getMaxPhpUploadSize(),
+			$postMax
+		);
+	}
+
 	/**
 	 * Get the result of a chunk upload.
 	 * @param array $warnings Array of Api upload warnings
@@ -196,7 +221,7 @@ class ApiUpload extends ApiBase {
 		$chunkPath = $request->getFileTempname( 'chunk' );
 		$chunkSize = $request->getUpload( 'chunk' )->getSize();
 		$totalSoFar = $this->mParams['offset'] + $chunkSize;
-		$minChunkSize = $this->getConfig()->get( 'MinUploadChunkSize' );
+		$minChunkSize = self::getMinUploadChunkSize( $this->getConfig() );
 
 		// Sanity check sizing
 		if ( $totalSoFar > $this->mParams['filesize'] ) {
@@ -306,7 +331,7 @@ class ApiUpload extends ApiBase {
 	 *   - When 'optional', only add a 'stashfailed' key to the data and return null.
 	 *     Use this when some error happened for a non-stash upload and we're stashing the file
 	 *     only to save the client the trouble of re-uploading it.
-	 * @param array &$data API result to which to add the information
+	 * @param array|null &$data API result to which to add the information
 	 * @return string|null File key
 	 */
 	private function performStash( $failureMode, &$data = null ) {
@@ -320,7 +345,7 @@ class ApiUpload extends ApiBase {
 			}
 		} catch ( Exception $e ) {
 			$debugMessage = 'Stashing temporary file failed: ' . get_class( $e ) . ' ' . $e->getMessage();
-			wfDebug( __METHOD__ . ' ' . $debugMessage . "\n" );
+			wfDebug( __METHOD__ . ' ' . $debugMessage );
 			$status = Status::newFatal( $this->getErrorFormatter()->getMessageFromException(
 				$e, [ 'wrap' => new ApiMessage( 'apierror-stashexception', 'stashfailed' ) ]
 			) );
@@ -339,7 +364,7 @@ class ApiUpload extends ApiBase {
 			// Statuses for it. Just extract the exception details and parse them ourselves.
 			list( $exceptionType, $message ) = $status->getMessage()->getParams();
 			$debugMessage = 'Stashing temporary file failed: ' . $exceptionType . ' ' . $message;
-			wfDebug( __METHOD__ . ' ' . $debugMessage . "\n" );
+			wfDebug( __METHOD__ . ' ' . $debugMessage );
 		}
 
 		// Bad status
@@ -403,6 +428,7 @@ class ApiUpload extends ApiBase {
 	 * otherwise true
 	 *
 	 * @return bool
+	 * @suppress PhanTypeArraySuspiciousNullable False positives
 	 */
 	protected function selectUploadModule() {
 		$request = $this->getMain()->getRequest();
@@ -417,7 +443,7 @@ class ApiUpload extends ApiBase {
 		if ( $this->mParams['filekey'] && $this->mParams['checkstatus'] ) {
 			$progress = UploadBase::getSessionStatus( $this->getUser(), $this->mParams['filekey'] );
 			if ( !$progress ) {
-				$this->dieWithError( 'api-upload-missingresult', 'missingresult' );
+				$this->dieWithError( 'apierror-upload-missingresult', 'missingresult' );
 			} elseif ( !$progress['status']->isGood() ) {
 				$this->dieStatusWithCode( $progress['status'], 'stashfailed' );
 			}
@@ -448,7 +474,7 @@ class ApiUpload extends ApiBase {
 		}
 
 		// The following modules all require the filename parameter to be set
-		if ( is_null( $this->mParams['filename'] ) ) {
+		if ( $this->mParams['filename'] === null ) {
 			$this->dieWithError( [ 'apierror-missingparam', 'filename' ] );
 		}
 
@@ -542,7 +568,7 @@ class ApiUpload extends ApiBase {
 		}
 
 		// Check blocks
-		if ( $user->isBlocked() ) {
+		if ( $user->isBlockedFromUpload() ) {
 			$this->dieBlocked( $user->getBlock() );
 		}
 
@@ -556,9 +582,35 @@ class ApiUpload extends ApiBase {
 	 * Performs file verification, dies on error.
 	 */
 	protected function verifyUpload() {
-		$verification = $this->mUpload->verifyUpload();
-		if ( $verification['status'] === UploadBase::OK ) {
-			return;
+		if ( $this->mParams['chunk'] ) {
+			$maxSize = UploadBase::getMaxUploadSize();
+			if ( $this->mParams['filesize'] > $maxSize ) {
+				$this->dieWithError( 'file-too-large' );
+			}
+			if ( !$this->mUpload->getTitle() ) {
+				$this->dieWithError( 'illegal-filename' );
+			}
+			// file will be assembled after having uploaded the last chunk,
+			// so we can only validate the name at this point
+			$verification = $this->mUpload->validateName();
+			if ( $verification === true ) {
+				return;
+			}
+		} elseif ( $this->mParams['async'] && $this->mParams['filekey'] ) {
+			// file will be assembled in a background process, so we
+			// can only validate the name at this point
+			// file verification will happen in background process
+			$verification = $this->mUpload->validateName();
+			if ( $verification === true ) {
+				return;
+			}
+		} else {
+			wfDebug( __METHOD__ . " about to verify" );
+
+			$verification = $this->mUpload->verifyUpload();
+			if ( $verification['status'] === UploadBase::OK ) {
+				return;
+			}
 		}
 
 		$this->checkVerification( $verification );
@@ -636,6 +688,7 @@ class ApiUpload extends ApiBase {
 				}
 				ApiResult::setIndexedTagName( $details, 'detail' );
 				$msg->setApiData( $msg->getApiData() + [ 'details' => $details ] );
+				// @phan-suppress-next-line PhanTypeMismatchArgument
 				$this->dieWithError( $msg );
 				break;
 
@@ -658,7 +711,9 @@ class ApiUpload extends ApiBase {
 	 * @return array
 	 */
 	protected function getApiWarnings() {
-		$warnings = $this->mUpload->checkWarnings();
+		$warnings = UploadBase::makeWarningsSerializable(
+			$this->mUpload->checkWarnings( $this->getUser() )
+		);
 
 		return $this->transformWarnings( $warnings );
 	}
@@ -670,9 +725,8 @@ class ApiUpload extends ApiBase {
 
 			if ( isset( $warnings['duplicate'] ) ) {
 				$dupes = [];
-				/** @var File $dupe */
 				foreach ( $warnings['duplicate'] as $dupe ) {
-					$dupes[] = $dupe->getName();
+					$dupes[] = $dupe['fileName'];
 				}
 				ApiResult::setIndexedTagName( $dupes, 'duplicate' );
 				$warnings['duplicate'] = $dupes;
@@ -681,29 +735,24 @@ class ApiUpload extends ApiBase {
 			if ( isset( $warnings['exists'] ) ) {
 				$warning = $warnings['exists'];
 				unset( $warnings['exists'] );
-				/** @var LocalFile $localFile */
-				$localFile = isset( $warning['normalizedFile'] )
-					? $warning['normalizedFile']
-					: $warning['file'];
-				$warnings[$warning['warning']] = $localFile->getName();
+				$localFile = $warning['normalizedFile'] ?? $warning['file'];
+				$warnings[$warning['warning']] = $localFile['fileName'];
 			}
 
 			if ( isset( $warnings['no-change'] ) ) {
-				/** @var File $file */
 				$file = $warnings['no-change'];
 				unset( $warnings['no-change'] );
 
 				$warnings['nochange'] = [
-					'timestamp' => wfTimestamp( TS_ISO_8601, $file->getTimestamp() )
+					'timestamp' => wfTimestamp( TS_ISO_8601, $file['timestamp'] )
 				];
 			}
 
 			if ( isset( $warnings['duplicate-version'] ) ) {
 				$dupes = [];
-				/** @var File $dupe */
 				foreach ( $warnings['duplicate-version'] as $dupe ) {
 					$dupes[] = [
-						'timestamp' => wfTimestamp( TS_ISO_8601, $dupe->getTimestamp() )
+						'timestamp' => wfTimestamp( TS_ISO_8601, $dupe['timestamp'] )
 					];
 				}
 				unset( $warnings['duplicate-version'] );
@@ -764,28 +813,31 @@ class ApiUpload extends ApiBase {
 	 */
 	protected function performUpload( $warnings ) {
 		// Use comment as initial page text by default
-		if ( is_null( $this->mParams['text'] ) ) {
+		if ( $this->mParams['text'] === null ) {
 			$this->mParams['text'] = $this->mParams['comment'];
 		}
 
 		/** @var LocalFile $file */
 		$file = $this->mUpload->getLocalFile();
+		$user = $this->getUser();
+		$title = $file->getTitle();
 
-		// For preferences mode, we want to watch if 'watchdefault' is set,
+		// for preferences mode, we want to watch if 'watchdefault' is set,
 		// or if the *file* doesn't exist, and either 'watchuploads' or
 		// 'watchcreations' is set. But getWatchlistValue()'s automatic
 		// handling checks if the *title* exists or not, so we need to check
 		// all three preferences manually.
 		$watch = $this->getWatchlistValue(
-			$this->mParams['watchlist'], $file->getTitle(), 'watchdefault'
+			$this->mParams['watchlist'], $title, $user, 'watchdefault'
 		);
 
 		if ( !$watch && $this->mParams['watchlist'] == 'preferences' && !$file->exists() ) {
 			$watch = (
-				$this->getWatchlistValue( 'preferences', $file->getTitle(), 'watchuploads' ) ||
-				$this->getWatchlistValue( 'preferences', $file->getTitle(), 'watchcreations' )
+				$this->getWatchlistValue( 'preferences', $title, $user, 'watchuploads' ) ||
+				$this->getWatchlistValue( 'preferences', $title, $user, 'watchcreations' )
 			);
 		}
+		$watchlistExpiry = $this->getExpiryFromParams( $this->mParams );
 
 		// Deprecated parameters
 		if ( $this->mParams['watch'] ) {
@@ -800,6 +852,7 @@ class ApiUpload extends ApiBase {
 		}
 
 		// No errors, no warnings: do the upload
+		$result = [];
 		if ( $this->mParams['async'] ) {
 			$progress = UploadBase::getSessionStatus( $this->getUser(), $this->mParams['filekey'] );
 			if ( $progress && $progress['result'] === 'Poll' ) {
@@ -819,6 +872,7 @@ class ApiUpload extends ApiBase {
 					'tags' => $this->mParams['tags'],
 					'text' => $this->mParams['text'],
 					'watch' => $watch,
+					'watchlistexpiry' => $watchlistExpiry,
 					'session' => $this->getContext()->exportSession()
 				]
 			) );
@@ -826,8 +880,14 @@ class ApiUpload extends ApiBase {
 			$result['stage'] = 'queued';
 		} else {
 			/** @var Status $status */
-			$status = $this->mUpload->performUpload( $this->mParams['comment'],
-				$this->mParams['text'], $watch, $this->getUser(), $this->mParams['tags'] );
+			$status = $this->mUpload->performUpload(
+				$this->mParams['comment'],
+				$this->mParams['text'],
+				$watch,
+				$this->getUser(),
+				$this->mParams['tags'],
+				$watchlistExpiry
+			);
 
 			if ( !$status->isGood() ) {
 				$this->dieRecoverableError( $status->getErrors() );
@@ -870,14 +930,17 @@ class ApiUpload extends ApiBase {
 				ApiBase::PARAM_DFLT => false,
 				ApiBase::PARAM_DEPRECATED => true,
 			],
-			'watchlist' => [
-				ApiBase::PARAM_DFLT => 'preferences',
-				ApiBase::PARAM_TYPE => [
-					'watch',
-					'preferences',
-					'nochange'
-				],
-			],
+		];
+
+		// Params appear in the docs in the order they are defined,
+		// which is why this is here and not at the bottom.
+		$params += $this->getWatchlistParams( [
+			'watch',
+			'preferences',
+			'nochange',
+		] );
+
+		$params += [
 			'ignorewarnings' => false,
 			'file' => [
 				ApiBase::PARAM_TYPE => 'upload',
