@@ -27,7 +27,6 @@ use MediaWiki\Config\ServiceOptions;
 use MediaWiki\HookContainer\HookContainer;
 use MediaWiki\HookContainer\HookRunner;
 use MediaWiki\Permissions\PermissionManager;
-use MediaWiki\User\UserGroupManager;
 use MediaWiki\User\UserIdentity;
 use Message;
 use MWCryptHash;
@@ -73,29 +72,23 @@ class BlockManager {
 	/** @var HookRunner */
 	private $hookRunner;
 
-	/** @var UserGroupManager */
-	private $userGroupManager;
-
 	/**
 	 * @param ServiceOptions $options
 	 * @param PermissionManager $permissionManager
 	 * @param LoggerInterface $logger
 	 * @param HookContainer $hookContainer
-	 * @param UserGroupManager $userGroupManager
 	 */
 	public function __construct(
 		ServiceOptions $options,
 		PermissionManager $permissionManager,
 		LoggerInterface $logger,
-		HookContainer $hookContainer,
-		UserGroupManager $userGroupManager
+		HookContainer $hookContainer
 	) {
 		$options->assertRequiredOptions( self::CONSTRUCTOR_OPTIONS );
 		$this->options = $options;
 		$this->permissionManager = $permissionManager;
 		$this->logger = $logger;
 		$this->hookRunner = new HookRunner( $hookContainer );
-		$this->userGroupManager = $userGroupManager;
 	}
 
 	/**
@@ -126,9 +119,11 @@ class BlockManager {
 	 * @param bool $fromReplica Whether to check the replica DB first.
 	 *  To improve performance, non-critical checks are done against replica DBs.
 	 *  Check when actually saving should be done against master.
+	 * @param bool $disableIpBlockExemptChecking This is used internally to prevent
+	 *   a infinite recursion with autopromote. See T270145.
 	 * @return AbstractBlock|null The most relevant block, or null if there is no block.
 	 */
-	public function getUserBlock( User $user, $request, $fromReplica ) {
+	public function getUserBlock( User $user, $request, $fromReplica, $disableIpBlockExemptChecking = false ) {
 		$fromMaster = !$fromReplica;
 		$ip = null;
 
@@ -136,17 +131,12 @@ class BlockManager {
 		// or they may be exempt (case #2). If affected, look for additional blocks
 		// against the IP address and referenced in a cookie.
 		$checkIpBlocks = $request &&
-			// We cannot use userHasRight here because Autopromote
-			// checkCondition calls user->getBlock which leads to
-			// this function which then leads back to checkCondition
-			// and continues the cycle until it runs out of memory.
+			// Because calling getBlock within Autopromote leads back to here,
+			// thus causing a infinite recursion. We fix this by not checking for
+			// ipblock-exempt when calling getBlock within Autopromote.
 			// See T270145.
-			!in_array(
-				'ipblock-exempt',
-				$this->permissionManager->getGroupPermissions(
-					$this->userGroupManager->getUserGroups( $user )
-				)
-			);
+			!$disableIpBlockExemptChecking &&
+			!$this->permissionManager->userHasRight( $user, 'ipblock-exempt' );
 
 		if ( $request && $checkIpBlocks ) {
 
