@@ -22,6 +22,12 @@
  * @ingroup Cache
  */
 
+use MediaWiki\Linker\LinkTarget;
+use MediaWiki\MediaWikiServices;
+use MediaWiki\User\UserIdentity;
+use MediaWiki\User\UserOptionsLookup;
+use Wikimedia\Rdbms\ILoadBalancer;
+
 /**
  * Caches user genders when needed to use correct namespace aliases.
  *
@@ -33,19 +39,33 @@ class GenderCache {
 	protected $misses = 0;
 	protected $missLimit = 1000;
 
+	/** @var NamespaceInfo */
+	private $nsInfo;
+
+	/** @var ILoadBalancer|null */
+	private $loadBalancer;
+
+	/** @var UserOptionsLookup */
+	private $userOptionsLookup;
+
+	public function __construct(
+		NamespaceInfo $nsInfo = null,
+		ILoadBalancer $loadBalancer = null,
+		UserOptionsLookup $userOptionsLookup = null
+	) {
+		$this->nsInfo = $nsInfo ?? MediaWikiServices::getInstance()->getNamespaceInfo();
+		$this->loadBalancer = $loadBalancer;
+		$this->userOptionsLookup = $userOptionsLookup ?? MediaWikiServices::getInstance()->getUserOptionsLookup();
+	}
+
 	/**
+	 * @see MediaWikiServices::getInstance()->getGenderCache()
+	 * @deprecated in 1.28 (soft), 1.35 (hard)
 	 * @return GenderCache
 	 */
 	public static function singleton() {
-		static $that = null;
-		if ( $that === null ) {
-			$that = new self();
-		}
-
-		return $that;
-	}
-
-	protected function __construct() {
+		wfDeprecated( __METHOD__, '1.28' );
+		return MediaWikiServices::getInstance()->getGenderCache();
 	}
 
 	/**
@@ -54,7 +74,7 @@ class GenderCache {
 	 */
 	protected function getDefault() {
 		if ( $this->default === null ) {
-			$this->default = User::getDefaultOption( 'gender' );
+			$this->default = $this->userOptionsLookup->getDefaultOption( 'gender' );
 		}
 
 		return $this->default;
@@ -62,14 +82,14 @@ class GenderCache {
 
 	/**
 	 * Returns the gender for given username.
-	 * @param string|User $username Username
+	 * @param string|UserIdentity $username
 	 * @param string $caller The calling method
 	 * @return string
 	 */
 	public function getGenderOf( $username, $caller = '' ) {
 		global $wgUser;
 
-		if ( $username instanceof User ) {
+		if ( $username instanceof UserIdentity ) {
 			$username = $username->getName();
 		}
 
@@ -78,7 +98,7 @@ class GenderCache {
 			if ( $this->misses >= $this->missLimit && $wgUser->getName() !== $username ) {
 				if ( $this->misses === $this->missLimit ) {
 					$this->misses++;
-					wfDebug( __METHOD__ . ": too many misses, returning default onwards\n" );
+					wfDebug( __METHOD__ . ": too many misses, returning default onwards" );
 				}
 
 				return $this->getDefault();
@@ -91,7 +111,7 @@ class GenderCache {
 		/* Undefined if there is a valid username which for some reason doesn't
 		 * exist in the database.
 		 */
-		return isset( $this->cache[$username] ) ? $this->cache[$username] : $this->getDefault();
+		return $this->cache[$username] ?? $this->getDefault();
 	}
 
 	/**
@@ -103,7 +123,7 @@ class GenderCache {
 	public function doLinkBatch( $data, $caller = '' ) {
 		$users = [];
 		foreach ( $data as $ns => $pagenames ) {
-			if ( !MWNamespace::hasGenderDistinction( $ns ) ) {
+			if ( !$this->nsInfo->hasGenderDistinction( $ns ) ) {
 				continue;
 			}
 			foreach ( array_keys( $pagenames ) as $username ) {
@@ -115,20 +135,16 @@ class GenderCache {
 	}
 
 	/**
-	 * Wrapper for doQuery that processes a title or string array.
+	 * Wrapper for doQuery that processes a title array.
 	 *
 	 * @since 1.20
-	 * @param array $titles Array of Title objects or strings
+	 * @param LinkTarget[] $titles
 	 * @param string $caller The calling method
 	 */
 	public function doTitlesArray( $titles, $caller = '' ) {
 		$users = [];
-		foreach ( $titles as $title ) {
-			$titleObj = is_string( $title ) ? Title::newFromText( $title ) : $title;
-			if ( !$titleObj ) {
-				continue;
-			}
-			if ( !MWNamespace::hasGenderDistinction( $titleObj->getNamespace() ) ) {
+		foreach ( $titles as $titleObj ) {
+			if ( !$this->nsInfo->hasGenderDistinction( $titleObj->getNamespace() ) ) {
 				continue;
 			}
 			$users[] = $titleObj->getText();
@@ -139,7 +155,7 @@ class GenderCache {
 
 	/**
 	 * Preloads genders for given list of users.
-	 * @param array|string $users Usernames
+	 * @param string[]|string $users Usernames
 	 * @param string $caller The calling method
 	 */
 	public function doQuery( $users, $caller = '' ) {
@@ -163,7 +179,13 @@ class GenderCache {
 			return;
 		}
 
-		$dbr = wfGetDB( DB_SLAVE );
+		// Only query database, when load balancer is provided by service wiring
+		// This maybe not happen when running as part of the installer
+		if ( $this->loadBalancer === null ) {
+			return;
+		}
+
+		$dbr = $this->loadBalancer->getConnectionRef( DB_REPLICA );
 		$table = [ 'user', 'user_properties' ];
 		$fields = [ 'user_name', 'up_value' ];
 		$conds = [ 'user_name' => $usersToCheck ];
@@ -177,7 +199,7 @@ class GenderCache {
 		$res = $dbr->select( $table, $fields, $conds, $comment, [], $joins );
 
 		foreach ( $res as $row ) {
-			$this->cache[$row->user_name] = $row->up_value ? $row->up_value : $default;
+			$this->cache[$row->user_name] = $row->up_value ?: $default;
 		}
 	}
 
