@@ -31,22 +31,22 @@ use MediaWiki\MediaWikiServices;
  * @ingroup Cache
  */
 class HTMLFileCache extends FileCacheBase {
-	const MODE_NORMAL = 0; // normal cache mode
-	const MODE_OUTAGE = 1; // fallback cache for DB outages
-	const MODE_REBUILD = 2; // background cache rebuild mode
+	public const MODE_NORMAL = 0; // normal cache mode
+	public const MODE_OUTAGE = 1; // fallback cache for DB outages
+	public const MODE_REBUILD = 2; // background cache rebuild mode
 
 	/**
 	 * @param Title|string $title Title object or prefixed DB key string
 	 * @param string $action
-	 * @throws MWException
+	 * @throws InvalidArgumentException
 	 */
 	public function __construct( $title, $action ) {
 		parent::__construct();
 
-		$allowedTypes = self::cacheablePageActions();
-		if ( !in_array( $action, $allowedTypes ) ) {
-			throw new MWException( 'Invalid file cache type given.' );
+		if ( !in_array( $action, self::cacheablePageActions() ) ) {
+			throw new InvalidArgumentException( 'Invalid file cache type given.' );
 		}
+
 		$this->mKey = ( $title instanceof Title )
 			? $title->getPrefixedDBkey()
 			: (string)$title;
@@ -78,7 +78,7 @@ class HTMLFileCache extends FileCacheBase {
 	 */
 	protected function typeSubdirectory() {
 		if ( $this->mType === 'view' ) {
-			return ''; //  b/c to not skip existing cache
+			return ''; // b/c to not skip existing cache
 		} else {
 			return $this->mType . '/';
 		}
@@ -94,10 +94,6 @@ class HTMLFileCache extends FileCacheBase {
 		$config = MediaWikiServices::getInstance()->getMainConfig();
 
 		if ( !$config->get( 'UseFileCache' ) && $mode !== self::MODE_REBUILD ) {
-			return false;
-		} elseif ( $config->get( 'DebugToolbar' ) ) {
-			wfDebug( "HTML file cache skipped. \$wgDebugToolbar on\n" );
-
 			return false;
 		}
 
@@ -123,18 +119,19 @@ class HTMLFileCache extends FileCacheBase {
 		$ulang = $context->getLanguage();
 
 		// Check that there are no other sources of variation
-		if ( $user->getId() || $ulang->getCode() !== $config->get( 'LanguageCode' ) ) {
+		if ( $user->getId() ||
+			!$ulang->equals( MediaWikiServices::getInstance()->getContentLanguage() ) ) {
 			return false;
 		}
 
-		if ( $mode === self::MODE_NORMAL ) {
-			if ( $user->getNewtalk() ) {
-				return false;
-			}
+		$userHasNewMessages = MediaWikiServices::getInstance()
+			->getTalkPageNotificationManager()->userHasNewMessages( $user );
+		if ( ( $mode === self::MODE_NORMAL ) && $userHasNewMessages ) {
+			return false;
 		}
 
 		// Allow extensions to disable caching
-		return Hooks::run( 'HTMLFileCache::useFileCache', [ $context ] );
+		return Hooks::runner()->onHTMLFileCache__useFileCache( $context );
 	}
 
 	/**
@@ -144,10 +141,9 @@ class HTMLFileCache extends FileCacheBase {
 	 * @return void
 	 */
 	public function loadFromFileCache( IContextSource $context, $mode = self::MODE_NORMAL ) {
-		global $wgContLang;
 		$config = MediaWikiServices::getInstance()->getMainConfig();
 
-		wfDebug( __METHOD__ . "()\n" );
+		wfDebug( __METHOD__ . "()" );
 		$filename = $this->cachePath();
 
 		if ( $mode === self::MODE_OUTAGE ) {
@@ -157,14 +153,15 @@ class HTMLFileCache extends FileCacheBase {
 
 		$context->getOutput()->sendCacheControl();
 		header( "Content-Type: {$config->get( 'MimeType' )}; charset=UTF-8" );
-		header( "Content-Language: {$wgContLang->getHtmlCode()}" );
+		header( 'Content-Language: ' .
+			MediaWikiServices::getInstance()->getContentLanguage()->getHtmlCode() );
 		if ( $this->useGzip() ) {
 			if ( wfClientAcceptsGzip() ) {
 				header( 'Content-Encoding: gzip' );
 				readfile( $filename );
 			} else {
 				/* Send uncompressed */
-				wfDebug( __METHOD__ . " uncompressing cache file and sending it\n" );
+				wfDebug( __METHOD__ . " uncompressing cache file and sending it" );
 				readgzfile( $filename );
 			}
 		} else {
@@ -180,7 +177,7 @@ class HTMLFileCache extends FileCacheBase {
 	 *
 	 * Normally this is only registed as a handler if $wgUseFileCache is on.
 	 * If can be explicitly called by rebuildFileCache.php when it takes over
-	 * handling file caching itself, disabling any automatic handling the the
+	 * handling file caching itself, disabling any automatic handling the
 	 * process.
 	 *
 	 * @param string $text
@@ -210,28 +207,24 @@ class HTMLFileCache extends FileCacheBase {
 		}
 
 		// gzip output to buffer as needed and set headers...
-		if ( $this->useGzip() ) {
-			// @todo Ugly wfClientAcceptsGzip() function - use context!
-			if ( wfClientAcceptsGzip() ) {
-				header( 'Content-Encoding: gzip' );
+		// @todo Ugly wfClientAcceptsGzip() function - use context!
+		if ( $this->useGzip() && wfClientAcceptsGzip() ) {
+			header( 'Content-Encoding: gzip' );
 
-				return $compressed;
-			} else {
-				return $text;
-			}
-		} else {
-			return $text;
+			return $compressed;
 		}
+
+		return $text;
 	}
 
 	/**
 	 * Clear the file caches for a page for all actions
-	 * @param Title $title
+	 *
+	 * @param Title|string $title Title or prefixed DB key
 	 * @return bool Whether $wgUseFileCache is enabled
 	 */
-	public static function clearFileCache( Title $title ) {
+	public static function clearFileCache( $title ) {
 		$config = MediaWikiServices::getInstance()->getMainConfig();
-
 		if ( !$config->get( 'UseFileCache' ) ) {
 			return false;
 		}
