@@ -24,6 +24,7 @@ use MediaWiki\MediaWikiServices;
  * The simplest way of implementing IContextSource is to hold a RequestContext as a
  * member variable and provide accessors to it.
  *
+ * @stable to extend
  * @since 1.18
  */
 abstract class ContextSource implements IContextSource {
@@ -35,13 +36,14 @@ abstract class ContextSource implements IContextSource {
 	/**
 	 * Get the base IContextSource object
 	 * @since 1.18
+	 * @stable to override
 	 * @return IContextSource
 	 */
 	public function getContext() {
 		if ( $this->context === null ) {
 			$class = static::class;
 			wfDebug( __METHOD__ . " ($class): called and \$context is null. " .
-				"Using RequestContext::getMain() for sanity\n" );
+				"Using RequestContext::getMain() for sanity" );
 			$this->context = RequestContext::getMain();
 		}
 
@@ -50,6 +52,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.18
+	 * @stable to override
 	 * @param IContextSource $context
 	 */
 	public function setContext( IContextSource $context ) {
@@ -58,6 +61,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.23
+	 * @stable to override
 	 * @return Config
 	 */
 	public function getConfig() {
@@ -66,6 +70,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.18
+	 * @stable to override
 	 * @return WebRequest
 	 */
 	public function getRequest() {
@@ -74,6 +79,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.18
+	 * @stable to override
 	 * @return Title|null
 	 */
 	public function getTitle() {
@@ -86,6 +92,7 @@ abstract class ContextSource implements IContextSource {
 	 * if this method returns false.
 	 *
 	 * @since 1.19
+	 * @stable to override
 	 * @return bool
 	 */
 	public function canUseWikiPage() {
@@ -99,6 +106,7 @@ abstract class ContextSource implements IContextSource {
 	 * canUseWikiPage() to check whether this method can be called safely.
 	 *
 	 * @since 1.19
+	 * @stable to override
 	 * @return WikiPage
 	 */
 	public function getWikiPage() {
@@ -107,6 +115,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.18
+	 * @stable to override
 	 * @return OutputPage
 	 */
 	public function getOutput() {
@@ -114,7 +123,9 @@ abstract class ContextSource implements IContextSource {
 	}
 
 	/**
+	 * @stable to override
 	 * @since 1.18
+	 * @stable to override
 	 * @return User
 	 */
 	public function getUser() {
@@ -123,6 +134,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.19
+	 * @stable to override
 	 * @return Language
 	 */
 	public function getLanguage() {
@@ -131,6 +143,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.18
+	 * @stable to override
 	 * @return Skin
 	 */
 	public function getSkin() {
@@ -139,6 +152,7 @@ abstract class ContextSource implements IContextSource {
 
 	/**
 	 * @since 1.27
+	 * @stable to override
 	 * @return Timing
 	 */
 	public function getTiming() {
@@ -149,6 +163,7 @@ abstract class ContextSource implements IContextSource {
 	 * @deprecated since 1.27 use a StatsdDataFactory from MediaWikiServices (preferably injected)
 	 *
 	 * @since 1.25
+	 * @stable to override
 	 * @return IBufferingStatsdDataFactory
 	 */
 	public function getStats() {
@@ -160,23 +175,23 @@ abstract class ContextSource implements IContextSource {
 	 * Parameters are the same as wfMessage()
 	 *
 	 * @since 1.18
+	 * @stable to override
 	 * @param string|string[]|MessageSpecifier $key Message key, or array of keys,
 	 *   or a MessageSpecifier.
-	 * @param mixed $args,...
+	 * @param mixed ...$params
 	 * @return Message
 	 */
-	public function msg( $key /* $args */ ) {
-		$args = func_get_args();
-
-		return call_user_func_array( [ $this->getContext(), 'msg' ], $args );
+	public function msg( $key, ...$params ) {
+		return $this->getContext()->msg( $key, ...$params );
 	}
 
 	/**
 	 * Export the resolved user IP, HTTP headers, user ID, and session ID.
 	 * The result will be reasonably sized to allow for serialization.
 	 *
-	 * @return array
 	 * @since 1.21
+	 * @stable to override
+	 * @return array
 	 */
 	public function exportSession() {
 		return $this->getContext()->exportSession();
