@@ -27,26 +27,25 @@
 /**
  * MediaWiki default table style diff formatter
  * @todo document
- * @private
+ * @newable
  * @ingroup DifferenceEngine
  */
 class TableDiffFormatter extends DiffFormatter {
 
-	function __construct() {
+	public function __construct() {
 		$this->leadingContextLines = 2;
 		$this->trailingContextLines = 2;
 	}
 
 	/**
-	 * @static
 	 * @param string $msg
 	 *
 	 * @return mixed
 	 */
 	public static function escapeWhiteSpace( $msg ) {
-		$msg = preg_replace( '/^ /m', '&#160; ', $msg );
-		$msg = preg_replace( '/ $/m', ' &#160;', $msg );
-		$msg = preg_replace( '/  /', '&#160; ', $msg );
+		$msg = preg_replace( '/^ /m', "\u{00A0} ", $msg );
+		$msg = preg_replace( '/ $/m', " \u{00A0}", $msg );
+		$msg = preg_replace( '/  /', "\u{00A0} ", $msg );
 
 		return $msg;
 	}
@@ -62,8 +61,14 @@ class TableDiffFormatter extends DiffFormatter {
 	protected function blockHeader( $xbeg, $xlen, $ybeg, $ylen ) {
 		// '<!--LINE \d+ -->' get replaced by a localised line number
 		// in DifferenceEngine::localiseLineNumbers
-		$r = '<tr><td colspan="2" class="diff-lineno" id="L' . $xbeg . '" ><!--LINE ' . $xbeg . "--></td>\n" .
-			'<td colspan="2" class="diff-lineno"><!--LINE ' . $ybeg . "--></td></tr>\n";
+		$r = '<tr><td colspan="2" class="diff-lineno" id="mw-diff-left-l' .
+			$xbeg .
+			'" ><!--LINE ' .
+			$xbeg .
+			"--></td>\n" .
+			'<td colspan="2" class="diff-lineno"><!--LINE ' .
+			$ybeg .
+			"--></td></tr>\n";
 
 		return $r;
 	}
@@ -74,7 +79,7 @@ class TableDiffFormatter extends DiffFormatter {
 	 * @param string $header
 	 */
 	protected function startBlock( $header ) {
-		echo $header;
+		$this->writeOutput( $header );
 	}
 
 	protected function endBlock() {
@@ -118,7 +123,7 @@ class TableDiffFormatter extends DiffFormatter {
 	 * @return string
 	 */
 	protected function contextLine( $line ) {
-		return $this->wrapLine( '&#160;', 'diff-context', $line );
+		return $this->wrapLine( "\u{00A0}", 'diff-context', $line );
 	}
 
 	/**
@@ -141,7 +146,7 @@ class TableDiffFormatter extends DiffFormatter {
 	 * @return string
 	 */
 	protected function emptyLine() {
-		return '<td colspan="2">&#160;</td>';
+		return "<td colspan=\"2\">\u{00A0}</td>";
 	}
 
 	/**
@@ -151,9 +156,9 @@ class TableDiffFormatter extends DiffFormatter {
 	 */
 	protected function added( $lines ) {
 		foreach ( $lines as $line ) {
-			echo '<tr>' . $this->emptyLine() .
+			$this->writeOutput( '<tr>' . $this->emptyLine() .
 				$this->addedLine( '<ins class="diffchange">' .
-					htmlspecialchars( $line ) . '</ins>' ) . "</tr>\n";
+					htmlspecialchars( $line ) . '</ins>' ) . "</tr>\n" );
 		}
 	}
 
@@ -164,9 +169,9 @@ class TableDiffFormatter extends DiffFormatter {
 	 */
 	protected function deleted( $lines ) {
 		foreach ( $lines as $line ) {
-			echo '<tr>' . $this->deletedLine( '<del class="diffchange">' .
+			$this->writeOutput( '<tr>' . $this->deletedLine( '<del class="diffchange">' .
 					htmlspecialchars( $line ) . '</del>' ) .
-				$this->emptyLine() . "</tr>\n";
+				$this->emptyLine() . "</tr>\n" );
 		}
 	}
 
@@ -177,9 +182,9 @@ class TableDiffFormatter extends DiffFormatter {
 	 */
 	protected function context( $lines ) {
 		foreach ( $lines as $line ) {
-			echo '<tr>' .
+			$this->writeOutput( '<tr>' .
 				$this->contextLine( htmlspecialchars( $line ) ) .
-				$this->contextLine( htmlspecialchars( $line ) ) . "</tr>\n";
+				$this->contextLine( htmlspecialchars( $line ) ) . "</tr>\n" );
 		}
 	}
 
@@ -190,7 +195,6 @@ class TableDiffFormatter extends DiffFormatter {
 	 * @param string[] $closing
 	 */
 	protected function changed( $orig, $closing ) {
-
 		$diff = new WordLevelDiff( $orig, $closing );
 		$del = $diff->orig();
 		$add = $diff->closing();
@@ -198,14 +202,13 @@ class TableDiffFormatter extends DiffFormatter {
 		# Notice that WordLevelDiff returns HTML-escaped output.
 		# Hence, we will be calling addedLine/deletedLine without HTML-escaping.
 
-		while ( $line = array_shift( $del ) ) {
-			$aline = array_shift( $add );
-			echo '<tr>' . $this->deletedLine( $line ) .
-				$this->addedLine( $aline ) . "</tr>\n";
-		}
-		foreach ( $add as $line ) { # If any leftovers
-			echo '<tr>' . $this->emptyLine() .
-				$this->addedLine( $line ) . "</tr>\n";
+		$ndel = count( $del );
+		$nadd = count( $add );
+		$n = max( $ndel, $nadd );
+		for ( $i = 0; $i < $n; $i++ ) {
+			$delLine = $i < $ndel ? $this->deletedLine( $del[$i] ) : $this->emptyLine();
+			$addLine = $i < $nadd ? $this->addedLine( $add[$i] ) : $this->emptyLine();
+			$this->writeOutput( "<tr>{$delLine}{$addLine}</tr>\n" );
 		}
 	}
 
