@@ -28,13 +28,13 @@ use MediaWiki\Logger\LoggerFactory;
  *
  * Example config:
  *
- * $wgForeignFileRepos[] = array(
- *   'class'                  => 'ForeignAPIRepo',
+ * $wgForeignFileRepos[] = [
+ *   'class'                  => ForeignAPIRepo::class,
  *   'name'                   => 'shared',
  *   'apibase'                => 'https://en.wikipedia.org/w/api.php',
  *   'fetchDescription'       => true, // Optional
  *   'descriptionCacheExpiry' => 3600,
- * );
+ * ];
  *
  * @ingroup FileRepo
  */
@@ -42,7 +42,7 @@ class ForeignAPIRepo extends FileRepo {
 	/* This version string is used in the user agent for requests and will help
 	 * server maintainers in identify ForeignAPI usage.
 	 * Update the version every time you make breaking or significant changes. */
-	const VERSION = "2.1";
+	private const VERSION = "2.1";
 
 	/**
 	 * List of iiprop values for the thumbnail fetch queries.
@@ -53,7 +53,7 @@ class ForeignAPIRepo extends FileRepo {
 		'timestamp',
 	];
 
-	protected $fileFactory = [ 'ForeignAPIFile', 'newFromTitle' ];
+	protected $fileFactory = [ ForeignAPIFile::class, 'newFromTitle' ];
 	/** @var int Check back with Commons after this expiry */
 	protected $apiThumbCacheExpiry = 86400; // 1 day (24*3600)
 
@@ -63,18 +63,18 @@ class ForeignAPIRepo extends FileRepo {
 	/** @var array */
 	protected $mFileExists = [];
 
-	/** @var array */
-	private $mQueryCache = [];
+	/** @var string */
+	private $mApiBase;
 
 	/**
 	 * @param array|null $info
 	 */
-	function __construct( $info ) {
+	public function __construct( $info ) {
 		global $wgLocalFileRepo;
 		parent::__construct( $info );
 
 		// https://commons.wikimedia.org/w/api.php
-		$this->mApiBase = isset( $info['apibase'] ) ? $info['apibase'] : null;
+		$this->mApiBase = $info['apibase'] ?? null;
 
 		if ( isset( $info['apiThumbCacheExpiry'] ) ) {
 			$this->apiThumbCacheExpiry = $info['apiThumbCacheExpiry'];
@@ -97,9 +97,8 @@ class ForeignAPIRepo extends FileRepo {
 
 	/**
 	 * @return string
-	 * @since 1.22
 	 */
-	function getApiUrl() {
+	private function getApiUrl() {
 		return $this->mApiBase;
 	}
 
@@ -109,9 +108,9 @@ class ForeignAPIRepo extends FileRepo {
 	 *
 	 * @param Title $title
 	 * @param string|bool $time
-	 * @return File
+	 * @return File|false
 	 */
-	function newFile( $title, $time = false ) {
+	public function newFile( $title, $time = false ) {
 		if ( $time ) {
 			return false;
 		}
@@ -120,10 +119,10 @@ class ForeignAPIRepo extends FileRepo {
 	}
 
 	/**
-	 * @param array $files
+	 * @param string[] $files
 	 * @return array
 	 */
-	function fileExistsBatch( array $files ) {
+	public function fileExistsBatch( array $files ) {
 		$results = [];
 		foreach ( $files as $k => $f ) {
 			if ( isset( $this->mFileExists[$f] ) ) {
@@ -143,7 +142,7 @@ class ForeignAPIRepo extends FileRepo {
 		}
 
 		$data = $this->fetchImageQuery( [
-			'titles' => implode( $files, '|' ),
+			'titles' => implode( '|', $files ),
 			'prop' => 'imageinfo' ]
 		);
 
@@ -176,17 +175,17 @@ class ForeignAPIRepo extends FileRepo {
 
 	/**
 	 * @param string $virtualUrl
-	 * @return bool
+	 * @return array
 	 */
-	function getFileProps( $virtualUrl ) {
-		return false;
+	public function getFileProps( $virtualUrl ) {
+		return [];
 	}
 
 	/**
 	 * @param array $query
-	 * @return string
+	 * @return array|null
 	 */
-	function fetchImageQuery( $query ) {
+	public function fetchImageQuery( $query ) {
 		global $wgLanguageCode;
 
 		$query = array_merge( $query,
@@ -213,7 +212,7 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param array $data
 	 * @return bool|array
 	 */
-	function getImageInfo( $data ) {
+	public function getImageInfo( $data ) {
 		if ( $data && isset( $data['query']['pages'] ) ) {
 			foreach ( $data['query']['pages'] as $info ) {
 				if ( isset( $info['imageinfo'][0] ) ) {
@@ -231,9 +230,9 @@ class ForeignAPIRepo extends FileRepo {
 
 	/**
 	 * @param string $hash
-	 * @return array
+	 * @return ForeignAPIFile[]
 	 */
-	function findBySha1( $hash ) {
+	public function findBySha1( $hash ) {
 		$results = $this->fetchImageQuery( [
 			'aisha1base36' => $hash,
 			'aiprop' => ForeignAPIFile::getProps(),
@@ -257,12 +256,14 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param string $name
 	 * @param int $width
 	 * @param int $height
-	 * @param array $result Out parameter that will be changed by the function.
+	 * @param array|null &$result Output-only parameter, guaranteed to become an array
 	 * @param string $otherParams
 	 *
-	 * @return bool
+	 * @return string|false
 	 */
-	function getThumbUrl( $name, $width = -1, $height = -1, &$result = null, $otherParams = '' ) {
+	private function getThumbUrl(
+		$name, $width = -1, $height = -1, &$result = null, $otherParams = ''
+	) {
 		$data = $this->fetchImageQuery( [
 			'titles' => 'File:' . $name,
 			'iiprop' => self::getIIProps(),
@@ -273,7 +274,7 @@ class ForeignAPIRepo extends FileRepo {
 		$info = $this->getImageInfo( $data );
 
 		if ( $data && $info && isset( $info['thumburl'] ) ) {
-			wfDebug( __METHOD__ . " got remote thumb " . $info['thumburl'] . "\n" );
+			wfDebug( __METHOD__ . " got remote thumb " . $info['thumburl'] );
 			$result = $info;
 
 			return $info['thumburl'];
@@ -287,11 +288,13 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param int $width
 	 * @param int $height
 	 * @param string $otherParams
-	 * @param string $lang Language code for language of error
+	 * @param string|null $lang Language code for language of error
 	 * @return bool|MediaTransformError
 	 * @since 1.22
 	 */
-	function getThumbError( $name, $width = -1, $height = -1, $otherParams = '', $lang = null ) {
+	public function getThumbError(
+		$name, $width = -1, $height = -1, $otherParams = '', $lang = null
+	) {
 		$data = $this->fetchImageQuery( [
 			'titles' => 'File:' . $name,
 			'iiprop' => self::getIIProps(),
@@ -304,7 +307,7 @@ class ForeignAPIRepo extends FileRepo {
 		$info = $this->getImageInfo( $data );
 
 		if ( $data && $info && isset( $info['thumberror'] ) ) {
-			wfDebug( __METHOD__ . " got remote thumb error " . $info['thumberror'] . "\n" );
+			wfDebug( __METHOD__ . " got remote thumb error " . $info['thumberror'] );
 
 			return new MediaTransformError(
 				'thumbnail_error_remote',
@@ -331,8 +334,7 @@ class ForeignAPIRepo extends FileRepo {
 	 *   from handler's makeParamString.
 	 * @return bool|string
 	 */
-	function getThumbUrlFromCache( $name, $width, $height, $params = "" ) {
-		$cache = ObjectCache::getMainWANInstance();
+	public function getThumbUrlFromCache( $name, $width, $height, $params = "" ) {
 		// We can't check the local cache using FileRepo functions because
 		// we override fileExistsBatch(). We have to use the FileBackend directly.
 		$backend = $this->getBackend(); // convenience
@@ -345,25 +347,22 @@ class ForeignAPIRepo extends FileRepo {
 		$sizekey = "$width:$height:$params";
 
 		/* Get the array of urls that we already know */
-		$knownThumbUrls = $cache->get( $key );
+		$knownThumbUrls = $this->wanCache->get( $key );
 		if ( !$knownThumbUrls ) {
 			/* No knownThumbUrls for this file */
 			$knownThumbUrls = [];
-		} else {
-			if ( isset( $knownThumbUrls[$sizekey] ) ) {
-				wfDebug( __METHOD__ . ': Got thumburl from local cache: ' .
-					"{$knownThumbUrls[$sizekey]} \n" );
+		} elseif ( isset( $knownThumbUrls[$sizekey] ) ) {
+			wfDebug( __METHOD__ . ': Got thumburl from local cache: ' .
+				"{$knownThumbUrls[$sizekey]}" );
 
-				return $knownThumbUrls[$sizekey];
-			}
-			/* This size is not yet known */
+			return $knownThumbUrls[$sizekey];
 		}
 
 		$metadata = null;
 		$foreignUrl = $this->getThumbUrl( $name, $width, $height, $metadata, $params );
 
 		if ( !$foreignUrl ) {
-			wfDebug( __METHOD__ . " Could not find thumburl\n" );
+			wfDebug( __METHOD__ . " Could not find thumburl" );
 
 			return false;
 		}
@@ -371,7 +370,7 @@ class ForeignAPIRepo extends FileRepo {
 		// We need the same filename as the remote one :)
 		$fileName = rawurldecode( pathinfo( $foreignUrl, PATHINFO_BASENAME ) );
 		if ( !$this->validateFilename( $fileName ) ) {
-			wfDebug( __METHOD__ . " The deduced filename $fileName is not safe\n" );
+			wfDebug( __METHOD__ . " The deduced filename $fileName is not safe" );
 
 			return false;
 		}
@@ -383,7 +382,7 @@ class ForeignAPIRepo extends FileRepo {
 		if ( $backend->fileExists( [ 'src' => $localFilename ] )
 			&& isset( $metadata['timestamp'] )
 		) {
-			wfDebug( __METHOD__ . " Thumbnail was already downloaded before\n" );
+			wfDebug( __METHOD__ . " Thumbnail was already downloaded before" );
 			$modified = $backend->getFileTimestamp( [ 'src' => $localFilename ] );
 			$remoteModified = strtotime( $metadata['timestamp'] );
 			$current = time();
@@ -391,15 +390,16 @@ class ForeignAPIRepo extends FileRepo {
 			if ( $remoteModified < $modified && $diff < $this->fileCacheExpiry ) {
 				/* Use our current and already downloaded thumbnail */
 				$knownThumbUrls[$sizekey] = $localUrl;
-				$cache->set( $key, $knownThumbUrls, $this->apiThumbCacheExpiry );
+				$this->wanCache->set( $key, $knownThumbUrls, $this->apiThumbCacheExpiry );
 
 				return $localUrl;
 			}
 			/* There is a new Commons file, or existing thumbnail older than a month */
 		}
-		$thumb = self::httpGet( $foreignUrl );
+
+		$thumb = self::httpGet( $foreignUrl, 'default', [], $mtime );
 		if ( !$thumb ) {
-			wfDebug( __METHOD__ . " Could not download thumb\n" );
+			wfDebug( __METHOD__ . " Could not download thumb" );
 
 			return false;
 		}
@@ -408,13 +408,17 @@ class ForeignAPIRepo extends FileRepo {
 		$backend->prepare( [ 'dir' => dirname( $localFilename ) ] );
 		$params = [ 'dst' => $localFilename, 'content' => $thumb ];
 		if ( !$backend->quickCreate( $params )->isOK() ) {
-			wfDebug( __METHOD__ . " could not write to thumb path '$localFilename'\n" );
+			wfDebug( __METHOD__ . " could not write to thumb path '$localFilename'" );
 
 			return $foreignUrl;
 		}
 		$knownThumbUrls[$sizekey] = $localUrl;
-		$cache->set( $key, $knownThumbUrls, $this->apiThumbCacheExpiry );
-		wfDebug( __METHOD__ . " got local thumb $localUrl, saving to cache \n" );
+
+		$ttl = $mtime
+			? $this->wanCache->adaptiveTTL( $mtime, $this->apiThumbCacheExpiry )
+			: $this->apiThumbCacheExpiry;
+		$this->wanCache->set( $key, $knownThumbUrls, $ttl );
+		wfDebug( __METHOD__ . " got local thumb $localUrl, saving to cache" );
 
 		return $localUrl;
 	}
@@ -425,7 +429,7 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param string|null $ext Optional file extension
 	 * @return string
 	 */
-	function getZoneUrl( $zone, $ext = null ) {
+	public function getZoneUrl( $zone, $ext = null ) {
 		switch ( $zone ) {
 			case 'public':
 				return $this->url;
@@ -441,7 +445,7 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param string $zone
 	 * @return bool|null|string
 	 */
-	function getZonePath( $zone ) {
+	public function getZonePath( $zone ) {
 		$supported = [ 'public', 'thumb' ];
 		if ( in_array( $zone, $supported ) ) {
 			return parent::getZonePath( $zone );
@@ -472,7 +476,7 @@ class ForeignAPIRepo extends FileRepo {
 	 * @return array
 	 * @since 1.22
 	 */
-	function getInfo() {
+	public function getInfo() {
 		$info = parent::getInfo();
 		$info['apiurl'] = $this->getApiUrl();
 
@@ -501,18 +505,22 @@ class ForeignAPIRepo extends FileRepo {
 	}
 
 	/**
-	 * Like a Http:get request, but with custom User-Agent.
-	 * @see Http::get
+	 * Like a HttpRequestFactory::get request, but with custom User-Agent.
+	 * @see HttpRequestFactory::get
+	 * @todo Can this use HttpRequestFactory::get() but just pass the 'userAgent' option?
 	 * @param string $url
 	 * @param string $timeout
 	 * @param array $options
+	 * @param int|bool &$mtime Resulting Last-Modified UNIX timestamp if received
 	 * @return bool|string
 	 */
-	public static function httpGet( $url, $timeout = 'default', $options = [] ) {
+	public static function httpGet(
+		$url, $timeout = 'default', $options = [], &$mtime = false
+	) {
 		$options['timeout'] = $timeout;
 		/* Http::get */
 		$url = wfExpandUrl( $url, PROTO_HTTP );
-		wfDebug( "ForeignAPIRepo: HTTP GET: $url\n" );
+		wfDebug( "ForeignAPIRepo: HTTP GET: $url" );
 		$options['method'] = "GET";
 
 		if ( !isset( $options['timeout'] ) ) {
@@ -520,10 +528,13 @@ class ForeignAPIRepo extends FileRepo {
 		}
 
 		$req = MWHttpRequest::factory( $url, $options, __METHOD__ );
-		$req->setUserAgent( ForeignAPIRepo::getUserAgent() );
+		$req->setUserAgent( self::getUserAgent() );
 		$status = $req->execute();
 
 		if ( $status->isOK() ) {
+			$lmod = $req->getResponseHeader( 'Last-Modified' );
+			$mtime = $lmod ? wfTimestamp( TS_UNIX, $lmod ) : false;
+
 			return $req->getContent();
 		} else {
 			$logger = LoggerFactory::getInstance( 'http' );
@@ -531,6 +542,7 @@ class ForeignAPIRepo extends FileRepo {
 				$status->getWikiText( false, false, 'en' ),
 				[ 'caller' => 'ForeignAPIRepo::httpGet' ]
 			);
+
 			return false;
 		}
 	}
@@ -548,7 +560,7 @@ class ForeignAPIRepo extends FileRepo {
 	 * @param string $target Used in cache key creation, mostly
 	 * @param array $query The query parameters for the API request
 	 * @param int $cacheTTL Time to live for the memcached caching
-	 * @return null
+	 * @return string|null
 	 */
 	public function httpGetCached( $target, $query, $cacheTTL = 3600 ) {
 		if ( $this->mApiBase ) {
@@ -557,42 +569,36 @@ class ForeignAPIRepo extends FileRepo {
 			$url = $this->makeUrl( $query, 'api' );
 		}
 
-		if ( !isset( $this->mQueryCache[$url] ) ) {
-			$data = ObjectCache::getMainWANInstance()->getWithSetCallback(
-				$this->getLocalCacheKey( get_class( $this ), $target, md5( $url ) ),
-				$cacheTTL,
-				function () use ( $url ) {
-					return ForeignAPIRepo::httpGet( $url );
+		return $this->wanCache->getWithSetCallback(
+			$this->getLocalCacheKey( static::class, $target, md5( $url ) ),
+			$cacheTTL,
+			function ( $curValue, &$ttl ) use ( $url ) {
+				$html = self::httpGet( $url, 'default', [], $mtime );
+				if ( $html !== false ) {
+					$ttl = $mtime ? $this->wanCache->adaptiveTTL( $mtime, $ttl ) : $ttl;
+				} else {
+					$ttl = $this->wanCache->adaptiveTTL( $mtime, $ttl );
+					$html = null; // caches negatives
 				}
-			);
 
-			if ( !$data ) {
-				return null;
-			}
-
-			if ( count( $this->mQueryCache ) > 100 ) {
-				// Keep the cache from growing infinitely
-				$this->mQueryCache = [];
-			}
-
-			$this->mQueryCache[$url] = $data;
-		}
-
-		return $this->mQueryCache[$url];
+				return $html;
+			},
+			[ 'pcGroup' => 'http-get:3', 'pcTTL' => WANObjectCache::TTL_PROC_LONG ]
+		);
 	}
 
 	/**
 	 * @param callable $callback
 	 * @throws MWException
 	 */
-	function enumFiles( $callback ) {
-		throw new MWException( 'enumFiles is not supported by ' . get_class( $this ) );
+	public function enumFiles( $callback ) {
+		throw new MWException( 'enumFiles is not supported by ' . static::class );
 	}
 
 	/**
 	 * @throws MWException
 	 */
 	protected function assertWritableRepo() {
-		throw new MWException( get_class( $this ) . ': write operations are not supported.' );
+		throw new MWException( static::class . ': write operations are not supported.' );
 	}
 }
