@@ -20,16 +20,22 @@
  * @file
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * Image gallery
  *
  * Add images to the gallery using add(), then render that list to HTML using toHTML().
- *
+ * @stable to extend
  * @ingroup Media
  */
 abstract class ImageGalleryBase extends ContextSource {
+	public const LOADING_DEFAULT = 1;
+	public const LOADING_LAZY = 2;
+
 	/**
-	 * @var array Gallery images
+	 * @var array[] Gallery images
+	 * @phan-var array<int,array{0:Title,1:string,2:string,3:string,4:array,5:int}>
 	 */
 	protected $mImages;
 
@@ -73,21 +79,30 @@ abstract class ImageGalleryBase extends ContextSource {
 	protected $mHideBadImages;
 
 	/**
-	 * @var Parser Registered parser object for output callbacks
+	 * @var Parser|false Registered parser object for output callbacks
 	 */
 	public $mParser;
 
 	/**
-	 * @var Title Contextual title, used when images are being screened against
+	 * @var Title|null Contextual title, used when images are being screened against
 	 *   the bad image list
 	 */
-	protected $contextTitle = false;
+	protected $contextTitle = null;
 
 	/** @var array */
 	protected $mAttribs = [];
 
-	/** @var bool */
-	static private $modeMapping = false;
+	/** @var int */
+	protected $mPerRow;
+
+	/** @var int */
+	protected $mWidths;
+
+	/** @var int */
+	protected $mHeights;
+
+	/** @var array */
+	private static $modeMapping;
 
 	/**
 	 * Get a new image gallery. This is the method other callers
@@ -98,8 +113,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @return ImageGalleryBase
 	 * @throws MWException
 	 */
-	static function factory( $mode = false, IContextSource $context = null ) {
-		global $wgContLang;
+	public static function factory( $mode = false, IContextSource $context = null ) {
 		self::loadModes();
 		if ( !$context ) {
 			$context = RequestContext::getMainAndWarn( __METHOD__ );
@@ -109,7 +123,7 @@ abstract class ImageGalleryBase extends ContextSource {
 			$mode = $galleryOptions['mode'];
 		}
 
-		$mode = $wgContLang->lc( $mode );
+		$mode = MediaWikiServices::getInstance()->getContentLanguage()->lc( $mode );
 
 		if ( isset( self::$modeMapping[$mode] ) ) {
 			$class = self::$modeMapping[$mode];
@@ -120,7 +134,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	}
 
 	private static function loadModes() {
-		if ( self::$modeMapping === false ) {
+		if ( self::$modeMapping === null ) {
 			self::$modeMapping = [
 				'traditional' => TraditionalImageGallery::class,
 				'nolines' => NolinesImageGallery::class,
@@ -130,7 +144,7 @@ abstract class ImageGalleryBase extends ContextSource {
 				'slideshow' => SlideshowImageGallery::class,
 			];
 			// Allow extensions to make a new gallery format.
-			Hooks::run( 'GalleryGetModes', [ &self::$modeMapping ] );
+			Hooks::runner()->onGalleryGetModes( self::$modeMapping );
 		}
 	}
 
@@ -139,10 +153,15 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * You should not call this directly, but instead use
 	 * ImageGalleryBase::factory().
+	 *
+	 * @stable to call
+	 * @note constructors of subclasses must have a compatible signature
+	 *       for use by the factory() method.
+	 *
 	 * @param string $mode
 	 * @param IContextSource|null $context
 	 */
-	function __construct( $mode = 'traditional', IContextSource $context = null ) {
+	public function __construct( $mode = 'traditional', IContextSource $context = null ) {
 		if ( $context ) {
 			$this->setContext( $context );
 		}
@@ -171,7 +190,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param Parser $parser
 	 */
-	function setParser( $parser ) {
+	public function setParser( $parser ) {
 		$this->mParser = $parser;
 	}
 
@@ -179,7 +198,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * Set bad image flag
 	 * @param bool $flag
 	 */
-	function setHideBadImages( $flag = true ) {
+	public function setHideBadImages( $flag = true ) {
 		$this->mHideBadImages = $flag;
 	}
 
@@ -188,7 +207,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param string $caption
 	 */
-	function setCaption( $caption ) {
+	public function setCaption( $caption ) {
 		$this->mCaption = htmlspecialchars( $caption );
 	}
 
@@ -244,6 +263,8 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * to allow extensions to add additional parameters to
 	 * <gallery> parser tag.
 	 *
+	 * @stable to override
+	 *
 	 * @param array $options Attributes of gallery tag
 	 */
 	public function setAdditionalOptions( $options ) {
@@ -258,14 +279,22 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @param string $alt Alt text for the image
 	 * @param string $link Override image link (optional)
 	 * @param array $handlerOpts Array of options for image handler (aka page number)
+	 * @param int $loading Sets loading attribute of the underlying <img> (optional)
 	 */
-	function add( $title, $html = '', $alt = '', $link = '', $handlerOpts = [] ) {
+	public function add(
+			$title,
+			$html = '',
+			$alt = '',
+			$link = '',
+			$handlerOpts = [],
+			$loading = self::LOADING_DEFAULT
+		) {
 		if ( $title instanceof File ) {
 			// Old calling convention
 			$title = $title->getTitle();
 		}
-		$this->mImages[] = [ $title, $html, $alt, $link, $handlerOpts ];
-		wfDebug( 'ImageGallery::add ' . $title->getText() . "\n" );
+		$this->mImages[] = [ $title, $html, $alt, $link, $handlerOpts, $loading ];
+		wfDebug( 'ImageGallery::add ' . $title->getText() );
 	}
 
 	/**
@@ -277,18 +306,27 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @param string $alt Alt text for the image
 	 * @param string $link Override image link (optional)
 	 * @param array $handlerOpts Array of options for image handler (aka page number)
+	 * @param int $loading Sets loading attribute of the underlying <img> (optional)
 	 */
-	function insert( $title, $html = '', $alt = '', $link = '', $handlerOpts = [] ) {
+	public function insert(
+			$title,
+			$html = '',
+			$alt = '',
+			$link = '',
+			$handlerOpts = [],
+			$loading = self::LOADING_DEFAULT
+		) {
 		if ( $title instanceof File ) {
 			// Old calling convention
 			$title = $title->getTitle();
 		}
-		array_unshift( $this->mImages, [ &$title, $html, $alt, $link, $handlerOpts ] );
+		array_unshift( $this->mImages, [ &$title, $html, $alt, $link, $handlerOpts, $loading ] );
 	}
 
 	/**
 	 * Returns the list of images this gallery contains
-	 * @return array
+	 * @return array[]
+	 * @phan-return array<int,array{0:Title,1:string,2:string,3:string,4:array}>
 	 */
 	public function getImages() {
 		return $this->mImages;
@@ -298,7 +336,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * isEmpty() returns true if the gallery contains no images
 	 * @return bool
 	 */
-	function isEmpty() {
+	public function isEmpty() {
 		return empty( $this->mImages );
 	}
 
@@ -308,7 +346,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param bool $f Set to false to disable
 	 */
-	function setShowDimensions( $f ) {
+	public function setShowDimensions( $f ) {
 		$this->mShowDimensions = (bool)$f;
 	}
 
@@ -318,7 +356,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param bool $f Set to false to disable
 	 */
-	function setShowBytes( $f ) {
+	public function setShowBytes( $f ) {
 		$this->mShowBytes = (bool)$f;
 	}
 
@@ -328,7 +366,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param bool $f Set to false to disable
 	 */
-	function setShowFilename( $f ) {
+	public function setShowFilename( $f ) {
 		$this->mShowFilename = (bool)$f;
 	}
 
@@ -341,7 +379,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *
 	 * @param array $attribs Array of HTML attribute pairs
 	 */
-	function setAttributes( $attribs ) {
+	public function setAttributes( $attribs ) {
 		$this->mAttribs = $attribs;
 	}
 
@@ -362,7 +400,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	/**
 	 * Set the contextual title
 	 *
-	 * @param Title $title Contextual title
+	 * @param Title|null $title Contextual title
 	 */
 	public function setContextTitle( $title ) {
 		$this->contextTitle = $title;
@@ -371,12 +409,10 @@ abstract class ImageGalleryBase extends ContextSource {
 	/**
 	 * Get the contextual title, if applicable
 	 *
-	 * @return Title|bool Title or false
+	 * @return Title|null
 	 */
 	public function getContextTitle() {
-		return is_object( $this->contextTitle ) && $this->contextTitle instanceof Title
-			? $this->contextTitle
-			: false;
+		return $this->contextTitle;
 	}
 
 	/**
