@@ -1,4 +1,8 @@
 <?php
+
+use MediaWiki\Linker\LinkRenderer;
+use MediaWiki\MediaWikiServices;
+
 /**
  * Image gallery.
  *
@@ -32,54 +36,66 @@ class TraditionalImageGallery extends ImageGalleryBase {
 	 *
 	 * @return string
 	 */
-	function toHTML() {
+	public function toHTML() {
+		$resolveFilesViaParser = $this->mParser instanceof Parser;
+		if ( $resolveFilesViaParser ) {
+			$out = $this->mParser->getOutput();
+			$repoGroup = null;
+			$linkRenderer = $this->mParser->getLinkRenderer();
+			$badFileLookup = $this->mParser->getBadFileLookup();
+		} else {
+			$out = $this->getOutput();
+			$services = MediaWikiServices::getInstance();
+			$repoGroup = $services->getRepoGroup();
+			$linkRenderer = $services->getLinkRenderer();
+			$badFileLookup = $services->getBadFileLookup();
+		}
+
 		if ( $this->mPerRow > 0 ) {
 			$maxwidth = $this->mPerRow * ( $this->mWidths + $this->getAllPadding() );
-			$oldStyle = isset( $this->mAttribs['style'] ) ? $this->mAttribs['style'] : '';
-			# _width is ignored by any sane browser. IE6 doesn't know max-width
-			# so it uses _width instead
-			$this->mAttribs['style'] = "max-width: {$maxwidth}px;_width: {$maxwidth}px;" .
-				$oldStyle;
+			$oldStyle = $this->mAttribs['style'] ?? '';
+			$this->mAttribs['style'] = "max-width: {$maxwidth}px;" . $oldStyle;
 		}
 
 		$attribs = Sanitizer::mergeAttributes(
 			[ 'class' => 'gallery mw-gallery-' . $this->mMode ], $this->mAttribs );
 
-		$modules = $this->getModules();
-
-		if ( $this->mParser ) {
-			$this->mParser->getOutput()->addModules( $modules );
-			$this->mParser->getOutput()->addModuleStyles( 'mediawiki.page.gallery.styles' );
-		} else {
-			$this->getOutput()->addModules( $modules );
-			$this->getOutput()->addModuleStyles( 'mediawiki.page.gallery.styles' );
-		}
+		$out->addModules( $this->getModules() );
+		$out->addModuleStyles( 'mediawiki.page.gallery.styles' );
 		$output = Xml::openElement( 'ul', $attribs );
 		if ( $this->mCaption ) {
 			$output .= "\n\t<li class='gallerycaption'>{$this->mCaption}</li>";
 		}
 
+		if ( $this->mShowFilename ) {
+			// Preload LinkCache info for when generating links
+			// of the filename below
+			$lb = new LinkBatch();
+			foreach ( $this->mImages as $img ) {
+				$lb->addObj( $img[0] );
+			}
+			$lb->execute();
+		}
+
 		$lang = $this->getRenderLang();
 		# Output each image...
 		foreach ( $this->mImages as $pair ) {
+			// "text" means "caption" here
 			/** @var Title $nt */
-			$nt = $pair[0];
-			$text = $pair[1]; # "text" means "caption" here
-			$alt = $pair[2];
-			$link = $pair[3];
+			list( $nt, $text, $alt, $link, $handlerOpts, $loading ) = $pair;
 
 			$descQuery = false;
 			if ( $nt->getNamespace() === NS_FILE ) {
 				# Get the file...
-				if ( $this->mParser instanceof Parser ) {
+				if ( $resolveFilesViaParser ) {
 					# Give extensions a chance to select the file revision for us
 					$options = [];
-					Hooks::run( 'BeforeParserFetchFileAndTitle',
-						[ $this->mParser, $nt, &$options, &$descQuery ] );
+					Hooks::runner()->onBeforeParserFetchFileAndTitle(
+						$this->mParser, $nt, $options, $descQuery );
 					# Fetch and register the file (file title may be different via hooks)
 					list( $img, $nt ) = $this->mParser->fetchFileAndTitle( $nt, $options );
 				} else {
-					$img = wfFindFile( $nt );
+					$img = $repoGroup->findFile( $nt );
 				}
 			} else {
 				$img = false;
@@ -97,19 +113,16 @@ class TraditionalImageGallery extends ImageGalleryBase {
 					. ( $this->getThumbPadding() + $this->mHeights ) . 'px;">'
 					. htmlspecialchars( $nt->getText() ) . '</div>';
 
-				if ( $this->mParser instanceof Parser ) {
+				if ( $resolveFilesViaParser ) {
 					$this->mParser->addTrackingCategory( 'broken-file-category' );
 				}
-			} elseif ( $this->mHideBadImages
-				&& wfIsBadImage( $nt->getDBkey(), $this->getContextTitle() )
+			} elseif ( $this->mHideBadImages &&
+				$badFileLookup->isBadFile( $nt->getDBkey(), $this->getContextTitle() )
 			) {
 				# The image is blacklisted, just show it as a text link.
 				$thumbhtml = "\n\t\t\t" . '<div class="thumb" style="height: ' .
 					( $this->getThumbPadding() + $this->mHeights ) . 'px;">' .
-					Linker::linkKnown(
-						$nt,
-						htmlspecialchars( $nt->getText() )
-					) .
+					$linkRenderer->makeKnownLink( $nt, $nt->getText() ) .
 					'</div>';
 			} else {
 				$thumb = $img->transform( $transformOptions );
@@ -135,6 +148,10 @@ class TraditionalImageGallery extends ImageGalleryBase {
 						$imageParameters['alt'] = $nt->getText();
 					}
 
+					if ( $loading === ImageGalleryBase::LOADING_LAZY ) {
+						$imageParameters['loading'] = 'lazy';
+					}
+
 					$this->adjustImageParameters( $thumb, $imageParameters );
 
 					Linker::processResponsiveImages( $img, $thumb, $transformOptions );
@@ -153,43 +170,44 @@ class TraditionalImageGallery extends ImageGalleryBase {
 					// Call parser transform hook
 					/** @var MediaHandler $handler */
 					$handler = $img->getHandler();
-					if ( $this->mParser && $handler ) {
+					if ( $resolveFilesViaParser && $handler ) {
 						$handler->parserTransformHook( $this->mParser, $img );
 					}
 				}
 			}
 
-			// @todo Code is incomplete.
-			// $linkTarget = Title::newFromText( $wgContLang->getNsText( MWNamespace::getUser() ) .
-			// ":{$ut}" );
-			// $ul = Linker::link( $linkTarget, $ut );
-
-			if ( $this->mShowBytes ) {
-				if ( $img ) {
-					$fileSize = htmlspecialchars( $lang->formatSize( $img->getSize() ) );
-				} else {
-					$fileSize = $this->msg( 'filemissing' )->escaped();
+			$meta = [];
+			if ( $img ) {
+				if ( $this->mShowDimensions ) {
+					$meta[] = htmlspecialchars( $img->getDimensionsString() );
 				}
-				$fileSize = "$fileSize<br />\n";
-			} else {
-				$fileSize = '';
+				if ( $this->mShowBytes ) {
+					$meta[] = htmlspecialchars( $lang->formatSize( $img->getSize() ) );
+				}
+			} elseif ( $this->mShowDimensions || $this->mShowBytes ) {
+				$meta[] = $this->msg( 'filemissing' )->escaped();
+			}
+			$meta = $lang->semicolonList( $meta );
+			if ( $meta ) {
+				$meta .= "<br />\n";
 			}
 
 			$textlink = $this->mShowFilename ?
-				Linker::linkKnown(
-					$nt,
-					htmlspecialchars( $lang->truncate( $nt->getText(), $this->mCaptionLength ) )
-				) . "<br />\n" :
+				$this->getCaptionHtml( $nt, $lang, $linkRenderer ) :
 				'';
 
-			$galleryText = $textlink . $text . $fileSize;
+			$galleryText = $textlink . $text . $meta;
 			$galleryText = $this->wrapGalleryText( $galleryText, $thumb );
 
+			$gbWidth = $this->getGBWidth( $thumb ) . 'px';
+			if ( $this->getGBWidthOverwrite( $thumb ) ) {
+				$gbWidth = $this->getGBWidthOverwrite( $thumb );
+			}
 			# Weird double wrapping (the extra div inside the li) needed due to FF2 bug
 			# Can be safely removed if FF2 falls completely out of existence
 			$output .= "\n\t\t" . '<li class="gallerybox" style="width: '
-				. $this->getGBWidth( $thumb ) . 'px">'
-				. '<div style="width: ' . $this->getGBWidth( $thumb ) . 'px">'
+				. $gbWidth . '">'
+				. '<div style="width: ' . $gbWidth . '">'
 				. $thumbhtml
 				. $galleryText
 				. "\n\t\t</div></li>";
@@ -199,6 +217,26 @@ class TraditionalImageGallery extends ImageGalleryBase {
 		return $output;
 	}
 
+	/**
+	 * @param Title $nt
+	 * @param Language $lang
+	 * @param LinkRenderer $linkRenderer
+	 * @return string HTML
+	 */
+	protected function getCaptionHtml( Title $nt, Language $lang, LinkRenderer $linkRenderer ) {
+		// Preloaded into LinkCache in toHTML
+		return $linkRenderer->makeKnownLink(
+			$nt,
+			is_int( $this->getCaptionLength() ) ?
+				$lang->truncateForVisual( $nt->getText(), $this->getCaptionLength() ) :
+				$nt->getText(),
+			[
+				'class' => 'galleryfilename' .
+					( $this->getCaptionLength() === true ? ' galleryfilename-truncate' : '' )
+			]
+		) . "\n";
+	}
+
 	/**
 	 * Add the wrapper html around the thumb's caption
 	 *
@@ -219,8 +257,8 @@ class TraditionalImageGallery extends ImageGalleryBase {
 	}
 
 	/**
-	 * How much padding such the thumb have between image and inner div that
-	 * that contains the border. This is both for verical and horizontal
+	 * How much padding the thumb has between the image and the inner div
+	 * that contains the border. This is for both vertical and horizontal
 	 * padding. (However, it is cut in half in the vertical direction).
 	 * @return int
 	 */
@@ -248,6 +286,17 @@ class TraditionalImageGallery extends ImageGalleryBase {
 		return 8;
 	}
 
+	/**
+	 * Length (in characters) to truncate filename to in caption when using "showfilename" (if int).
+	 * A value of 'true' will truncate the filename to one line using CSS, while
+	 * 'false' will disable truncating.
+	 *
+	 * @return int|bool
+	 */
+	protected function getCaptionLength() {
+		return $this->mCaptionLength;
+	}
+
 	/**
 	 * Get total padding.
 	 *
@@ -295,7 +344,7 @@ class TraditionalImageGallery extends ImageGalleryBase {
 	}
 
 	/**
-	 * Width of gallerybox <li>.
+	 * Computed width of gallerybox <li>.
 	 *
 	 * Generally is the width of the image, plus padding on image
 	 * plus padding on gallerybox.
@@ -308,6 +357,21 @@ class TraditionalImageGallery extends ImageGalleryBase {
 		return $this->mWidths + $this->getThumbPadding() + $this->getGBPadding();
 	}
 
+	/**
+	 * Allows overwriting the computed width of the gallerybox <li> with a string,
+	 * like '100%'.
+	 *
+	 * Generally is the width of the image, plus padding on image
+	 * plus padding on gallerybox.
+	 *
+	 * @note Important: parameter will be false if no thumb used.
+	 * @param MediaTransformOutput|bool $thumb MediaTransformObject object or false.
+	 * @return bool|string Ignored if false.
+	 */
+	protected function getGBWidthOverwrite( $thumb ) {
+		return false;
+	}
+
 	/**
 	 * Get a list of modules to include in the page.
 	 *
@@ -324,22 +388,8 @@ class TraditionalImageGallery extends ImageGalleryBase {
 	 *
 	 * Used by a subclass to insert extra high resolution images.
 	 * @param MediaTransformOutput $thumb The thumbnail
-	 * @param array $imageParameters Array of options
+	 * @param array &$imageParameters Array of options
 	 */
 	protected function adjustImageParameters( $thumb, &$imageParameters ) {
 	}
 }
-
-/**
- * Backwards compatibility. This always uses traditional mode
- * if called the old way, for extensions that may expect traditional
- * mode.
- *
- * @deprecated since 1.22 Use ImageGalleryBase::factory instead.
- */
-class ImageGallery extends TraditionalImageGallery {
-	function __construct( $mode = 'traditional' ) {
-		wfDeprecated( __METHOD__, '1.22' );
-		parent::__construct( $mode );
-	}
-}
