@@ -3,8 +3,11 @@
 /**
  * The parent class to generate form fields.  Any field type should
  * be a subclass of this.
+ *
+ * @stable to extend
  */
 abstract class HTMLFormField {
+	/** @var array|array[] */
 	public $mParams;
 
 	protected $mValidationCallback;
@@ -17,6 +20,9 @@ abstract class HTMLFormField {
 	protected $mVFormClass = '';
 	protected $mHelpClass = false;
 	protected $mDefault;
+	/**
+	 * @var array|bool|null
+	 */
 	protected $mOptions = false;
 	protected $mOptionsLabelsNotFromMessage = false;
 	protected $mHideIf = null;
@@ -37,7 +43,7 @@ abstract class HTMLFormField {
 	 * the input object itself.  It should not implement the surrounding
 	 * table cells/rows, or labels/help messages.
 	 *
-	 * @param string $value The value to set the input to; eg a default
+	 * @param mixed $value The value to set the input to; eg a default
 	 *     text for a text input.
 	 *
 	 * @return string Valid HTML.
@@ -47,6 +53,7 @@ abstract class HTMLFormField {
 	/**
 	 * Same as getInputHTML, but returns an OOUI object.
 	 * Defaults to false, which getOOUI will interpret as "use the HTML version"
+	 * @stable to override
 	 *
 	 * @param string $value
 	 * @return OOUI\Widget|false
@@ -58,6 +65,7 @@ abstract class HTMLFormField {
 	/**
 	 * True if this field type is able to display errors; false if validation errors need to be
 	 * displayed in the main HTMLForm error area.
+	 * @stable to override
 	 * @return bool
 	 */
 	public function canDisplayErrors() {
@@ -72,23 +80,21 @@ abstract class HTMLFormField {
 	 *
 	 * Parameters are the same as wfMessage().
 	 *
+	 * @param string|string[]|MessageSpecifier $key
+	 * @param mixed ...$params
 	 * @return Message
 	 */
-	public function msg() {
-		$args = func_get_args();
-
+	public function msg( $key, ...$params ) {
 		if ( $this->mParent ) {
-			$callback = [ $this->mParent, 'msg' ];
-		} else {
-			$callback = 'wfMessage';
+			return $this->mParent->msg( $key, ...$params );
 		}
-
-		return call_user_func_array( $callback, $args );
+		return wfMessage( $key, ...$params );
 	}
 
 	/**
 	 * If this field has a user-visible output or not. If not,
 	 * it will not be rendered
+	 * @stable to override
 	 *
 	 * @return bool
 	 */
@@ -137,8 +143,7 @@ abstract class HTMLFormField {
 		for ( $i = count( $thisKeys ) - 1; $i >= 0; $i-- ) {
 			$keys = array_merge( array_slice( $thisKeys, 0, $i ), $nameKeys );
 			$data = $alldata;
-			while ( $keys ) {
-				$key = array_shift( $keys );
+			foreach ( $keys as $key ) {
 				if ( !is_array( $data ) || !array_key_exists( $key, $data ) ) {
 					continue 2;
 				}
@@ -278,6 +283,7 @@ abstract class HTMLFormField {
 	 * Override this function if the control can somehow trigger a form
 	 * submission that shouldn't actually submit the HTMLForm.
 	 *
+	 * @stable to override
 	 * @since 1.23
 	 * @param string|array $value The value the field was submitted with
 	 * @param array $alldata The data collected from the form
@@ -292,6 +298,7 @@ abstract class HTMLFormField {
 	 * Override this function to add specific validation checks on the
 	 * field input.  Don't forget to call parent::validate() to ensure
 	 * that the user-defined callback mValidationCallback is still run
+	 * @stable to override
 	 *
 	 * @param string|array $value The value the field was submitted with
 	 * @param array $alldata The data collected from the form
@@ -312,15 +319,23 @@ abstract class HTMLFormField {
 		}
 
 		if ( isset( $this->mValidationCallback ) ) {
-			return call_user_func( $this->mValidationCallback, $value, $alldata, $this->mParent );
+			return ( $this->mValidationCallback )( $value, $alldata, $this->mParent );
 		}
 
 		return true;
 	}
 
+	/**
+	 * @stable to override
+	 *
+	 * @param mixed $value
+	 * @param HTMLFormField[] $alldata
+	 *
+	 * @return mixed
+	 */
 	public function filter( $value, $alldata ) {
 		if ( isset( $this->mFilterCallback ) ) {
-			$value = call_user_func( $this->mFilterCallback, $value, $alldata, $this->mParent );
+			$value = ( $this->mFilterCallback )( $value, $alldata, $this->mParent );
 		}
 
 		return $value;
@@ -329,6 +344,7 @@ abstract class HTMLFormField {
 	/**
 	 * Should this field have a label, or is there no input element with the
 	 * appropriate id for the label to point to?
+	 * @stable to override
 	 *
 	 * @return bool True to output a label, false to suppress
 	 */
@@ -366,9 +382,10 @@ abstract class HTMLFormField {
 	/**
 	 * Get the value that this input has been set to from a posted form,
 	 * or the input's default value if it has not been set.
+	 * @stable to override
 	 *
 	 * @param WebRequest $request
-	 * @return string The value
+	 * @return mixed The value
 	 */
 	public function loadDataFromRequest( $request ) {
 		if ( $request->getCheck( $this->mName ) ) {
@@ -381,6 +398,7 @@ abstract class HTMLFormField {
 	/**
 	 * Initialise the object
 	 *
+	 * @stable to call
 	 * @param array $params Associative Array. See HTMLForm doc for syntax.
 	 *
 	 * @since 1.22 The 'label' attribute no longer accepts raw HTML, use 'label-raw' instead
@@ -397,9 +415,9 @@ abstract class HTMLFormField {
 		if ( isset( $params['label-message'] ) ) {
 			$this->mLabel = $this->getMessage( $params['label-message'] )->parse();
 		} elseif ( isset( $params['label'] ) ) {
-			if ( $params['label'] === '&#160;' ) {
+			if ( $params['label'] === '&#160;' || $params['label'] === "\u{00A0}" ) {
 				// Apparently some things set &nbsp directly and in an odd format
-				$this->mLabel = '&#160;';
+				$this->mLabel = "\u{00A0}";
 			} else {
 				$this->mLabel = htmlspecialchars( $params['label'] );
 			}
@@ -467,6 +485,7 @@ abstract class HTMLFormField {
 	/**
 	 * Get the complete table row for the input, including help text,
 	 * labels, and whatever.
+	 * @stable to override
 	 *
 	 * @param string $value The value to set the input to.
 	 *
@@ -475,7 +494,7 @@ abstract class HTMLFormField {
 	public function getTableRow( $value ) {
 		list( $errors, $errorClass ) = $this->getErrorsAndErrorClass( $value );
 		$inputHtml = $this->getInputHTML( $value );
-		$fieldType = static::class;
+		$fieldType = $this->getClassName();
 		$helptext = $this->getHelpTextHtmlTable( $this->getHelpText() );
 		$cellAttributes = [];
 		$rowAttributes = [];
@@ -524,6 +543,7 @@ abstract class HTMLFormField {
 	/**
 	 * Get the complete div for the input, including help text,
 	 * labels, and whatever.
+	 * @stable to override
 	 * @since 1.20
 	 *
 	 * @param string $value The value to set the input to.
@@ -533,7 +553,7 @@ abstract class HTMLFormField {
 	public function getDiv( $value ) {
 		list( $errors, $errorClass ) = $this->getErrorsAndErrorClass( $value );
 		$inputHtml = $this->getInputHTML( $value );
-		$fieldType = static::class;
+		$fieldType = $this->getClassName();
 		$helptext = $this->getHelpTextHtmlDiv( $this->getHelpText() );
 		$cellAttributes = [];
 		$label = $this->getLabelHtml( $cellAttributes );
@@ -543,14 +563,14 @@ abstract class HTMLFormField {
 			'mw-htmlform-nolabel' => ( $label === '' )
 		];
 
-		$horizontalLabel = isset( $this->mParams['horizontal-label'] )
-			? $this->mParams['horizontal-label'] : false;
+		$horizontalLabel = $this->mParams['horizontal-label'] ?? false;
 
 		if ( $horizontalLabel ) {
-			$field = '&#160;' . $inputHtml . "\n$errors";
+			$field = "\u{00A0}" . $inputHtml . "\n$errors";
 		} else {
 			$field = Html::rawElement(
 				'div',
+				// @phan-suppress-next-line PhanUselessBinaryAddRight
 				[ 'class' => $outerDivClass ] + $cellAttributes,
 				$inputHtml . "\n$errors"
 			);
@@ -573,6 +593,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Get the OOUI version of the div. Falls back to getDiv by default.
+	 * @stable to override
 	 * @since 1.26
 	 *
 	 * @param string $value The value to set the input to.
@@ -588,7 +609,7 @@ abstract class HTMLFormField {
 			// It might look weird, but it'll work OK.
 			return $this->getFieldLayoutOOUI(
 				new OOUI\Widget( [ 'content' => new OOUI\HtmlSnippet( $this->getDiv( $value ) ) ] ),
-				[ 'infusable' => false, 'align' => 'top' ]
+				[ 'align' => 'top' ]
 			);
 		}
 
@@ -601,25 +622,20 @@ abstract class HTMLFormField {
 			$infusable = false;
 		}
 
-		$fieldType = static::class;
+		$fieldType = $this->getClassName();
 		$help = $this->getHelpText();
 		$errors = $this->getErrorsRaw( $value );
 		foreach ( $errors as &$error ) {
 			$error = new OOUI\HtmlSnippet( $error );
 		}
 
-		$notices = $this->getNotices();
-		foreach ( $notices as &$notice ) {
-			$notice = new OOUI\HtmlSnippet( $notice );
-		}
-
 		$config = [
 			'classes' => [ "mw-htmlform-field-$fieldType", $this->mClass ],
 			'align' => $this->getLabelAlignOOUI(),
 			'help' => ( $help !== null && $help !== '' ) ? new OOUI\HtmlSnippet( $help ) : null,
 			'errors' => $errors,
-			'notices' => $notices,
 			'infusable' => $infusable,
+			'helpInline' => $this->isHelpInline(),
 		];
 
 		$preloadModules = false;
@@ -631,7 +647,7 @@ abstract class HTMLFormField {
 
 		// the element could specify, that the label doesn't need to be added
 		$label = $this->getLabel();
-		if ( $label && $label !== '&#160;' ) {
+		if ( $label && $label !== "\u{00A0}" && $label !== '&#160;' ) {
 			$config['label'] = new OOUI\HtmlSnippet( $label );
 		}
 
@@ -650,8 +666,21 @@ abstract class HTMLFormField {
 		return $this->getFieldLayoutOOUI( $inputField, $config );
 	}
 
+	/**
+	 * Gets the non namespaced class name
+	 *
+	 * @since 1.36
+	 *
+	 * @return string
+	 */
+	protected function getClassName() {
+		$name = explode( '\\', static::class );
+		return end( $name );
+	}
+
 	/**
 	 * Get label alignment when generating field for OOUI.
+	 * @stable to override
 	 * @return string 'left', 'right', 'top' or 'inline'
 	 */
 	protected function getLabelAlignOOUI() {
@@ -660,9 +689,10 @@ abstract class HTMLFormField {
 
 	/**
 	 * Get a FieldLayout (or subclass thereof) to wrap this field in when using OOUI output.
-	 * @param string $inputField
+	 * @param OOUI\Widget $inputField
 	 * @param array $config
 	 * @return OOUI\FieldLayout|OOUI\ActionFieldLayout
+	 * @suppress PhanUndeclaredProperty Only some subclasses declare mClassWithButton
 	 */
 	protected function getFieldLayoutOOUI( $inputField, $config ) {
 		if ( isset( $this->mClassWithButton ) ) {
@@ -676,17 +706,19 @@ abstract class HTMLFormField {
 	 * Whether the field should be automatically infused. Note that all OOUI HTMLForm fields are
 	 * infusable (you can call OO.ui.infuse() on them), but not all are infused by default, since
 	 * there is no benefit in doing it e.g. for buttons and it's a small performance hit on page load.
+	 * @stable to override
 	 *
 	 * @return bool
 	 */
 	protected function shouldInfuseOOUI() {
-		// Always infuse fields with help text, since the interface for it is nicer with JS
-		return $this->getHelpText() !== null;
+		// Always infuse fields with popup help text, since the interface for it is nicer with JS
+		return $this->getHelpText() !== null && !$this->isHelpInline();
 	}
 
 	/**
 	 * Get the list of extra ResourceLoader modules which must be loaded client-side before it's
 	 * possible to infuse this field's OOUI widget.
+	 * @stable to override
 	 *
 	 * @return string[]
 	 */
@@ -697,6 +729,7 @@ abstract class HTMLFormField {
 	/**
 	 * Get the complete raw fields for the input, including help text,
 	 * labels, and whatever.
+	 * @stable to override
 	 * @since 1.20
 	 *
 	 * @param string $value The value to set the input to.
@@ -722,6 +755,7 @@ abstract class HTMLFormField {
 	 * Get the complete field for the input, including help text,
 	 * labels, and whatever. Fall back from 'vform' to 'div' when not overridden.
 	 *
+	 * @stable to override
 	 * @since 1.25
 	 * @param string $value The value to set the input to.
 	 * @return string Complete HTML field.
@@ -734,6 +768,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Get the complete field as an inline element.
+	 * @stable to override
 	 * @since 1.25
 	 * @param string $value The value to set the input to.
 	 * @return string Complete HTML inline element
@@ -746,7 +781,7 @@ abstract class HTMLFormField {
 		$label = $this->getLabelHtml( $cellAttributes );
 
 		$html = "\n" . $errors .
-			$label . '&#160;' .
+			$label . "\u{00A0}" .
 			$inputHtml .
 			$helptext;
 
@@ -761,7 +796,7 @@ abstract class HTMLFormField {
 	 * @return string
 	 */
 	public function getHelpTextHtmlTable( $helptext ) {
-		if ( is_null( $helptext ) ) {
+		if ( $helptext === null ) {
 			return '';
 		}
 
@@ -790,7 +825,7 @@ abstract class HTMLFormField {
 	 * @return string
 	 */
 	public function getHelpTextHtmlDiv( $helptext ) {
-		if ( is_null( $helptext ) ) {
+		if ( $helptext === null ) {
 			return '';
 		}
 
@@ -822,6 +857,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Determine the help text to display
+	 * @stable to override
 	 * @since 1.20
 	 * @return string|null HTML
 	 */
@@ -837,7 +873,7 @@ abstract class HTMLFormField {
 				$msg = $this->getMessage( $msg );
 
 				if ( $msg->exists() ) {
-					if ( is_null( $helptext ) ) {
+					if ( $helptext === null ) {
 						$helptext = '';
 					} else {
 						$helptext .= $this->msg( 'word-separator' )->escaped(); // some space
@@ -852,12 +888,29 @@ abstract class HTMLFormField {
 		return $helptext;
 	}
 
+	/**
+	 * Determine if the help text should be displayed inline.
+	 *
+	 * Only applies to OOUI forms.
+	 *
+	 * @since 1.31
+	 * @return bool
+	 */
+	public function isHelpInline() {
+		return $this->mParams['help-inline'] ?? true;
+	}
+
 	/**
 	 * Determine form errors to display and their classes
 	 * @since 1.20
 	 *
+	 * phan-taint-check gets confused with returning both classes
+	 * and errors and thinks double escaping is happening, so specify
+	 * that return value has no taint.
+	 *
 	 * @param string $value The value of the input
-	 * @return array array( $errors, $errorClass )
+	 * @return array [ $errors, $errorClass ]
+	 * @return-taint none
 	 */
 	public function getErrorsAndErrorClass( $value ) {
 		$errors = $this->validate( $value, $this->mParent->mFieldData );
@@ -900,36 +953,19 @@ abstract class HTMLFormField {
 	}
 
 	/**
-	 * Determine notices to display for the field.
-	 *
-	 * @since 1.28
-	 * @return string[]
-	 */
-	public function getNotices() {
-		$notices = [];
-
-		if ( isset( $this->mParams['notice-message'] ) ) {
-			$notices[] = $this->getMessage( $this->mParams['notice-message'] )->parse();
-		}
-
-		if ( isset( $this->mParams['notice-messages'] ) ) {
-			foreach ( $this->mParams['notice-messages'] as $msg ) {
-				$notices[] = $this->getMessage( $msg )->parse();
-			}
-		} elseif ( isset( $this->mParams['notice'] ) ) {
-			$notices[] = $this->mParams['notice'];
-		}
-
-		return $notices;
-	}
-
-	/**
+	 * @stable to override
 	 * @return string HTML
 	 */
 	public function getLabel() {
-		return is_null( $this->mLabel ) ? '' : $this->mLabel;
+		return $this->mLabel ?? '';
 	}
 
+	/**
+	 * @stable to override
+	 * @param array $cellAttributes
+	 *
+	 * @return string
+	 */
 	public function getLabelHtml( $cellAttributes = [] ) {
 		# Don't output a for= attribute for labels with no associated input.
 		# Kind of hacky here, possibly we don't want these to be <label>s at all.
@@ -941,14 +977,13 @@ abstract class HTMLFormField {
 
 		$labelValue = trim( $this->getLabel() );
 		$hasLabel = false;
-		if ( $labelValue !== '&#160;' && $labelValue !== '' ) {
+		if ( $labelValue !== "\u{00A0}" && $labelValue !== '&#160;' && $labelValue !== '' ) {
 			$hasLabel = true;
 		}
 
 		$displayFormat = $this->mParent->getDisplayFormat();
 		$html = '';
-		$horizontalLabel = isset( $this->mParams['horizontal-label'] )
-			? $this->mParams['horizontal-label'] : false;
+		$horizontalLabel = $this->mParams['horizontal-label'] ?? false;
 
 		if ( $displayFormat === 'table' ) {
 			$html =
@@ -969,12 +1004,12 @@ abstract class HTMLFormField {
 		return $html;
 	}
 
+	/**
+	 * @stable to override
+	 * @return mixed
+	 */
 	public function getDefault() {
-		if ( isset( $this->mDefault ) ) {
-			return $this->mDefault;
-		} else {
-			return null;
-		}
+		return $this->mDefault ?? null;
 	}
 
 	/**
@@ -1008,6 +1043,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Returns the given attributes from the parameters
+	 * @stable to override
 	 *
 	 * @param array $list List of attributes to get
 	 * @return array Attributes
@@ -1051,7 +1087,7 @@ abstract class HTMLFormField {
 	 * Recursively forces values in an array to strings, because issues arise
 	 * with integer 0 as a value.
 	 *
-	 * @param array $array
+	 * @param array|string $array
 	 * @return array|string
 	 */
 	public static function forceToStringRecursive( $array ) {
@@ -1088,6 +1124,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Get options and make them into arrays suitable for OOUI.
+	 * @stable to override
 	 * @return array Options for inclusion in a select or whatever.
 	 */
 	public function getOptionsOOUI() {
@@ -1125,6 +1162,12 @@ abstract class HTMLFormField {
 	 * Formats one or more errors as accepted by field validation-callback.
 	 *
 	 * @param string|Message|array $errors Array of strings or Message instances
+	 * To work around limitations in phan-taint-check the calling
+	 * class has taintedness disabled. So instead we pretend that
+	 * this method outputs html, since the result is eventually
+	 * outputted anyways without escaping and this allows us to verify
+	 * stuff is safe even though the caller has taintedness cleared.
+	 * @param-taint $errors exec_html
 	 * @return string HTML
 	 * @since 1.18
 	 */
@@ -1146,14 +1189,14 @@ abstract class HTMLFormField {
 				}
 			}
 
-			return Html::rawElement( 'ul', [ 'class' => 'error' ], implode( "\n", $lines ) );
+			$errors = Html::rawElement( 'ul', [], implode( "\n", $lines ) );
 		} else {
 			if ( $errors instanceof Message ) {
 				$errors = $errors->parse();
 			}
-
-			return Html::rawElement( 'span', [ 'class' => 'error' ], $errors );
 		}
+
+		return Html::rawElement( 'div', [ 'class' => 'errorbox' ], $errors );
 	}
 
 	/**
@@ -1174,6 +1217,7 @@ abstract class HTMLFormField {
 
 	/**
 	 * Skip this field when collecting data.
+	 * @stable to override
 	 * @param WebRequest $request
 	 * @return bool
 	 * @since 1.27
