@@ -8,9 +8,17 @@
  * Plus a text field underneath for an additional reason.  The 'value' of the field is
  * "<select>: <extra reason>", or "<extra reason>" if nothing has been selected in the
  * select dropdown.
+ *
+ * @stable to extend
  * @todo FIXME: If made 'required', only the text field should be compulsory.
  */
 class HTMLSelectAndOtherField extends HTMLSelectField {
+	/** @var string[] */
+	private $mFlatOptions;
+
+	/*
+	 * @stable to call
+	 */
 	public function __construct( $params ) {
 		if ( array_key_exists( 'other', $params ) ) {
 			// Do nothing
@@ -119,27 +127,66 @@ class HTMLSelectAndOtherField extends HTMLSelectField {
 			$dropdownInputAttribs['classes'] = [ $this->mClass ];
 		}
 
+		$disabled = false;
+		if ( isset( $this->mParams[ 'disabled' ] ) && $this->mParams[ 'disabled' ] ) {
+			$disabled = true;
+		}
+
 		return $this->getInputWidget( [
 			'id' => $this->mID,
+			'disabled' => $disabled,
 			'textinput' => $textAttribs,
 			'dropdowninput' => $dropdownInputAttribs,
 			'or' => false,
+			'required' => $this->mParams[ 'required' ] ?? false,
 			'classes' => [ 'mw-htmlform-select-and-other-field' ],
 			'data' => [
-				'maxlengthUnit' => isset( $this->mParams['maxlength-unit'] )
-					? $this->mParams['maxlength-unit'] : 'bytes'
+				'maxlengthUnit' => $this->mParams['maxlength-unit'] ?? 'bytes'
 			],
 		] );
 	}
 
+	/**
+	 * @inheritDoc
+	 * @stable to override
+	 */
 	public function getInputWidget( $params ) {
 		return new MediaWiki\Widget\SelectWithInputWidget( $params );
 	}
 
+	/**
+	 * @inheritDoc
+	 */
+	public function getDefault() {
+		$default = parent::getDefault();
+
+		// Default values of empty form
+		$final = '';
+		$list = 'other';
+		$text = '';
+
+		if ( $default !== null ) {
+			$final = $default;
+			// Assume the default is a text value, with the 'other' option selected.
+			// Then check if that assumption is correct, and update $list and $text if not.
+			$text = $final;
+			foreach ( $this->mFlatOptions as $option ) {
+				$match = $option . $this->msg( 'colon-separator' )->inContentLanguage()->text();
+				if ( strpos( $final, $match ) === 0 ) {
+					$list = $option;
+					$text = substr( $final, strlen( $match ) );
+					break;
+				}
+			}
+		}
+
+		return [ $final, $list, $text ];
+	}
+
 	/**
 	 * @param WebRequest $request
 	 *
-	 * @return array("<overall message>","<select value>","<text field value>")
+	 * @return array ["<overall message>","<select value>","<text field value>"]
 	 */
 	public function loadDataFromRequest( $request ) {
 		if ( $request->getCheck( $this->mName ) ) {
@@ -158,26 +205,13 @@ class HTMLSelectAndOtherField extends HTMLSelectField {
 			} else {
 				$final = $list . $this->msg( 'colon-separator' )->inContentLanguage()->text() . $text;
 			}
-		} else {
-			$final = $this->getDefault();
-
-			$list = 'other';
-			$text = $final;
-			foreach ( $this->mFlatOptions as $option ) {
-				$match = $option . $this->msg( 'colon-separator' )->inContentLanguage()->text();
-				if ( strpos( $text, $match ) === 0 ) {
-					$list = $option;
-					$text = substr( $text, strlen( $match ) );
-					break;
-				}
-			}
+			return [ $final, $list, $text ];
 		}
-
-		return [ $final, $list, $text ];
+		return $this->getDefault();
 	}
 
 	public function getSize() {
-		return isset( $this->mParams['size'] ) ? $this->mParams['size'] : 45;
+		return $this->mParams['size'] ?? 45;
 	}
 
 	public function validate( $value, $alldata ) {
@@ -192,7 +226,7 @@ class HTMLSelectAndOtherField extends HTMLSelectField {
 
 		if ( isset( $this->mParams['required'] )
 			&& $this->mParams['required'] !== false
-			&& $value[1] === ''
+			&& $value[0] === ''
 		) {
 			return $this->msg( 'htmlform-required' );
 		}
