@@ -1,8 +1,17 @@
 <?php
 /**
  * Wrapper for Html::namespaceSelector to use in HTMLForm
+ *
+ * @stable to extend
  */
 class HTMLSelectNamespace extends HTMLFormField {
+
+	/** @var string|null */
+	protected $mAllValue;
+
+	/*
+	 * @stable to call
+	 */
 	public function __construct( $params ) {
 		parent::__construct( $params );
 
@@ -11,6 +20,10 @@ class HTMLSelectNamespace extends HTMLFormField {
 			: 'all';
 	}
 
+	/**
+	 * @inheritDoc
+	 * @stable to override
+	 */
 	public function getInputHTML( $value ) {
 		return Html::namespaceSelector(
 			[
@@ -24,6 +37,10 @@ class HTMLSelectNamespace extends HTMLFormField {
 		);
 	}
 
+	/**
+	 * @inheritDoc
+	 * @stable to override
+	 */
 	public function getInputOOUI( $value ) {
 		return new MediaWiki\Widget\NamespaceInputWidget( [
 			'value' => $value,
@@ -33,11 +50,19 @@ class HTMLSelectNamespace extends HTMLFormField {
 		] );
 	}
 
+	/**
+	 * @inheritDoc
+	 * @stable to override
+	 */
 	protected function getOOUIModules() {
 		// FIXME: NamespaceInputWidget should be in its own module (probably?)
 		return [ 'mediawiki.widgets' ];
 	}
 
+	/**
+	 * @inheritDoc
+	 * @stable to override
+	 */
 	protected function shouldInfuseOOUI() {
 		return true;
 	}
