@@ -79,7 +79,8 @@ class WikiImporter {
 
 		// Enable the entity loader, as it is needed for loading external URLs via
 		// XMLReader::open (T86036)
-		$oldDisable = libxml_disable_entity_loader( false );
+		// phpcs:ignore Generic.PHP.NoSilencedErrors -- suppress deprecation per T268847
+		$oldDisable = @libxml_disable_entity_loader( false );
 		if ( defined( 'LIBXML_PARSEHUGE' ) ) {
 			$status = $this->reader->open( "uploadsource://$id", null, LIBXML_PARSEHUGE );
 		} else {
@@ -87,11 +88,13 @@ class WikiImporter {
 		}
 		if ( !$status ) {
 			$error = libxml_get_last_error();
-			libxml_disable_entity_loader( $oldDisable );
+			// phpcs:ignore Generic.PHP.NoSilencedErrors
+			@libxml_disable_entity_loader( $oldDisable );
 			throw new MWException( 'Encountered an internal error while initializing WikiImporter object: ' .
 				$error->message );
 		}
-		libxml_disable_entity_loader( $oldDisable );
+		// phpcs:ignore Generic.PHP.NoSilencedErrors
+		@libxml_disable_entity_loader( $oldDisable );
 
 		// Default callbacks
 		$this->setPageCallback( [ $this, 'beforeImportPage' ] );
@@ -572,13 +575,15 @@ class WikiImporter {
 		// Calls to reader->read need to be wrapped in calls to
 		// libxml_disable_entity_loader() to avoid local file
 		// inclusion attacks (T48932).
-		$oldDisable = libxml_disable_entity_loader( true );
+		// phpcs:ignore Generic.PHP.NoSilencedErrors -- suppress deprecation per T268847
+		$oldDisable = @libxml_disable_entity_loader( true );
 		$rethrow = null;
 		try {
 			$this->reader->read();
 
 			if ( $this->reader->localName != 'mediawiki' ) {
-				libxml_disable_entity_loader( $oldDisable );
+				// phpcs:ignore Generic.PHP.NoSilencedErrors
+				@libxml_disable_entity_loader( $oldDisable );
 				throw new MWException( "Expected <mediawiki> tag, got " .
 					$this->reader->localName );
 			}
@@ -627,7 +632,8 @@ class WikiImporter {
 				}
 			}
 		} finally {
-			libxml_disable_entity_loader( $oldDisable );
+			// phpcs:ignore Generic.PHP.NoSilencedErrors
+			@libxml_disable_entity_loader( $oldDisable );
 			$this->reader->close();
 		}
 
