@@ -20,6 +20,7 @@
  * @file
  * @ingroup Upload
  */
+use Wikimedia\ScopedCallback;
 
 /**
  * Assemble the segments of a chunked upload.
@@ -33,7 +34,6 @@ class AssembleUploadChunksJob extends Job {
 	}
 
 	public function run() {
-		/** @noinspection PhpUnusedLocalVariableInspection */
 		$scope = RequestContext::importScopedSession( $this->params['session'] );
 		$this->addTeardownCallback( function () use ( &$scope ) {
 			ScopedCallback::consume( $scope ); // T126450
@@ -74,8 +74,16 @@ class AssembleUploadChunksJob extends Job {
 				return false;
 			}
 
+			// We can only get warnings like 'duplicate' after concatenating the chunks
+			$status = Status::newGood();
+			$status->value = [
+				'warnings' => UploadBase::makeWarningsSerializable(
+					$upload->checkWarnings( $user )
+				)
+			];
+
 			// We have a new filekey for the fully concatenated file
-			$newFileKey = $upload->getLocalFile()->getFileKey();
+			$newFileKey = $upload->getStashFile()->getFileKey();
 
 			// Remove the old stash file row and first chunk file
 			$upload->stash->removeFileNoAuth( $this->params['filekey'] );
@@ -96,7 +104,7 @@ class AssembleUploadChunksJob extends Job {
 					'stage' => 'assembling',
 					'filekey' => $newFileKey,
 					'imageinfo' => $imageInfo,
-					'status' => Status::newGood()
+					'status' => $status
 				]
 			);
 		} catch ( Exception $e ) {
