@@ -352,16 +352,19 @@ class MapCacheLRU implements ExpirationAwareness, Serializable {
 		return ( $this->getCurrentTime() - $mtime );
 	}
 
-	public function serialize() {
-		return serialize( [
+	public function __serialize() {
+		return [
 			'entries' => $this->cache,
 			'timestamps' => $this->timestamps,
 			'maxCacheKeys' => $this->maxCacheKeys,
-		] );
+		];
 	}
 
-	public function unserialize( $serialized ) {
-		$data = unserialize( $serialized );
+	public function serialize() {
+		return serialize( $this->__serialize() );
+	}
+
+	public function __unserialize( $data ) {
 		$this->cache = $data['entries'] ?? [];
 		$this->timestamps = $data['timestamps'] ?? [];
 		// Fallback needed for serializations prior to T218511
@@ -369,6 +372,10 @@ class MapCacheLRU implements ExpirationAwareness, Serializable {
 		$this->epoch = $this->getCurrentTime();
 	}
 
+	public function unserialize( $serialized ) {
+		$this->__unserialize( unserialize( $serialized ) );
+	}
+
 	/**
 	 * @return float UNIX timestamp
 	 * @codeCoverageIgnore
