@@ -23,14 +23,16 @@
 
 namespace Wikimedia\Rdbms;
 
-use Psr\Log\LoggerInterface;
-use Wikimedia\ScopedCallback;
 use BagOStuff;
 use EmptyBagOStuff;
-use WANObjectCache;
 use Exception;
-use RuntimeException;
 use LogicException;
+use Psr\Log\LoggerInterface;
+use Psr\Log\NullLogger;
+use RuntimeException;
+use Throwable;
+use WANObjectCache;
+use Wikimedia\ScopedCallback;
 
 /**
  * An interface for generating database load balancers
@@ -38,23 +40,27 @@ use LogicException;
  */
 abstract class LBFactory implements ILBFactory {
 	/** @var ChronologyProtector */
-	protected $chronProt;
-	/** @var object|string Class name or object With profileIn/profileOut methods */
-	protected $profiler;
+	private $chronProt;
+	/**
+	 * @var callable|null An optional callback that returns a ScopedCallback instance,
+	 * meant to profile the actual query execution in {@see Database::doQuery}
+	 */
+	private $profiler;
 	/** @var TransactionProfiler */
-	protected $trxProfiler;
+	private $trxProfiler;
 	/** @var LoggerInterface */
-	protected $replLogger;
+	private $replLogger;
 	/** @var LoggerInterface */
-	protected $connLogger;
+	private $connLogger;
 	/** @var LoggerInterface */
-	protected $queryLogger;
+	private $queryLogger;
 	/** @var LoggerInterface */
-	protected $perfLogger;
+	private $perfLogger;
 	/** @var callable Error logger */
-	protected $errorLogger;
+	private $errorLogger;
 	/** @var callable Deprecation logger */
-	protected $deprecationLogger;
+	private $deprecationLogger;
+
 	/** @var BagOStuff */
 	protected $srvCache;
 	/** @var BagOStuff */
@@ -64,29 +70,56 @@ abstract class LBFactory implements ILBFactory {
 
 	/** @var DatabaseDomain Local domain */
 	protected $localDomain;
-	/** @var string Local hostname of the app server */
-	protected $hostname;
-	/** @var array Web request information about the client */
-	protected $requestInfo;
 
-	/** @var mixed */
-	protected $ticket;
-	/** @var string|bool String if a requested DBO_TRX transaction round is active */
-	protected $trxRoundId = false;
-	/** @var string|bool Reason all LBs are read-only or false if not */
-	protected $readOnlyReason = false;
-	/** @var callable[] */
-	protected $replicationWaitCallbacks = [];
+	/** @var string Local hostname of the app server */
+	private $hostname;
+	/** @var array Web request information about the client */
+	private $requestInfo;
+	/** @var bool Whether this PHP instance is for a CLI script */
+	private $cliMode;
+	/** @var string Agent name for query profiling */
+	private $agent;
+	/** @var string Secret string for HMAC hashing */
+	private $secret;
 
 	/** @var array[] $aliases Map of (table => (dbname, schema, prefix) map) */
-	protected $tableAliases = [];
+	private $tableAliases = [];
 	/** @var string[] Map of (index alias => index) */
-	protected $indexAliases = [];
+	private $indexAliases = [];
+	/** @var DatabaseDomain[]|string[] Map of (domain alias => DB domain) */
+	private $domainAliases = [];
+	/** @var callable[] */
+	private $replicationWaitCallbacks = [];
 
-	/** @var bool Whether this PHP instance is for a CLI script */
-	protected $cliMode;
-	/** @var string Agent name for query profiling */
-	protected $agent;
+	/** var int An identifier for this class instance */
+	private $id;
+	/** @var int|null Ticket used to delegate transaction ownership */
+	private $ticket;
+	/** @var string|bool String if a requested DBO_TRX transaction round is active */
+	private $trxRoundId = false;
+	/** @var string One of the ROUND_* class constants */
+	private $trxRoundStage = self::ROUND_CURSORY;
+	/** @var int Default replication wait timeout */
+	private $replicationWaitTimeout;
+
+	/** @var string|bool Reason all LBs are read-only or false if not */
+	protected $readOnlyReason = false;
+
+	/** @var string|null */
+	private $defaultGroup = null;
+
+	/** @var int|null */
+	protected $maxLag;
+
+	/** @var DatabaseDomain[] Map of (domain ID => domain instance) */
+	private $nonLocalDomainCache = [];
+
+	private const ROUND_CURSORY = 'cursory';
+	private const ROUND_BEGINNING = 'within-begin';
+	private const ROUND_COMMITTING = 'within-commit';
+	private const ROUND_ROLLING_BACK = 'within-rollback';
+	private const ROUND_COMMIT_CALLBACKS = 'within-commit-callbacks';
+	private const ROUND_ROLLBACK_CALLBACKS = 'within-rollback-callbacks';
 
 	private static $loggerFields =
 		[ 'replLogger', 'connLogger', 'queryLogger', 'perfLogger' ];
@@ -96,59 +129,100 @@ abstract class LBFactory implements ILBFactory {
 			? DatabaseDomain::newFromId( $conf['localDomain'] )
 			: DatabaseDomain::newUnspecified();
 
+		$this->maxLag = $conf['maxLag'] ?? null;
 		if ( isset( $conf['readOnlyReason'] ) && is_string( $conf['readOnlyReason'] ) ) {
 			$this->readOnlyReason = $conf['readOnlyReason'];
 		}
 
-		$this->srvCache = isset( $conf['srvCache'] ) ? $conf['srvCache'] : new EmptyBagOStuff();
-		$this->memStash = isset( $conf['memStash'] ) ? $conf['memStash'] : new EmptyBagOStuff();
-		$this->wanCache = isset( $conf['wanCache'] )
-			? $conf['wanCache']
-			: WANObjectCache::newEmpty();
+		$this->srvCache = $conf['srvCache'] ?? new EmptyBagOStuff();
+		$this->memStash = $conf['memStash'] ?? new EmptyBagOStuff();
+		$this->wanCache = $conf['wanCache'] ?? WANObjectCache::newEmpty();
 
 		foreach ( self::$loggerFields as $key ) {
-			$this->$key = isset( $conf[$key] ) ? $conf[$key] : new \Psr\Log\NullLogger();
+			$this->$key = $conf[$key] ?? new NullLogger();
 		}
-		$this->errorLogger = isset( $conf['errorLogger'] )
-			? $conf['errorLogger']
-			: function ( Exception $e ) {
-				trigger_error( get_class( $e ) . ': ' . $e->getMessage(), E_USER_WARNING );
-			};
-		$this->deprecationLogger = isset( $conf['deprecationLogger'] )
-			? $conf['deprecationLogger']
-			: function ( $msg ) {
-				trigger_error( $msg, E_USER_DEPRECATED );
-			};
+		$this->errorLogger = $conf['errorLogger'] ?? function ( Throwable $e ) {
+			trigger_error( get_class( $e ) . ': ' . $e->getMessage(), E_USER_WARNING );
+		};
+		$this->deprecationLogger = $conf['deprecationLogger'] ?? function ( $msg ) {
+			trigger_error( $msg, E_USER_DEPRECATED );
+		};
 
-		$this->profiler = isset( $conf['profiler'] ) ? $conf['profiler'] : null;
-		$this->trxProfiler = isset( $conf['trxProfiler'] )
-			? $conf['trxProfiler']
-			: new TransactionProfiler();
+		$this->profiler = $conf['profiler'] ?? null;
+		$this->trxProfiler = $conf['trxProfiler'] ?? new TransactionProfiler();
 
 		$this->requestInfo = [
-			'IPAddress' => isset( $_SERVER[ 'REMOTE_ADDR' ] ) ? $_SERVER[ 'REMOTE_ADDR' ] : '',
-			'UserAgent' => isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '',
-			'ChronologyProtection' => 'true',
-			'ChronologyPositionIndex' => isset( $_GET['cpPosIndex'] ) ? $_GET['cpPosIndex'] : null
+			'IPAddress' => $_SERVER[ 'REMOTE_ADDR' ] ?? '',
+			'UserAgent' => $_SERVER['HTTP_USER_AGENT'] ?? '',
+			// Headers application can inject via LBFactory::setRequestInfo()
+			'ChronologyProtection' => null,
+			'ChronologyClientId' => null, // prior $cpClientId value from LBFactory::shutdown()
+			'ChronologyPositionIndex' => null // prior $cpIndex value from LBFactory::shutdown()
 		];
 
-		$this->cliMode = isset( $conf['cliMode'] )
-			? $conf['cliMode']
-			: ( PHP_SAPI === 'cli' || PHP_SAPI === 'phpdbg' );
-		$this->hostname = isset( $conf['hostname'] ) ? $conf['hostname'] : gethostname();
-		$this->agent = isset( $conf['agent'] ) ? $conf['agent'] : '';
+		$this->cliMode = $conf['cliMode'] ?? ( PHP_SAPI === 'cli' || PHP_SAPI === 'phpdbg' );
+		$this->hostname = $conf['hostname'] ?? gethostname();
+		$this->agent = $conf['agent'] ?? '';
+		$this->defaultGroup = $conf['defaultGroup'] ?? null;
+		$this->secret = $conf['secret'] ?? '';
+		$this->replicationWaitTimeout = $this->cliMode ? 60 : 1;
 
-		$this->ticket = mt_rand();
+		static $nextId, $nextTicket;
+		$this->id = $nextId = ( is_int( $nextId ) ? $nextId++ : mt_rand() );
+		$this->ticket = $nextTicket = ( is_int( $nextTicket ) ? $nextTicket++ : mt_rand() );
 	}
 
 	public function destroy() {
-		$this->shutdown( self::SHUTDOWN_NO_CHRONPROT );
-		$this->forEachLBCallMethod( 'disable' );
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
+		$this->forEachLBCallMethod( 'disable', [ __METHOD__, $this->id ] );
+	}
+
+	public function getLocalDomainID() {
+		return $this->localDomain->getId();
+	}
+
+	public function resolveDomainID( $domain ) {
+		return $this->resolveDomainInstance( $domain )->getId();
+	}
+
+	/**
+	 * @param DatabaseDomain|string|bool $domain
+	 * @return DatabaseDomain
+	 */
+	final protected function resolveDomainInstance( $domain ) {
+		if ( $domain instanceof DatabaseDomain ) {
+			return $domain; // already a domain instance
+		} elseif ( $domain === false || $domain === $this->localDomain->getId() ) {
+			return $this->localDomain;
+		} elseif ( isset( $this->domainAliases[$domain] ) ) {
+			// This array acts as both the original map and as instance cache.
+			// Instances pass-through DatabaseDomain::newFromId as-is.
+			$this->domainAliases[$domain] =
+				DatabaseDomain::newFromId( $this->domainAliases[$domain] );
+
+			return $this->domainAliases[$domain];
+		}
+
+		$cachedDomain = $this->nonLocalDomainCache[$domain] ?? null;
+		if ( $cachedDomain === null ) {
+			$cachedDomain = DatabaseDomain::newFromId( $domain );
+			$this->nonLocalDomainCache = [ $domain => $cachedDomain ];
+		}
+
+		return $cachedDomain;
 	}
 
 	public function shutdown(
-		$mode = self::SHUTDOWN_CHRONPROT_SYNC, callable $workCallback = null, &$cpIndex = null
+		$mode = self::SHUTDOWN_CHRONPROT_SYNC,
+		callable $workCallback = null,
+		&$cpIndex = null,
+		&$cpClientId = null
 	) {
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
 		$chronProt = $this->getChronologyProtector();
 		if ( $mode === self::SHUTDOWN_CHRONPROT_SYNC ) {
 			$this->shutdownChronologyProtector( $chronProt, $workCallback, 'sync', $cpIndex );
@@ -156,37 +230,11 @@ abstract class LBFactory implements ILBFactory {
 			$this->shutdownChronologyProtector( $chronProt, null, 'async', $cpIndex );
 		}
 
+		$cpClientId = $chronProt->getClientId();
+
 		$this->commitMasterChanges( __METHOD__ ); // sanity
 	}
 
-	/**
-	 * @see ILBFactory::newMainLB()
-	 * @param bool $domain
-	 * @return LoadBalancer
-	 */
-	abstract public function newMainLB( $domain = false );
-
-	/**
-	 * @see ILBFactory::getMainLB()
-	 * @param bool $domain
-	 * @return LoadBalancer
-	 */
-	abstract public function getMainLB( $domain = false );
-
-	/**
-	 * @see ILBFactory::newExternalLB()
-	 * @param string $cluster
-	 * @return LoadBalancer
-	 */
-	abstract public function newExternalLB( $cluster );
-
-	/**
-	 * @see ILBFactory::getExternalLB()
-	 * @param string $cluster
-	 * @return LoadBalancer
-	 */
-	abstract public function getExternalLB( $cluster );
-
 	/**
 	 * Call a method of each tracked load balancer
 	 *
@@ -196,80 +244,126 @@ abstract class LBFactory implements ILBFactory {
 	protected function forEachLBCallMethod( $methodName, array $args = [] ) {
 		$this->forEachLB(
 			function ( ILoadBalancer $loadBalancer, $methodName, array $args ) {
-				call_user_func_array( [ $loadBalancer, $methodName ], $args );
+				$loadBalancer->$methodName( ...$args );
 			},
 			[ $methodName, $args ]
 		);
 	}
 
 	public function flushReplicaSnapshots( $fname = __METHOD__ ) {
-		$this->forEachLBCallMethod( 'flushReplicaSnapshots', [ $fname ] );
+		if ( $this->trxRoundId !== false && $this->trxRoundId !== $fname ) {
+			$this->queryLogger->warning(
+				"$fname: transaction round '{$this->trxRoundId}' still running",
+				[ 'trace' => ( new RuntimeException() )->getTraceAsString() ]
+			);
+		}
+		$this->forEachLBCallMethod( 'flushReplicaSnapshots', [ $fname, $this->id ] );
 	}
 
-	public function commitAll( $fname = __METHOD__, array $options = [] ) {
+	final public function commitAll( $fname = __METHOD__, array $options = [] ) {
 		$this->commitMasterChanges( $fname, $options );
-		$this->forEachLBCallMethod( 'commitAll', [ $fname ] );
+		$this->forEachLBCallMethod( 'flushMasterSnapshots', [ $fname, $this->id ] );
+		$this->forEachLBCallMethod( 'flushReplicaSnapshots', [ $fname, $this->id ] );
 	}
 
-	public function beginMasterChanges( $fname = __METHOD__ ) {
+	final public function beginMasterChanges( $fname = __METHOD__ ) {
+		$this->assertTransactionRoundStage( self::ROUND_CURSORY );
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
+		$this->trxRoundStage = self::ROUND_BEGINNING;
 		if ( $this->trxRoundId !== false ) {
 			throw new DBTransactionError(
 				null,
-				"$fname: transaction round '{$this->trxRoundId}' already started."
+				"$fname: transaction round '{$this->trxRoundId}' already started"
 			);
 		}
 		$this->trxRoundId = $fname;
 		// Set DBO_TRX flags on all appropriate DBs
-		$this->forEachLBCallMethod( 'beginMasterChanges', [ $fname ] );
+		$this->forEachLBCallMethod( 'beginMasterChanges', [ $fname, $this->id ] );
+		$this->trxRoundStage = self::ROUND_CURSORY;
 	}
 
-	public function commitMasterChanges( $fname = __METHOD__, array $options = [] ) {
+	final public function commitMasterChanges( $fname = __METHOD__, array $options = [] ) {
+		$this->assertTransactionRoundStage( self::ROUND_CURSORY );
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
+		$this->trxRoundStage = self::ROUND_COMMITTING;
 		if ( $this->trxRoundId !== false && $this->trxRoundId !== $fname ) {
 			throw new DBTransactionError(
 				null,
-				"$fname: transaction round '{$this->trxRoundId}' still running."
+				"$fname: transaction round '{$this->trxRoundId}' still running"
 			);
 		}
-		/** @noinspection PhpUnusedLocalVariableInspection */
-		$scope = $this->getScopedPHPBehaviorForCommit(); // try to ignore client aborts
 		// Run pre-commit callbacks and suppress post-commit callbacks, aborting on failure
-		$this->forEachLBCallMethod( 'finalizeMasterChanges' );
+		do {
+			$count = 0; // number of callbacks executed this iteration
+			$this->forEachLB( function ( ILoadBalancer $lb ) use ( &$count, $fname ) {
+				$count += $lb->finalizeMasterChanges( $fname, $this->id );
+			} );
+		} while ( $count > 0 );
 		$this->trxRoundId = false;
 		// Perform pre-commit checks, aborting on failure
-		$this->forEachLBCallMethod( 'approveMasterChanges', [ $options ] );
+		$this->forEachLBCallMethod( 'approveMasterChanges', [ $options, $fname, $this->id ] );
 		// Log the DBs and methods involved in multi-DB transactions
 		$this->logIfMultiDbTransaction();
 		// Actually perform the commit on all master DB connections and revert DBO_TRX
-		$this->forEachLBCallMethod( 'commitMasterChanges', [ $fname ] );
-		// Run all post-commit callbacks
-		/** @var Exception $e */
-		$e = null; // first callback exception
-		$this->forEachLB( function ( ILoadBalancer $lb ) use ( &$e ) {
-			$ex = $lb->runMasterPostTrxCallbacks( IDatabase::TRIGGER_COMMIT );
-			$e = $e ?: $ex;
-		} );
-		// Commit any dangling DBO_TRX transactions from callbacks on one DB to another DB
-		$this->forEachLBCallMethod( 'commitMasterChanges', [ $fname ] );
+		$this->forEachLBCallMethod( 'commitMasterChanges', [ $fname, $this->id ] );
+		// Run all post-commit callbacks in a separate step
+		$this->trxRoundStage = self::ROUND_COMMIT_CALLBACKS;
+		$e = $this->executePostTransactionCallbacks();
+		$this->trxRoundStage = self::ROUND_CURSORY;
 		// Throw any last post-commit callback error
 		if ( $e instanceof Exception ) {
 			throw $e;
 		}
 	}
 
-	public function rollbackMasterChanges( $fname = __METHOD__ ) {
+	final public function rollbackMasterChanges( $fname = __METHOD__ ) {
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
+		$this->trxRoundStage = self::ROUND_ROLLING_BACK;
 		$this->trxRoundId = false;
-		$this->forEachLBCallMethod( 'suppressTransactionEndCallbacks' );
-		$this->forEachLBCallMethod( 'rollbackMasterChanges', [ $fname ] );
-		// Run all post-rollback callbacks
-		$this->forEachLB( function ( ILoadBalancer $lb ) {
-			$lb->runMasterPostTrxCallbacks( IDatabase::TRIGGER_ROLLBACK );
+		// Actually perform the rollback on all master DB connections and revert DBO_TRX
+		$this->forEachLBCallMethod( 'rollbackMasterChanges', [ $fname, $this->id ] );
+		// Run all post-commit callbacks in a separate step
+		$this->trxRoundStage = self::ROUND_ROLLBACK_CALLBACKS;
+		$this->executePostTransactionCallbacks();
+		$this->trxRoundStage = self::ROUND_CURSORY;
+	}
+
+	/**
+	 * @return Exception|null
+	 */
+	private function executePostTransactionCallbacks() {
+		$fname = __METHOD__;
+		// Run all post-commit callbacks until new ones stop getting added
+		$e = null; // first callback exception
+		do {
+			$this->forEachLB( function ( ILoadBalancer $lb ) use ( &$e, $fname ) {
+				$ex = $lb->runMasterTransactionIdleCallbacks( $fname, $this->id );
+				$e = $e ?: $ex;
+			} );
+		} while ( $this->hasMasterChanges() );
+		// Run all listener callbacks once
+		$this->forEachLB( function ( ILoadBalancer $lb ) use ( &$e, $fname ) {
+			$ex = $lb->runMasterTransactionListenerCallbacks( $fname, $this->id );
+			$e = $e ?: $ex;
 		} );
+
+		return $e;
 	}
 
 	public function hasTransactionRound() {
 		return ( $this->trxRoundId !== false );
 	}
 
+	public function isReadyForRoundOperations() {
+		return ( $this->trxRoundStage === self::ROUND_CURSORY );
+	}
+
 	/**
 	 * Log query info if multi DB transactions are going to be committed now
 	 */
@@ -323,10 +417,11 @@ abstract class LBFactory implements ILBFactory {
 		$opts += [
 			'domain' => false,
 			'cluster' => false,
-			'timeout' => $this->cliMode ? 60 : 10,
+			'timeout' => $this->replicationWaitTimeout,
 			'ifWritesSince' => null
 		];
 
+		// @phan-suppress-next-line PhanSuspiciousValueComparison
 		if ( $opts['domain'] === false && isset( $opts['wiki'] ) ) {
 			$opts['domain'] = $opts['wiki']; // b/c
 		}
@@ -334,6 +429,7 @@ abstract class LBFactory implements ILBFactory {
 		// Figure out which clusters need to be checked
 		/** @var ILoadBalancer[] $lbs */
 		$lbs = [];
+		// @phan-suppress-next-line PhanSuspiciousValueComparison
 		if ( $opts['cluster'] !== false ) {
 			$lbs[] = $this->getExternalLB( $opts['cluster'] );
 		} elseif ( $opts['domain'] !== false ) {
@@ -343,7 +439,7 @@ abstract class LBFactory implements ILBFactory {
 				$lbs[] = $lb;
 			} );
 			if ( !$lbs ) {
-				return; // nothing actually used
+				return true; // nothing actually used
 			}
 		}
 
@@ -352,15 +448,21 @@ abstract class LBFactory implements ILBFactory {
 		// time needed to wait on the next clusters.
 		$masterPositions = array_fill( 0, count( $lbs ), false );
 		foreach ( $lbs as $i => $lb ) {
-			if ( $lb->getServerCount() <= 1 ) {
-				// T29975 - Don't try to wait for replica DBs if there are none
-				// Prevents permission error when getting master position
-				continue;
-			} elseif ( $opts['ifWritesSince']
-				&& $lb->lastMasterChangeTimestamp() < $opts['ifWritesSince']
+			if (
+				// No writes to wait on getting replicated
+				!$lb->hasMasterConnection() ||
+				// No replication; avoid getMasterPos() permissions errors (T29975)
+				!$lb->hasStreamingReplicaServers() ||
+				// No writes since the last replication wait
+				(
+					// @phan-suppress-next-line PhanImpossibleConditionInLoop
+					$opts['ifWritesSince'] &&
+					$lb->lastMasterChangeTimestamp() < $opts['ifWritesSince']
+				)
 			) {
-				continue; // no writes since the last wait
+				continue; // no need to wait
 			}
+
 			$masterPositions[$i] = $lb->getMasterPos();
 		}
 
@@ -380,13 +482,7 @@ abstract class LBFactory implements ILBFactory {
 			}
 		}
 
-		if ( $failed ) {
-			throw new DBReplicationWaitError(
-				null,
-				"Could not wait for replica DBs to catch up to " .
-				implode( ', ', $failed )
-			);
-		}
+		return !$failed;
 	}
 
 	public function setWaitForReplicationListener( $name, callable $callback = null ) {
@@ -399,8 +495,10 @@ abstract class LBFactory implements ILBFactory {
 
 	public function getEmptyTransactionTicket( $fname ) {
 		if ( $this->hasMasterChanges() ) {
-			$this->queryLogger->error( __METHOD__ . ": $fname does not have outer scope.\n" .
-				( new RuntimeException() )->getTraceAsString() );
+			$this->queryLogger->error(
+				__METHOD__ . ": $fname does not have outer scope",
+				[ 'trace' => ( new RuntimeException() )->getTraceAsString() ]
+			);
 
 			return null;
 		}
@@ -408,34 +506,38 @@ abstract class LBFactory implements ILBFactory {
 		return $this->ticket;
 	}
 
-	public function commitAndWaitForReplication( $fname, $ticket, array $opts = [] ) {
+	final public function commitAndWaitForReplication( $fname, $ticket, array $opts = [] ) {
 		if ( $ticket !== $this->ticket ) {
-			$this->perfLogger->error( __METHOD__ . ": $fname does not have outer scope.\n" .
-				( new RuntimeException() )->getTraceAsString() );
+			$this->perfLogger->error(
+				__METHOD__ . ": $fname does not have outer scope",
+				[ 'trace' => ( new RuntimeException() )->getTraceAsString() ]
+			);
 
-			return;
+			return false;
 		}
 
 		// The transaction owner and any caller with the empty transaction ticket can commit
 		// so that getEmptyTransactionTicket() callers don't risk seeing DBTransactionError.
 		if ( $this->trxRoundId !== false && $fname !== $this->trxRoundId ) {
-			$this->queryLogger->info( "$fname: committing on behalf of {$this->trxRoundId}." );
+			$this->queryLogger->info( "$fname: committing on behalf of {$this->trxRoundId}" );
 			$fnameEffective = $this->trxRoundId;
 		} else {
 			$fnameEffective = $fname;
 		}
 
 		$this->commitMasterChanges( $fnameEffective );
-		$this->waitForReplication( $opts );
+		$waitSucceeded = $this->waitForReplication( $opts );
 		// If a nested caller committed on behalf of $fname, start another empty $fname
 		// transaction, leaving the caller with the same empty transaction state as before.
 		if ( $fnameEffective !== $fname ) {
 			$this->beginMasterChanges( $fnameEffective );
 		}
+
+		return $waitSucceeded;
 	}
 
-	public function getChronologyProtectorTouched( $dbName ) {
-		return $this->getChronologyProtector()->getTouched( $dbName );
+	public function getChronologyProtectorTouched( $domain = false ) {
+		return $this->getChronologyProtector()->getTouched( $this->getMainLB( $domain ) );
 	}
 
 	public function disableChronologyProtection() {
@@ -455,8 +557,10 @@ abstract class LBFactory implements ILBFactory {
 			[
 				'ip' => $this->requestInfo['IPAddress'],
 				'agent' => $this->requestInfo['UserAgent'],
+				'clientId' => $this->requestInfo['ChronologyClientId'] ?: null
 			],
-			$this->requestInfo['ChronologyPositionIndex']
+			$this->requestInfo['ChronologyPositionIndex'],
+			$this->secret
 		);
 		$this->chronProt->setLogger( $this->replLogger );
 
@@ -469,11 +573,13 @@ abstract class LBFactory implements ILBFactory {
 		} elseif ( $this->memStash instanceof EmptyBagOStuff ) {
 			// No where to store any DB positions and wait for them to appear
 			$this->chronProt->setEnabled( false );
-			$this->replLogger->info( 'Cannot use ChronologyProtector with EmptyBagOStuff.' );
+			$this->replLogger->info( 'Cannot use ChronologyProtector with EmptyBagOStuff' );
 		}
 
-		$this->replLogger->debug( __METHOD__ . ': using request info ' .
-			json_encode( $this->requestInfo, JSON_PRETTY_PRINT ) );
+		$this->replLogger->debug(
+			__METHOD__ . ': request info ' .
+			json_encode( $this->requestInfo, JSON_PRETTY_PRINT )
+		);
 
 		return $this->chronProt;
 	}
@@ -491,7 +597,7 @@ abstract class LBFactory implements ILBFactory {
 	) {
 		// Record all the master positions needed
 		$this->forEachLB( function ( ILoadBalancer $lb ) use ( $cp ) {
-			$cp->shutdownLB( $lb );
+			$cp->storeSessionReplicationPosition( $lb );
 		} );
 		// Write them to the persistent stash. Try to do something useful by running $work
 		// while ChronologyProtector waits for the stash write to replicate to all DCs.
@@ -513,10 +619,20 @@ abstract class LBFactory implements ILBFactory {
 	}
 
 	/**
-	 * Base parameters to LoadBalancer::__construct()
+	 * Get parameters to ILoadBalancer::__construct()
+	 *
+	 * @param int|null $owner Use getOwnershipId() if this is for getMainLB()/getExternalLB()
 	 * @return array
 	 */
-	final protected function baseLoadBalancerParams() {
+	final protected function baseLoadBalancerParams( $owner ) {
+		if ( $this->trxRoundStage === self::ROUND_COMMIT_CALLBACKS ) {
+			$initStage = ILoadBalancer::STAGE_POSTCOMMIT_CALLBACKS;
+		} elseif ( $this->trxRoundStage === self::ROUND_ROLLBACK_CALLBACKS ) {
+			$initStage = ILoadBalancer::STAGE_POSTROLLBACK_CALLBACKS;
+		} else {
+			$initStage = null;
+		}
+
 		return [
 			'localDomain' => $this->localDomain,
 			'readOnlyReason' => $this->readOnlyReason,
@@ -532,11 +648,15 @@ abstract class LBFactory implements ILBFactory {
 			'hostname' => $this->hostname,
 			'cliMode' => $this->cliMode,
 			'agent' => $this->agent,
+			'maxLag' => $this->maxLag,
+			'defaultGroup' => $this->defaultGroup,
 			'chronologyCallback' => function ( ILoadBalancer $lb ) {
 				// Defer ChronologyProtector construction in case setRequestInfo() ends up
 				// being called later (but before the first connection attempt) (T192611)
-				$this->getChronologyProtector()->initLB( $lb );
-			}
+				$this->getChronologyProtector()->applySessionReplicationPosition( $lb );
+			},
+			'roundStage' => $initStage,
+			'ownerId' => $owner
 		];
 	}
 
@@ -545,11 +665,12 @@ abstract class LBFactory implements ILBFactory {
 	 */
 	protected function initLoadBalancer( ILoadBalancer $lb ) {
 		if ( $this->trxRoundId !== false ) {
-			$lb->beginMasterChanges( $this->trxRoundId ); // set DBO_TRX
+			$lb->beginMasterChanges( $this->trxRoundId, $this->id ); // set DBO_TRX
 		}
 
 		$lb->setTableAliases( $this->tableAliases );
 		$lb->setIndexAliases( $this->indexAliases );
+		$lb->setDomainAliases( $this->domainAliases );
 	}
 
 	public function setTableAliases( array $aliases ) {
@@ -560,20 +681,41 @@ abstract class LBFactory implements ILBFactory {
 		$this->indexAliases = $aliases;
 	}
 
-	public function setDomainPrefix( $prefix ) {
+	public function setDomainAliases( array $aliases ) {
+		$this->domainAliases = $aliases;
+	}
+
+	public function getTransactionProfiler(): TransactionProfiler {
+		return $this->trxProfiler;
+	}
+
+	public function setLocalDomainPrefix( $prefix ) {
 		$this->localDomain = new DatabaseDomain(
 			$this->localDomain->getDatabase(),
-			null,
+			$this->localDomain->getSchema(),
 			$prefix
 		);
 
 		$this->forEachLB( function ( ILoadBalancer $lb ) use ( $prefix ) {
-			$lb->setDomainPrefix( $prefix );
+			$lb->setLocalDomainPrefix( $prefix );
+		} );
+	}
+
+	public function redefineLocalDomain( $domain ) {
+		$this->closeAll();
+
+		$this->localDomain = DatabaseDomain::newFromId( $domain );
+
+		$this->forEachLB( function ( ILoadBalancer $lb ) {
+			$lb->redefineLocalDomain( $this->localDomain );
 		} );
 	}
 
 	public function closeAll() {
-		$this->forEachLBCallMethod( 'closeAll', [] );
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = ScopedCallback::newScopedIgnoreUserAbort();
+
+		$this->forEachLBCallMethod( 'closeAll', [ __METHOD__, $this->id ] );
 	}
 
 	public function setAgentName( $agent ) {
@@ -583,7 +725,7 @@ abstract class LBFactory implements ILBFactory {
 	public function appendShutdownCPIndexAsQuery( $url, $index ) {
 		$usedCluster = 0;
 		$this->forEachLB( function ( ILoadBalancer $lb ) use ( &$usedCluster ) {
-			$usedCluster |= ( $lb->getServerCount() > 1 );
+			$usedCluster |= $lb->hasStreamingReplicaServers();
 		} );
 
 		if ( !$usedCluster ) {
@@ -593,34 +735,82 @@ abstract class LBFactory implements ILBFactory {
 		return strpos( $url, '?' ) === false ? "$url?cpPosIndex=$index" : "$url&cpPosIndex=$index";
 	}
 
+	public function getChronologyProtectorClientId() {
+		return $this->getChronologyProtector()->getClientId();
+	}
+
+	/**
+	 * @param int $index Write index
+	 * @param int $time UNIX timestamp; can be used to detect stale cookies (T190082)
+	 * @param string $clientId Agent ID hash from ILBFactory::shutdown()
+	 * @return string Timestamp-qualified write index of the form "<index>@<timestamp>#<hash>"
+	 * @since 1.32
+	 */
+	public static function makeCookieValueFromCPIndex( $index, $time, $clientId ) {
+		return "$index@$time#$clientId";
+	}
+
+	/**
+	 * @param string $value Possible result of LBFactory::makeCookieValueFromCPIndex()
+	 * @param int $minTimestamp Lowest UNIX timestamp that a non-expired value can have
+	 * @return array (index: int or null, clientId: string or null)
+	 * @since 1.32
+	 */
+	public static function getCPInfoFromCookieValue( $value, $minTimestamp ) {
+		static $placeholder = [ 'index' => null, 'clientId' => null ];
+
+		if ( !preg_match( '/^(\d+)@(\d+)#([0-9a-f]{32})$/', $value, $m ) ) {
+			return $placeholder; // invalid
+		}
+
+		$index = (int)$m[1];
+		if ( $index <= 0 ) {
+			return $placeholder; // invalid
+		} elseif ( isset( $m[2] ) && $m[2] !== '' && (int)$m[2] < $minTimestamp ) {
+			return $placeholder; // expired
+		}
+
+		$clientId = ( isset( $m[3] ) && $m[3] !== '' ) ? $m[3] : null;
+
+		return [ 'index' => $index, 'clientId' => $clientId ];
+	}
+
 	public function setRequestInfo( array $info ) {
 		if ( $this->chronProt ) {
-			throw new LogicException( 'ChronologyProtector already initialized.' );
+			throw new LogicException( 'ChronologyProtector already initialized' );
 		}
 
 		$this->requestInfo = $info + $this->requestInfo;
 	}
 
-	/**
-	 * Make PHP ignore user aborts/disconnects until the returned
-	 * value leaves scope. This returns null and does nothing in CLI mode.
-	 *
-	 * @return ScopedCallback|null
-	 */
-	final protected function getScopedPHPBehaviorForCommit() {
-		if ( PHP_SAPI != 'cli' ) { // https://bugs.php.net/bug.php?id=47540
-			$old = ignore_user_abort( true ); // avoid half-finished operations
-			return new ScopedCallback( function () use ( $old ) {
-				ignore_user_abort( $old );
-			} );
-		}
+	public function setDefaultReplicationWaitTimeout( $seconds ) {
+		$old = $this->replicationWaitTimeout;
+		$this->replicationWaitTimeout = max( 1, (int)$seconds );
 
-		return null;
+		return $old;
 	}
 
-	function __destruct() {
+	/**
+	 * @return int Internal instance ID used to assert ownership of ILoadBalancer instances
+	 * @since 1.34
+	 */
+	final protected function getOwnershipId() {
+		return $this->id;
+	}
+
+	/**
+	 * @param string $stage
+	 */
+	private function assertTransactionRoundStage( $stage ) {
+		if ( $this->trxRoundStage !== $stage ) {
+			throw new DBTransactionError(
+				null,
+				"Transaction round stage must be '$stage' (not '{$this->trxRoundStage}')"
+			);
+		}
+	}
+
+	public function __destruct() {
 		$this->destroy();
 	}
 }
-
-class_alias( LBFactory::class, 'LBFactory' );
