@@ -21,79 +21,72 @@
  * @ingroup Parser
  */
 
+use MediaWiki\HookContainer\HookContainer;
+use MediaWiki\HookContainer\HookRunner;
+use MediaWiki\MediaWikiServices;
+
 /**
+ * @internal for using in Parser only.
+ *
  * @ingroup Parser
  */
 class LinkHolderArray {
+	/** @var array[][] */
 	public $internals = [];
+	/** @var array[] */
 	public $interwikis = [];
+	/** @var int */
 	public $size = 0;
 
 	/**
 	 * @var Parser
 	 */
 	public $parent;
-	protected $tempIdOffset;
+
+	/**
+	 * Current language converter
+	 * @var ILanguageConverter
+	 */
+	private $languageConverter;
+
+	/**
+	 * @var HookRunner
+	 */
+	private $hookRunner;
 
 	/**
 	 * @param Parser $parent
+	 * @param ILanguageConverter|null $languageConverter
+	 * @param HookContainer|null $hookContainer
 	 */
-	public function __construct( $parent ) {
+	public function __construct( Parser $parent, ILanguageConverter $languageConverter = null,
+		HookContainer $hookContainer = null
+	) {
 		$this->parent = $parent;
+
+		if ( !$languageConverter ) {
+			wfDeprecated( __METHOD__ . ' without $languageConverter parameter', '1.35' );
+			$languageConverter = MediaWikiServices::getInstance()
+				->getLanguageConverterFactory()
+				->getLanguageConverter( $parent->getTargetLanguage() );
+		}
+		$this->languageConverter = $languageConverter;
+		if ( !$hookContainer ) {
+			$hookContainer = MediaWikiServices::getInstance()->getHookContainer();
+		}
+		$this->hookRunner = new HookRunner( $hookContainer );
 	}
 
 	/**
 	 * Reduce memory usage to reduce the impact of circular references
 	 */
 	public function __destruct() {
-		foreach ( $this as $name => $value ) {
+		// @phan-suppress-next-line PhanTypeSuspiciousNonTraversableForeach
+		foreach ( $this as $name => $_ ) {
 			unset( $this->$name );
 		}
 	}
 
-	/**
-	 * Don't serialize the parent object, it is big, and not needed when it is
-	 * a parameter to mergeForeign(), which is the only application of
-	 * serializing at present.
-	 *
-	 * Compact the titles, only serialize the text form.
-	 * @return array
-	 */
-	public function __sleep() {
-		foreach ( $this->internals as &$nsLinks ) {
-			foreach ( $nsLinks as &$entry ) {
-				unset( $entry['title'] );
-			}
-		}
-		unset( $nsLinks );
-		unset( $entry );
-
-		foreach ( $this->interwikis as &$entry ) {
-			unset( $entry['title'] );
-		}
-		unset( $entry );
-
-		return [ 'internals', 'interwikis', 'size' ];
-	}
-
-	/**
-	 * Recreate the Title objects
-	 */
-	public function __wakeup() {
-		foreach ( $this->internals as &$nsLinks ) {
-			foreach ( $nsLinks as &$entry ) {
-				$entry['title'] = Title::newFromText( $entry['pdbk'] );
-			}
-		}
-		unset( $nsLinks );
-		unset( $entry );
-
-		foreach ( $this->interwikis as &$entry ) {
-			$entry['title'] = Title::newFromText( $entry['pdbk'] );
-		}
-		unset( $entry );
-	}
-
 	/**
 	 * Merge another LinkHolderArray into this one
 	 * @param LinkHolderArray $other
@@ -110,92 +103,6 @@ class LinkHolderArray {
 		$this->interwikis += $other->interwikis;
 	}
 
-	/**
-	 * Merge a LinkHolderArray from another parser instance into this one. The
-	 * keys will not be preserved. Any text which went with the old
-	 * LinkHolderArray and needs to work with the new one should be passed in
-	 * the $texts array. The strings in this array will have their link holders
-	 * converted for use in the destination link holder. The resulting array of
-	 * strings will be returned.
-	 *
-	 * @param LinkHolderArray $other
-	 * @param array $texts Array of strings
-	 * @return array
-	 */
-	public function mergeForeign( $other, $texts ) {
-		$this->tempIdOffset = $idOffset = $this->parent->nextLinkID();
-		$maxId = 0;
-
-		# Renumber internal links
-		foreach ( $other->internals as $ns => $nsLinks ) {
-			foreach ( $nsLinks as $key => $entry ) {
-				$newKey = $idOffset + $key;
-				$this->internals[$ns][$newKey] = $entry;
-				$maxId = $newKey > $maxId ? $newKey : $maxId;
-			}
-		}
-		$texts = preg_replace_callback( '/(<!--LINK \d+:)(\d+)(-->)/',
-			[ $this, 'mergeForeignCallback' ], $texts );
-
-		# Renumber interwiki links
-		foreach ( $other->interwikis as $key => $entry ) {
-			$newKey = $idOffset + $key;
-			$this->interwikis[$newKey] = $entry;
-			$maxId = $newKey > $maxId ? $newKey : $maxId;
-		}
-		$texts = preg_replace_callback( '/(<!--IWLINK )(\d+)(-->)/',
-			[ $this, 'mergeForeignCallback' ], $texts );
-
-		# Set the parent link ID to be beyond the highest used ID
-		$this->parent->setLinkID( $maxId + 1 );
-		$this->tempIdOffset = null;
-		return $texts;
-	}
-
-	/**
-	 * @param array $m
-	 * @return string
-	 */
-	protected function mergeForeignCallback( $m ) {
-		return $m[1] . ( $m[2] + $this->tempIdOffset ) . $m[3];
-	}
-
-	/**
-	 * Get a subset of the current LinkHolderArray which is sufficient to
-	 * interpret the given text.
-	 * @param string $text
-	 * @return LinkHolderArray
-	 */
-	public function getSubArray( $text ) {
-		$sub = new LinkHolderArray( $this->parent );
-
-		# Internal links
-		$pos = 0;
-		while ( $pos < strlen( $text ) ) {
-			if ( !preg_match( '/<!--LINK (\d+):(\d+)-->/',
-				$text, $m, PREG_OFFSET_CAPTURE, $pos )
-			) {
-				break;
-			}
-			$ns = $m[1][0];
-			$key = $m[2][0];
-			$sub->internals[$ns][$key] = $this->internals[$ns][$key];
-			$pos = $m[0][1] + strlen( $m[0][0] );
-		}
-
-		# Interwiki links
-		$pos = 0;
-		while ( $pos < strlen( $text ) ) {
-			if ( !preg_match( '/<!--IWLINK (\d+)-->/', $text, $m, PREG_OFFSET_CAPTURE, $pos ) ) {
-				break;
-			}
-			$key = $m[1][0];
-			$sub->interwikis[$key] = $this->interwikis[$key];
-			$pos = $m[0][1] + strlen( $m[0][0] );
-		}
-		return $sub;
-	}
-
 	/**
 	 * Returns true if the memory requirements of this object are getting large
 	 * @return bool
@@ -223,73 +130,59 @@ class LinkHolderArray {
 	 *
 	 * @param Title $nt
 	 * @param string $text
-	 * @param array $query [optional]
 	 * @param string $trail [optional]
 	 * @param string $prefix [optional]
 	 * @return string
 	 */
-	public function makeHolder( $nt, $text = '', $query = [], $trail = '', $prefix = '' ) {
-		if ( !is_object( $nt ) ) {
-			# Fail gracefully
-			$retVal = "<!-- ERROR -->{$prefix}{$text}{$trail}";
+	public function makeHolder( Title $nt, $text = '', $trail = '', $prefix = '' ) {
+		# Separate the link trail from the rest of the link
+		list( $inside, $trail ) = Linker::splitTrail( $trail );
+
+		$key = $this->parent->nextLinkID();
+		$entry = [
+			'title' => $nt,
+			'text' => $prefix . $text . $inside,
+			'pdbk' => $nt->getPrefixedDBkey(),
+		];
+
+		$this->size++;
+		if ( $nt->isExternal() ) {
+			// Use a globally unique ID to keep the objects mergable
+			$this->interwikis[$key] = $entry;
+			return "<!--IWLINK'\" $key-->{$trail}";
 		} else {
-			# Separate the link trail from the rest of the link
-			list( $inside, $trail ) = Linker::splitTrail( $trail );
-
-			$entry = [
-				'title' => $nt,
-				'text' => $prefix . $text . $inside,
-				'pdbk' => $nt->getPrefixedDBkey(),
-			];
-			if ( $query !== [] ) {
-				$entry['query'] = $query;
-			}
-
-			if ( $nt->isExternal() ) {
-				// Use a globally unique ID to keep the objects mergable
-				$key = $this->parent->nextLinkID();
-				$this->interwikis[$key] = $entry;
-				$retVal = "<!--IWLINK $key-->{$trail}";
-			} else {
-				$key = $this->parent->nextLinkID();
-				$ns = $nt->getNamespace();
-				$this->internals[$ns][$key] = $entry;
-				$retVal = "<!--LINK $ns:$key-->{$trail}";
-			}
-			$this->size++;
+			$ns = $nt->getNamespace();
+			$this->internals[$ns][$key] = $entry;
+			return "<!--LINK'\" $ns:$key-->{$trail}";
 		}
-		return $retVal;
 	}
 
 	/**
 	 * Replace <!--LINK--> link placeholders with actual links, in the buffer
 	 *
-	 * @param string $text
+	 * @param string &$text
 	 */
 	public function replace( &$text ) {
-
 		$this->replaceInternal( $text );
 		$this->replaceInterwiki( $text );
-
 	}
 
 	/**
 	 * Replace internal links
-	 * @param string $text
+	 * @suppress SecurityCheck-XSS Gets confused with $entry['pdbk']
+	 * @param string &$text
 	 */
 	protected function replaceInternal( &$text ) {
 		if ( !$this->internals ) {
 			return;
 		}
 
-		global $wgContLang, $wgContentHandlerUseDB, $wgPageLanguageUseDB;
-
 		$colours = [];
-		$linkCache = LinkCache::singleton();
+		$linkCache = MediaWikiServices::getInstance()->getLinkCache();
 		$output = $this->parent->getOutput();
+		$linkRenderer = $this->parent->getLinkRenderer();
 
-		$dbr = wfGetDB( DB_SLAVE );
-		$threshold = $this->parent->getOptions()->getStubThreshold();
+		$dbr = wfGetDB( DB_REPLICA );
 
 		# Sort by namespace
 		ksort( $this->internals );
@@ -297,16 +190,16 @@ class LinkHolderArray {
 		$linkcolour_ids = [];
 
 		# Generate query
-		$queries = [];
+		$lb = new LinkBatch();
+		$lb->setCaller( __METHOD__ );
+
 		foreach ( $this->internals as $ns => $entries ) {
-			foreach ( $entries as $entry ) {
+			foreach ( $entries as [ 'title' => $title, 'pdbk' => $pdbk ] ) {
 				/** @var Title $title */
-				$title = $entry['title'];
-				$pdbk = $entry['pdbk'];
 
 				# Skip invalid entries.
 				# Result will be ugly, but prevents crash.
-				if ( is_null( $title ) ) {
+				if ( $title === null ) {
 					continue;
 				}
 
@@ -318,44 +211,28 @@ class LinkHolderArray {
 				} else {
 					$id = $linkCache->getGoodLinkID( $pdbk );
 					if ( $id != 0 ) {
-						$colours[$pdbk] = Linker::getLinkColour( $title, $threshold );
+						$colours[$pdbk] = $linkRenderer->getLinkClasses( $title );
 						$output->addLink( $title, $id );
 						$linkcolour_ids[$id] = $pdbk;
 					} elseif ( $linkCache->isBadLink( $pdbk ) ) {
 						$colours[$pdbk] = 'new';
 					} else {
 						# Not in the link cache, add it to the query
-						$queries[$ns][] = $title->getDBkey();
+						$lb->addObj( $title );
 					}
 				}
 			}
 		}
-		if ( $queries ) {
-			$where = [];
-			foreach ( $queries as $ns => $pages ) {
-				$where[] = $dbr->makeList(
-					[
-						'page_namespace' => $ns,
-						'page_title' => array_unique( $pages ),
-					],
-					LIST_AND
-				);
-			}
-
-			$fields = [ 'page_id', 'page_namespace', 'page_title',
-				'page_is_redirect', 'page_len', 'page_latest' ];
-
-			if ( $wgContentHandlerUseDB ) {
-				$fields[] = 'page_content_model';
-			}
-			if ( $wgPageLanguageUseDB ) {
-				$fields[] = 'page_lang';
-			}
+		if ( !$lb->isEmpty() ) {
+			$fields = array_merge(
+				LinkCache::getSelectFields(),
+				[ 'page_namespace', 'page_title' ]
+			);
 
 			$res = $dbr->select(
 				'page',
 				$fields,
-				$dbr->makeList( $where, LIST_OR ),
+				$lb->constructSet( 'page', $dbr ),
 				__METHOD__
 			);
 
@@ -366,22 +243,19 @@ class LinkHolderArray {
 				$pdbk = $title->getPrefixedDBkey();
 				$linkCache->addGoodLinkObjFromRow( $title, $s );
 				$output->addLink( $title, $s->page_id );
-				# @todo FIXME: Convoluted data flow
-				# The redirect status and length is passed to getLinkColour via the LinkCache
-				# Use formal parameters instead
-				$colours[$pdbk] = Linker::getLinkColour( $title, $threshold );
+				$colours[$pdbk] = $linkRenderer->getLinkClasses( $title );
 				// add id to the extension todolist
 				$linkcolour_ids[$s->page_id] = $pdbk;
 			}
 			unset( $res );
 		}
-		if ( count( $linkcolour_ids ) ) {
+		if ( $linkcolour_ids !== [] ) {
 			// pass an array of page_ids to an extension
-			Hooks::run( 'GetLinkColours', [ $linkcolour_ids, &$colours ] );
+			$this->hookRunner->onGetLinkColours( $linkcolour_ids, $colours, $this->parent->getTitle() );
 		}
 
 		# Do a second query for different language variants of links and categories
-		if ( $wgContLang->hasVariants() ) {
+		if ( $this->languageConverter->hasVariants() ) {
 			$this->doVariants( $colours );
 		}
 
@@ -391,49 +265,51 @@ class LinkHolderArray {
 			foreach ( $entries as $index => $entry ) {
 				$pdbk = $entry['pdbk'];
 				$title = $entry['title'];
-				$query = isset( $entry['query'] ) ? $entry['query'] : [];
-				$key = "$ns:$index";
-				$searchkey = "<!--LINK $key-->";
-				$displayText = $entry['text'];
+				$query = $entry['query'] ?? [];
+				$searchkey = "<!--LINK'\" $ns:$index-->";
+				$displayTextHtml = $entry['text'];
 				if ( isset( $entry['selflink'] ) ) {
-					$replacePairs[$searchkey] = Linker::makeSelfLinkObj( $title, $displayText, $query );
+					$replacePairs[$searchkey] = Linker::makeSelfLinkObj( $title, $displayTextHtml, $query );
 					continue;
 				}
-				if ( $displayText === '' ) {
+				if ( $displayTextHtml === '' ) {
 					$displayText = null;
+				} else {
+					$displayText = new HtmlArmor( $displayTextHtml );
 				}
 				if ( !isset( $colours[$pdbk] ) ) {
 					$colours[$pdbk] = 'new';
 				}
-				$attribs = [];
 				if ( $colours[$pdbk] == 'new' ) {
 					$linkCache->addBadLinkObj( $title );
 					$output->addLink( $title, 0 );
-					$type = [ 'broken' ];
+					$link = $linkRenderer->makeBrokenLink(
+						$title, $displayText, [], $query
+					);
 				} else {
-					if ( $colours[$pdbk] != '' ) {
-						$attribs['class'] = $colours[$pdbk];
-					}
-					$type = [ 'known', 'noclasses' ];
+					$link = $linkRenderer->makePreloadedLink(
+						$title, $displayText, $colours[$pdbk], [], $query
+					);
 				}
-				$replacePairs[$searchkey] = Linker::link( $title, $displayText,
-						$attribs, $query, $type );
+
+				$replacePairs[$searchkey] = $link;
 			}
 		}
-		$replacer = new HashtableReplacer( $replacePairs, 1 );
 
 		# Do the thing
 		$text = preg_replace_callback(
-			'/(<!--LINK .*?-->)/',
-			$replacer->cb(),
+			'/(<!--LINK\'" .*?-->)/',
+			function ( array $matches ) use ( $replacePairs ) {
+				return $replacePairs[$matches[1]];
+			},
 			$text
 		);
-
 	}
 
 	/**
 	 * Replace interwiki links
-	 * @param string $text
+	 * @param string &$text
+	 * @suppress SecurityCheck-XSS Gets confused with $this->interwikis['pdbk']
 	 */
 	protected function replaceInterwiki( &$text ) {
 		if ( empty( $this->interwikis ) ) {
@@ -443,32 +319,33 @@ class LinkHolderArray {
 		# Make interwiki link HTML
 		$output = $this->parent->getOutput();
 		$replacePairs = [];
-		$options = [
-			'stubThreshold' => $this->parent->getOptions()->getStubThreshold(),
-		];
+		$linkRenderer = $this->parent->getLinkRenderer();
 		foreach ( $this->interwikis as $key => $link ) {
-			$replacePairs[$key] = Linker::link( $link['title'], $link['text'], [], [], $options );
+			$replacePairs[$key] = $linkRenderer->makeLink(
+				$link['title'],
+				new HtmlArmor( $link['text'] )
+			);
 			$output->addInterwikiLink( $link['title'] );
 		}
-		$replacer = new HashtableReplacer( $replacePairs, 1 );
 
 		$text = preg_replace_callback(
-			'/<!--IWLINK (.*?)-->/',
-			$replacer->cb(),
-			$text );
+			'/<!--IWLINK\'" (.*?)-->/',
+			function ( array $matches ) use ( $replacePairs ) {
+				return $replacePairs[$matches[1]];
+			},
+			$text
+		);
 	}
 
 	/**
 	 * Modify $this->internals and $colours according to language variant linking rules
-	 * @param array $colours
+	 * @param array &$colours
 	 */
 	protected function doVariants( &$colours ) {
-		global $wgContLang, $wgContentHandlerUseDB, $wgPageLanguageUseDB;
 		$linkBatch = new LinkBatch();
 		$variantMap = []; // maps $pdbkey_Variant => $keys (of link holders)
 		$output = $this->parent->getOutput();
-		$linkCache = LinkCache::singleton();
-		$threshold = $this->parent->getOptions()->getStubThreshold();
+		$linkCache = MediaWikiServices::getInstance()->getLinkCache();
 		$titlesToBeConverted = '';
 		$titlesAttrs = [];
 
@@ -479,43 +356,38 @@ class LinkHolderArray {
 			if ( $ns == NS_SPECIAL ) {
 				continue;
 			}
-			foreach ( $entries as $index => $entry ) {
-				$pdbk = $entry['pdbk'];
+			foreach ( $entries as $index => [ 'title' => $title, 'pdbk' => $pdbk ] ) {
 				// we only deal with new links (in its first query)
 				if ( !isset( $colours[$pdbk] ) || $colours[$pdbk] === 'new' ) {
-					$titlesAttrs[] = [ $index, $entry['title'] ];
+					$titlesAttrs[] = [ $index, $title ];
 					// separate titles with \0 because it would never appears
 					// in a valid title
-					$titlesToBeConverted .= $entry['title']->getText() . "\0";
+					$titlesToBeConverted .= $title->getText() . "\0";
 				}
 			}
 		}
 
 		// Now do the conversion and explode string to text of titles
-		$titlesAllVariants = $wgContLang->autoConvertToAllVariants( rtrim( $titlesToBeConverted, "\0" ) );
-		$allVariantsName = array_keys( $titlesAllVariants );
+		$titlesAllVariants = $this->languageConverter->
+			autoConvertToAllVariants( rtrim( $titlesToBeConverted, "\0" ) );
 		foreach ( $titlesAllVariants as &$titlesVariant ) {
 			$titlesVariant = explode( "\0", $titlesVariant );
 		}
 
 		// Then add variants of links to link batch
 		$parentTitle = $this->parent->getTitle();
-		foreach ( $titlesAttrs as $i => $attrs ) {
+		foreach ( $titlesAttrs as $i => [ $index, $title ] ) {
 			/** @var Title $title */
-			list( $index, $title ) = $attrs;
 			$ns = $title->getNamespace();
 			$text = $title->getText();
 
-			foreach ( $allVariantsName as $variantName ) {
-				$textVariant = $titlesAllVariants[$variantName][$i];
+			foreach ( $titlesAllVariants as $variantName => $textVariants ) {
+				$textVariant = $textVariants[$i];
 				if ( $textVariant === $text ) {
 					continue;
 				}
 
 				$variantTitle = Title::makeTitle( $ns, $textVariant );
-				if ( is_null( $variantTitle ) ) {
-					continue;
-				}
 
 				// Self-link checking for mixed/different variant titles. At this point, we
 				// already know the exact title does not exist, so the link cannot be to a
@@ -536,11 +408,11 @@ class LinkHolderArray {
 		foreach ( $output->getCategoryLinks() as $category ) {
 			$categoryTitle = Title::makeTitleSafe( NS_CATEGORY, $category );
 			$linkBatch->addObj( $categoryTitle );
-			$variants = $wgContLang->autoConvertToAllVariants( $category );
+			$variants = $this->languageConverter->autoConvertToAllVariants( $category );
 			foreach ( $variants as $variant ) {
 				if ( $variant !== $category ) {
 					$variantTitle = Title::makeTitleSafe( NS_CATEGORY, $variant );
-					if ( is_null( $variantTitle ) ) {
+					if ( $variantTitle === null ) {
 						continue;
 					}
 					$linkBatch->addObj( $variantTitle );
@@ -551,16 +423,11 @@ class LinkHolderArray {
 
 		if ( !$linkBatch->isEmpty() ) {
 			// construct query
-			$dbr = wfGetDB( DB_SLAVE );
-			$fields = [ 'page_id', 'page_namespace', 'page_title',
-				'page_is_redirect', 'page_len', 'page_latest' ];
-
-			if ( $wgContentHandlerUseDB ) {
-				$fields[] = 'page_content_model';
-			}
-			if ( $wgPageLanguageUseDB ) {
-				$fields[] = 'page_lang';
-			}
+			$dbr = wfGetDB( DB_REPLICA );
+			$fields = array_merge(
+				LinkCache::getSelectFields(),
+				[ 'page_namespace', 'page_title' ]
+			);
 
 			$varRes = $dbr->select( 'page',
 				$fields,
@@ -569,6 +436,7 @@ class LinkHolderArray {
 			);
 
 			$linkcolour_ids = [];
+			$linkRenderer = $this->parent->getLinkRenderer();
 
 			// for each found variants, figure out link holders and replace
 			foreach ( $varRes as $s ) {
@@ -595,10 +463,7 @@ class LinkHolderArray {
 						$entry['pdbk'] = $varPdbk;
 
 						// set pdbk and colour
-						# @todo FIXME: Convoluted data flow
-						# The redirect status and length is passed to getLinkColour via the LinkCache
-						# Use formal parameters instead
-						$colours[$varPdbk] = Linker::getLinkColour( $variantTitle, $threshold );
+						$colours[$varPdbk] = $linkRenderer->getLinkClasses( $variantTitle );
 						$linkcolour_ids[$s->page_id] = $pdbk;
 					}
 				}
@@ -611,10 +476,10 @@ class LinkHolderArray {
 					}
 				}
 			}
-			Hooks::run( 'GetLinkColours', [ $linkcolour_ids, &$colours ] );
+			$this->hookRunner->onGetLinkColours( $linkcolour_ids, $colours, $this->parent->getTitle() );
 
 			// rebuild the categories in original order (if there are replacements)
-			if ( count( $varCategories ) > 0 ) {
+			if ( $varCategories !== [] ) {
 				$newCats = [];
 				$originalCats = $output->getCategories();
 				foreach ( $originalCats as $cat => $sortkey ) {
@@ -631,42 +496,26 @@ class LinkHolderArray {
 	}
 
 	/**
-	 * Replace <!--LINK--> link placeholders with plain text of links
+	 * Replace <!--LINK--> and <!--IWLINK--> link placeholders with plain text of links
 	 * (not HTML-formatted).
 	 *
 	 * @param string $text
 	 * @return string
 	 */
 	public function replaceText( $text ) {
+		return preg_replace_callback(
+			'/<!--(IW)?LINK\'" (.*?)-->/',
+			function ( $matches ) {
+				list( $unchanged, $isInterwiki, $key ) = $matches;
 
-		$text = preg_replace_callback(
-			'/<!--(LINK|IWLINK) (.*?)-->/',
-			[ $this, 'replaceTextCallback' ],
-			$text );
-
-		return $text;
-	}
-
-	/**
-	 * Callback for replaceText()
-	 *
-	 * @param array $matches
-	 * @return string
-	 * @private
-	 */
-	public function replaceTextCallback( $matches ) {
-		$type = $matches[1];
-		$key = $matches[2];
-		if ( $type == 'LINK' ) {
-			list( $ns, $index ) = explode( ':', $key, 2 );
-			if ( isset( $this->internals[$ns][$index]['text'] ) ) {
-				return $this->internals[$ns][$index]['text'];
-			}
-		} elseif ( $type == 'IWLINK' ) {
-			if ( isset( $this->interwikis[$key]['text'] ) ) {
-				return $this->interwikis[$key]['text'];
-			}
-		}
-		return $matches[0];
+				if ( !$isInterwiki ) {
+					list( $ns, $index ) = explode( ':', $key, 2 );
+					return $this->internals[$ns][$index]['text'] ?? $unchanged;
+				} else {
+					return $this->interwikis[$key]['text'] ?? $unchanged;
+				}
+			},
+			$text
+		);
 	}
 }
